#ifndef IATK_CONTELE_H
#define IATK_CONTELE_H
/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM Agent Building Environment                                             */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/library/c/contele.h, tk_library, abe.l6
// VERSION: 1.10 97/06/20 10:44:22
//
// INCLUDE AS:  #include <iatk/contele.h>
//
// PURPOSE:  Define the IALibInferenceContentsElement object.
//
// DESCRIPTION:
//    Each IALibInferenceContentsElement (ContentsElement for short)
//    represents an object that is grouped in persistent storage in a
//    particular (parent) Collector.  The set of ContentsElements for
//    a Collector are referred to as contents of the Collector.
//
// CLASSES DECLARED:
//    IALibInferenceContentsElement - Represents a single element of the
//                                    contents of an IALibInferenceCollector.
//
// NOTES:
//    IALibInferenceContentsElementImp is the related class used to implement
//    the IALibInferenceContentsElement class.
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ---   ------ --- ------ -----------
// $xn = yymmdd iii pnnnnn (abstract of feature/defect)
// $a1 = 960919 tlw F00296 Add support for LogRecordSet
// $a2 = 961029 tlw F00445 Support new IALibrary class; constructor and destructor
//                         are now private
// $a3 = 970519 jeg F00813 Support for Microsoft Visual C++ compiler.
// $a4 = 970620 jeg D00888 Rename getimpl() to getImplP()
//
//******************************************************************************

#include <iatk/types.h>
#include <iatk/libconst.h>
/*
#ifndef IATK_CETYPE
#define IATK_CETYPE
      enum IALibInferenceContentsElementType  // Contents Element types
              {IALibICET_collector='a', IALibICET_collectormetadata='b', IALibICET_ruleset='c',
               IALibICET_rulesetmetadata='d', IALibICET_ltfactset='e', IALibICET_ltfactsetmetadata='f',
               IALibICET_rulesmetadata='g', IALibICET_ltfactsmetadata='h', IALibICET_logset='i',        // $a1
               IALibICET_ceunknown='u'};                                                                // $a1

const char nometadataname = ' ';     // Blank metadataname means no metadata file
#endif
*/
class IALibrary;                                                                                        //$a3
class IALibInferenceContentsElementProv;                                                                //$a3
class IALibInferenceContentsElementImp;

class ostream;                                                                                          //$a3

class IAEXPCLASS IALibInferenceContentsElement {                                                        //$a3

  public:

     IALibInferenceContentsElement(const IALibInferenceContentsElement &from);  // Copy constructor

     IALibInferenceContentsElement & operator=(const IALibInferenceContentsElement &from);  // Assignment operator

    // Prototype for member functions
    const char* extractElementName() const;                     // Get the Element name
    IALibInferenceContentsElementType extractType() const;      // Get the Element type
    char extractMetadataName() const;                           // Get the Metadata name
    void debug_print(ostream& outfile) const;                   // Print a ContentsElement - debugging

    IALibInferenceContentsElementImp* getImpl() const;                                                  //$a3

  private:

    IALibInferenceContentsElement (IALibInferenceContentsElementProv* cimpl, IALibrary* _lib);          //$a3

    ~IALibInferenceContentsElement();     // Destructor                                                    $a2

    IALibInferenceContentsElementProv *impl; // Implementation-specific private object pointer          //$a3
    IALibrary *lib;                                                                                     //$a3

    // The following methods are only to be used by IALibInference classes to
    // access or set the pointer to a IALibInferenceContentsElementImp.
    void setImpl(IALibInferenceContentsElementImp* ceImpPtr);
    IALibInferenceContentsElementProv* getImplP() const;                                                //$a4

    friend class IALibInferenceContentsElementImp;
    friend class IALibInferenceCollector;
    friend class IALibrary;                                                                             // $a2
};

#endif
