#ifndef IATK_LIBRARY_H
#define IATK_LIBRARY_H
/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM Agent Building Environment                                             */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/library/c/library.h, tk_library, abe.l6
// VERSION: 1.11 97/06/19 18:11:13
//
// INCLUDE AS:  #include <iatk/library.h>
//
// PURPOSE:  Define the IALibrary object.
//
// DESCRIPTION:
//    Methods and attributes used to instantiate an IA Library to be used
//    by an Intelligent Agent or Rule Editor.
//
// CLASSES DECLARED:
//    IALibrary -
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ---   ------ --- ------ -----------
// $xn = yymmdd iii pnnnnn (abstract of feature/defect)
// $a1 = 961024 tlw F00445 New methods and attributes of IALibrary class
// $a2 = 961115 tlw F00445 Metadata added to IALibLogRecordSet
// $a3 = 961202 tlw F00495 Support for hiding Collectors
// $a4 = 970519 jeg F00813 Support for Microsoft Visual C++ compiler
//
//******************************************************************************

#include <iatk/types.h>
#include <iatk/libconst.h>                                                                               //$a1

class IALibInferenceCollector;                                                                          //$a4
class IALibInferenceContentsElement;                                                                    //$a4
class IALibInferenceLTFact;                                                                             //$a4
class IALibInferenceLTFactSet;                                                                          //$a4
class IALibInferenceRule;                                                                               //$a4
class IALibInferenceRuleSet;                                                                            //$a4
class IALibLogRecord;                                                                                   //$a4
class IALibLogRecordSet;                                                                                //$a4
class IALibraryProv;                                                                                    //$a4

class IAEXPCLASS IALibrary {                                                                            //$a4

  public:

    IALibrary();               // Constructor - no args                                                    $a1

    ~IALibrary();              // Destructor

    // Prototype for member functions
    const char* getMap() const;      // Get the map to top of Library                                      $a1
    const char* getType() const;     // Get the type of the Library implementation (e.g., file)            $a1
    const char* getDllName() const;  // Get the Library dll name                                           $a1
    const char* getLibName() const;  // Get the Library name                                               $a1
    IALibInferenceCollector* getTop() const; // Get pointer to the Library's top collector                 $a1

    void attachImplementation(                  // Attach a specific library implementation to this object $a1
                              const char* map,        // Map to top of library                             $a1
                              const char* type,       // Library type                                      $a1
                              const char* dllname,    // Name of dll to load                               $a1
                              const char* libname,    // Name of the library                               $a1
                              int numColLevels);      // Number of collector levels                        $a3

    void getConductSet(  // Load a conduct set (RuleSet and LTFactSet named the same)                      $a3
                  const char* csName,  // conduct set name = RuleSet and LTFactSet names                   $a3
                  const char mdName,   // metadata name (of RuleSet)                                       $a3
                  const char* qualifiedCollectorName,  // /x/y/z/  collector path under top collector      $a3
                  IALibInferenceRuleSet* * ruleSet,    // Return pointer to RuleSet                        $a3
                  IALibInferenceLTFactSet* * factSet); // Return pointer to LTFactSet                      $a3

    void getInferenceSet( // Load a RuleSet and LTFactSet for inferencing                                  $a3
                  const char* rsName,      // RuleSet name                                                 $a3
                  const char* ltfsName,    // LTFactSet name                                               $a3
                  const char rsmdName,     // RuleSet metadata name                                        $a3
                  const char ltfsmdName,   // LTFactSet metadata name                                      $a3
                  const char* qualifiedCollectorName,  // /x/y/z/  collector path under top collector      $a3
                  IALibInferenceRuleSet* * ruleSet,    // Return pointer to RuleSet                        $a3
                  IALibInferenceLTFactSet* * factSet); // Return pointer to LTFactSet                      $a3

    IALibInferenceCollector* new_IALibInferenceCollector(     // Factory for collector                     $a1
             IALibInferenceCollector* cparent,    // Pointer to parent collector                           $a1
             const char* cname);                  // Collector name                                        $a1

    IALibInferenceContentsElement* new_IALibInferenceContentsElement (     // Factory for contents element $a1
             const char* ename = "",              // Name, default is unnamed                              $a1
             IALibInferenceContentsElementType etype = IALibICET_ceunknown,      // type, default in unknown$a1
             char mname = nometadataname);        // metadata name, default is blank (meaning no name)     $a1

    IALibInferenceLTFact* new_IALibInferenceLTFact(                 // Factory for LTFact                  $a1
           const char* fname="",            // name, default = null string                                 $a1
           const char* fid="",              // id, default = null string                                   $a1
           const char* fkif="",             // kif, default = null string                                  $a1
           const char* fmetadata="",        // metadata, default = null string                             $a1
           const char* fstate=ACTIVEF,      // Active state, default = active                              $a1
           IABool fdelete=NOTDELETEDF);     // Delete state, default = not deleted                         $a1

    IALibInferenceLTFact* new_IALibInferenceElement(                 // Factory for LTFact                 $a1
           IALibInferenceLTFact* fptr,      // pointer to LTFact (only needed for signature)               $a1
           const char* fname="",            // name, default = null string                                 $a1
           const char* fid="",              // id, default = null string                                   $a1
           const char* fkif="",             // kif, default = null string                                  $a1
           const char* fmetadata="",        // metadata, default = null string                             $a1
           const char* fstate=ACTIVEF,      // Active state, default = active                              $a1
           IABool fdelete=NOTDELETEDF);     // Delete state, default = not deleted                         $a1

    IALibInferenceLTFactSet* new_IALibInferenceLTFactSet (                   // Factory for LTFactSet      $a1
             IALibInferenceCollector* lsparent,   // pointer to parent collector                           $a1
             const char* lsname = "default",   // name, default = "default"                                $a1
             const char* lsid = "",            // id, default = ""                                         $a1
             char factsmdname = ' ',           // LTFacts-level metadata set name, default = ' ' (none)    $a1
             const char* lsmd = "",            // LTFactSet metadata, default = ""                         $a1
             char lsmdname = ' ',              // LTFactSet metadata set name, default = ' ' (none)        $a1
             const char* lsstate = ACTIVEFS);  // state, default = Active                                  $a1

    IALibInferenceRule* new_IALibInferenceRule(                   // Factory for rule                      $a1
           const char* rname="",            // name, default = null string                                 $a1
           const char* rid="",              // id, default = null string                                   $a1
           const char* rkif="",             // kif, default = null string                                  $a1
           const char* rmetadata="",        // metadata, default = null string                             $a1
           const char* rstate=ACTIVER,      // Active state, default = active                              $a1
           IABool rdelete=NOTDELETEDR);     // Delete state, default = not deleted                         $a1

    IALibInferenceRule* new_IALibInferenceElement(                   // Factory for rule                   $a1
           IALibInferenceRule* rptr,        // Pointer to Rule (only needed for signature)                 $a1
           const char* rname="",            // name, default = null string                                 $a1
           const char* rid="",              // id, default = null string                                   $a1
           const char* rkif="",             // kif, default = null string                                  $a1
           const char* rmetadata="",        // metadata, default = null string                             $a1
           const char* rstate=ACTIVER,      // Active state, default = active                              $a1
           IABool rdelete=NOTDELETEDR);     // Delete state, default = not deleted                         $a1

    IALibInferenceRuleSet* new_IALibInferenceRuleSet (                // Factory for RuleSet               $a1
             IALibInferenceCollector* rsparent,   // pointer to parent collector                           $a1
             const char* rsname = "default",   // name, default = "default"                                $a1
             const char* rsid = "",            // id, default = ""                                         $a1
             char rulesmdname = ' ',            // Rules-level metadata set name, default = ' ' (none)     $a1
             const char* rsmd = "",            // RuleSet metadata, default = ""                           $a1
             char rsmdname = ' ',              // RuleSet metadata set name, default = ' ' (none)          $a1
             const char* rsstate = ACTIVERS);  // state, default = Active                                  $a1

    IALibLogRecord* new_IALibLogRecord (       // Factory for LogRecord                                    $a1
             unsigned int lrtype = IALibLRT_activity_normal,     // type, default is activity_normal       $a1
             unsigned int lrcategory = 0,    // category, default is 0                            $a1
             const char* lroriginator = "",  // originator, default is null string                         $a1
             const char* lrsid = "",         // source identifier, default is null string                  $a1
             unsigned long lrmid = 0,        // message identifier, default is 0                           $a1
             const char* lrtoken1 = "");     // first token, default is null string                        $a1

    IALibLogRecordSet* new_IALibLogRecordSet(                 // Factory for LogRecordSet                  $a1
             IALibInferenceCollector* cparent,    // Pointer to parent collector                           $a1
             const char* name,                    // LogRecordSet name,                                    $a1
                                                  //  normally is collector name                           $a1
             unsigned int cacheSize = 40,         // Number of LogRecords to buffer in memory before       $a1
                                                  // being written to persistent storage                   $a1
             unsigned int logSize = 800,          // Number of LogRecords in persistent storage            $a1
                                                  // before wrap-around                                    $a1
             unsigned int level = IALibLRT_activity_normal+IALibLRT_error,                               //$a2
                                                  // Types of LogRecords that will be read or written      $a1
             unsigned int metadataSize = 1,       // Size (in chars) of a metadata cell                    $a2
             char metadataName = ' ');            // Metadata name                                         $a2


    void delete_IALibInferenceCollector(IALibInferenceCollector *p);
                                                           // Destroyer of collector                       $a1

    void delete_IALibInferenceContentsElement(IALibInferenceContentsElement *p);
                                                                 // Destroyer of contents element          $a1

    void delete_IALibInferenceLTFact(IALibInferenceLTFact *p);
                                                        // Destroyer of LTFact                             $a1

    void delete_IALibInferenceElement(IALibInferenceLTFact *p);
                                                        // Destroyer of LTFact                             $a1

    void delete_IALibInferenceLTFactSet(IALibInferenceLTFactSet *p);
                                                           // Destroyer of LTFactSet                       $a1

    void delete_IALibInferenceRule(IALibInferenceRule *p);
                                                      // Destroyer of Rule                                 $a1

    void delete_IALibInferenceElement(IALibInferenceRule *p);
                                                      // Destroyer of Rule                                 $a1

    void delete_IALibInferenceRuleSet(IALibInferenceRuleSet *p);
                                                         // Destroyer of RuleSet                           $a1

    void delete_IALibLogRecord(IALibLogRecord *p);
                                                  // Destroyer of LogRecord                                $a1

    void delete_IALibLogRecordSet(IALibLogRecordSet *p);
                                                     // Destroyer of LogRecordSet                          $a1

  private:

    IALibInferenceCollector* new_IALibInferenceTopCollector(     // Factory for top collector              $a1
             const char* cmap,                    // Map of collector                                      $a1
             const char* cname);                  // Collector name                                        $a1

    // Attributes:
    IALibraryProv *impl;                   // Implementation-specific private object pointer            //$a4
    IALibInferenceCollector *top;          // Pointer to top collector of Library                          $a1

};

#endif
