#ifndef IATK_LOGREC_H
#define IATK_LOGREC_H
/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM Agent Building Environment                                             */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/library/c/logrec.h, tk_library, abe.l6
// VERSION: 1.10 97/06/19 18:11:21
//
// INCLUDE AS:  #include <iatk/logrec.h>
//
// PURPOSE:  Define the IALibLogRecord object.
//
// DESCRIPTION:
//    Each IALibLogRecord (LogRecord for short) represents
//    an object that exists in persistent storage in a
//    LogRecordSet.  A LogRecord may record an activity
//    (normal or detailed amount), an error message, or
//    simple information, etc.
//
// CLASSES DECLARED:
//    IALibLogRecord - Represents a single element of the
//                     contents of an IALibLogRecordSet.
//
// NOTES:
//    IALibLogRecordImp is the related class used to implement
//    the IALibLogRecord class.
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ---   ------ --- ------ -----------
// $xn = yymmdd iii pnnnnn (abstract of feature/defect)
//       960904 tlw F00296 File created
// $a1 = 961029 tlw F00445 Support new IALibrary class; constructor
//                         is now private
// $a2 = 961210 tlw D00524 Add destructor-like method delete_
// $a3 = 961212 tlw D00536 Add extractKeyPosition method
// $a4 = 970128 tlw D00612 Add numberOfTokens method
// $a5 = 970519 jeg F00813 Support for Microsoft Visual C++ compiler
//
//******************************************************************************

#include <iatk/types.h>
#include <iatk/logkey.h>                                                                                //$a3

class IALibrary;                                                                                        //$a5
class IALibLogRecordProv;                                                                               //$a5
class IALibLogRecordImp;

class IAEXPCLASS IALibLogRecord {                                                                       //$a5

  public:

     IALibLogRecord(const IALibLogRecord &from);  // Copy constructor

     IALibLogRecord & operator=(const IALibLogRecord &from);  // Assignment operator

    // The destructor is public only to allow the java interface to have access                         //$a5
    // Users should delete the object via the IALibrary delete_IALibLogRecord() function.               //$a5
     ~IALibLogRecord();      // Destructor

    // Prototypes for member functions
    void setType(unsigned int lrType);               // Set the LogRecord type
    unsigned int extractType() const;                // Get the LogRecord type

    void setCategory(int category);                  // Set the LogRecord category
    int extractCategory() const;                     // Get the LogRecord category

    void setOriginator(const char* originator);      // Set the LogRecord originator
    const char* extractOriginator() const;           // Get the LogRecord originator

    const char* extractDate() const;                 // Get the LogRecord date
    const char* extractTime() const;                 // Get the LogRecord time

    void setSourceId(const char* sourceId);          // Set the LogRecord source identifier
    const char* extractSourceId() const;             // Get the LogRecord source identifier

    void setMessageId(unsigned long messageId);       // Set the LogRecord message identifier
    unsigned long extractMessageId() const;           // Get the LogRecord message identifier

    void addToken(const char* sourceToken);          // Add the next token to the LogRecord
    char* getToken();                                // Get the next token from the LogRecord
    int numberOfTokens() const;                      // Get the number of tokens in the LogRecord         $a4

    void delete_();                                  // Destroy this object                               $a2

    logKey extractKeyPosition() const;               // Get the LogKey for this LogRecord                 $a3
                                                     // (0,0 means not in a LogRecordSet)                 $a3

    void debug_print(ostream& outfile) const;        // Print a LogRecord - debugging

  private:

    IALibLogRecord (IALibLogRecordProv* lrimpl, IALibrary* _lib);                                       //$a5

    IALibLogRecordProv *impl;  // Implementation-specific private object pointer                        //$a5
    IALibrary          *lib;                                                                            //$a5

    // The following method is only to be used by IALib classes to
    // access the pointer to a IALibLogRecordImp.
    IALibLogRecordImp* getImpl() const;                                                                  //$a1

    friend class IALibLogRecordImp;
    friend class IALibLogRecordSetImp;
    friend class IALibLogRecordSet;
    friend class IALibrary;                                                                              //$a1
};

#endif
