#ifndef IATK_LOGSET_H
#define IATK_LOGSET_H
/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM Agent Building Environment                                             */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/library/c/logset.h, tk_library, abe.l6
// VERSION: 1.10 97/06/19 18:11:28
//
// INCLUDE AS:  #include <iatk/logset.h>
//
// PURPOSE:  Define the IALibLogRecordSet object.
//
// DESCRIPTION:
//    An IALibLogRecordSet (LogSet) combines LogRecords into a logically
//    related set.  A LogSet is defined having a maximum size, and when
//    a LogRecord is added that would exceed that size, wrap-around occurs.
//
// CLASSES DECLARED:
//    IALibLogRecordSet - Set of logically related LogRecords
//
// NOTES:
//    IALibLogRecordSetImp is the related class used to implement
//    the IALibLogRecordSet class.
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ---   ------ --- ------ -----------
// $xn = yymmdd iii pnnnnn (abstract of feature/defect)
//       960904 tlw F00296 File created
// $a1 = 961029 tlw F00445 Support new IALibrary class; constructor and destructor
//                         are now private
// $a2 = 961114 tlw F00445 Metadata added to IALibLogRecordSet
// $a3 = 961207 tlw D00502 Position parm for get() should be int, since -1 is valid value.
//                         Copy constructor and assignment operator should be private
//                         since not implemented.
// $a4 = 970124 tlw D00600 Default setMetadataName() parm to '0'
// $a5 = 970519 jeg F00813 Support for Microsoft Visual C++ compiler
//
//******************************************************************************


#include <iatk/types.h>
#include <iatk/logrec.h>
#include <iatk/logkey.h>                                                                                 //$a2

class IALibrary;                                                                                        //$a5
class IALibLogRecordSetProv;  // Will point to the implementation of this class.                        //$a5

class IAEXPCLASS IALibLogRecordSet {                                                                    //$a5

  public:

    // Prototypes for member functions
    void setLevel(unsigned int newLevel);             // Set the LogRecordSet level
    unsigned int extractLevel() const;                // Get LogRecordSet level

    unsigned int extractCacheSize() const;            // Get LogRecordSet cacheSize

    unsigned int extractLogSize() const;              // Get LogRecordSet logSize

    unsigned int extractCurrentLogSize() const;       // Get LogRecordSet currentLogSize (# log records)

    char* extractName() const;                        // Get LogRecordSet name

    unsigned int extractVersion() const;              // Get LogRecordSet version                          $a2

    unsigned int extractMetadataSize() const;         // Get LogRecordSet metadataSize                     $a2

    char* extractMetadata() const;                    // Get metadata for current LogRecord                $a2
    char* extractMetadata(logKey position) const;     // Get metadata for LogRecord specified by position  $a2

    void setMetadataName(char mdname='0');            // Set LogRecordSet metadataName                  $a2 $a4
    char extractMetadataName() const;                 // Get LogRecordSet metadataName                     $a2

    void del();                                       // Delete the persistent copy of the LogRecordSet
                                                      // and the object's LogRecords

    void clear();                                     // Clear the LogRecords from the LogRecordSet
                                                      // (in both the object and the persistent copy)

    void addRecord(const IALibLogRecord& newLogRecord);  // Add a LogRecord to the LogRecordSet

    IALibLogRecord* getRecord();                         // Get a LogRecord from the LogRecordSet

    void get(int position);                            // Get contents of the LogRecordSet from            $a3
                                                       // persistent storage

    void get(logKey keyPosition);                     // Get contents of the LogRecordSet from             $a2
                                                      // persistent storage                                $a2

    void put();                           // Put contents of the LogRecordSet to persistent storage

    void putMetadata();                   // Put LogRecordSet metadata to persistent storage               $a2

    void setMetadata(const char* metadata);                                                                    //$a2
                      // Set LogRecord metadata at current cursor position                                 $a2

    void setMetadata(const char* metadata, logKey blk_offset);                                                 //$a2
                      // Set LogRecord metadata at blk_offset                                              $a2

    void debug_print(ostream& outfile) const;   // Print the LogRecordSet - debugging

  private:

    IALibLogRecordSet(IALibLogRecordSetProv* lsimpl, IALibrary* _lib);                                  //$a5

    ~IALibLogRecordSet();         // Destructor                                                            $a1

    IALibLogRecordSetProv *impl;   // Implementation-specific private object pointer                    //$a5
    IALibrary             *lib;                                                                         //$a5

    friend class IALibrary;                                                                              //$a1

};

#endif
