#ifndef IATK_LTFACT_H
#define IATK_LTFACT_H
/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM Agent Building Environment                                             */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/library/c/ltfact.h, tk_library, abe.l6
// VERSION: 1.10 97/06/19 18:11:36
//
// INCLUDE AS:  #include <iatk/ltfact.h>
//
// PURPOSE:  Define the IALibInferenceLTFact object.
//
// DESCRIPTION:
//    An IALibInferenceLTFact (LTFact) represents a long term fact
//    that is normally built by a rule editor and then used for
//    inferencing.  LTFacts are grouped into LTFactSets, an object
//    that contains them.
//
// CLASSES DECLARED:
//    IALibInferenceLTFact - Long Term Fact
//
// NOTES:
//    IALibInferenceLTFactImp is the related class used to implement
//    the IALibInferenceLTFact class.
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ---   ------ --- ------ -----------
// $xn = yymmdd iii pnnnnn (abstract of feature/defect)
// $a1 = 961029 tlw F00445 Support new IALibrary class; constructor is
//                         now private
// $a2 = 961210 tlw D00524 Add destructor-like method delete_
// $a3 = 970116 tlw D00577 Add getImpl method
// $a4 = 970519 jeg F00813 Support for Microsoft Visual C++ compiler
//
//******************************************************************************


#include <iatk/types.h>
/*
#ifndef IATK_LTFACTCONSTANTS
#define IATK_LTFACTCONSTANTS
char* const ACTIVEF = "A";                 // LTFact is active
char* const INACTIVEF = "I";               // LTFact is inactive
const IABool DELETEDF = true;              // LTFact deleted
const IABool NOTDELETEDF = false;          // LTFact not deleted
#endif
*/
class IALibrary;                                                                                        //$a4
class IALibInferenceLTFactProv;                                                                         //$a4
class IALibInferenceLTFactImp;

class IAEXPCLASS IALibInferenceLTFact {                                                                 //$a4

  public:

     IALibInferenceLTFact(const IALibInferenceLTFact &from);  // Copy constructor

     IALibInferenceLTFact & operator=(const IALibInferenceLTFact &from);  // Assignment operator

    // The destructor is public only to allow the java interface to have access                         //$a4
    // Users should delete the object via the IALibrary delete_IALibInferenceLTFact() function.         //$a4
     ~IALibInferenceLTFact();                 // Destructor

    // Prototype for member functions
    const char* extractName() const;       // Get the LTFact Name
    const char* extractID() const;         // Get the LTFact ID
    IABool isActive() const;               // Is the LTFact active?
    IABool isDeleted() const;              // Is the LTFact deleted?

    void setName(const char* LTFactName);     // Set the LTFact name
    void setID(const char* LTFactID);         // Set the LTFact ID
    void activate();                 // Activate the LTFact
    void inactivate();               // Inactivate the LTFact
    void del();                      // Delete the LTFact
    void undelete();                 // Undelete the LTFact

    const char* extractKif() const;       // Get KIF part of LTFact
    void setKif(const char* kifPart);     // Store string into KIF part of LTFact

    const char* extractMetadata() const;                // Get metadata part of LTFact
    void setMetadata(const char* metadataPart);         // Store string into metadata part of LTFact

    IABool validate() const;          // Is the LTFact valid?

    void delete_();                   // Destroy this object                                              $a2

    void debug_print(ostream& outfile) const;        // Print the LTFact - debugging

    IALibInferenceLTFactImp* getImpl() const;  // get pointer to impl object                            //$a4

  private:

    IALibInferenceLTFact(IALibInferenceLTFactProv *cimpl, IALibrary* _lib);                             //$a4

    IALibInferenceLTFactProv *impl;   // Implementation-specific private object pointer                 //$a4
    IALibrary                *lib;                                                                      //$a4

    friend class IALibrary;                                                                              //$a1
    friend class IALibInferenceLTFactSet;                                                                //$a1
};
#endif
