#ifndef IATK_LTFSET_H
#define IATK_LTFSET_H
/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM Agent Building Environment                                             */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/library/c/ltfset.h, tk_library, abe.l6
// VERSION: 1.14 97/06/19 18:11:44
//
// INCLUDE AS:  #include <iatk/ltfset.h>
//
// PURPOSE:  Define the IALibInferenceLTFactSet object.
//
// DESCRIPTION:
//    An IALibInferenceLTFactSet is a set of LTFacts grouped
//    together to be used for inferencing.
//
// CLASSES DECLARED:
//    IALibInferenceLTFactSet - Long Term Fact Set
//
// NOTES:
//    IALibInferenceLTFactSetImp is the related class used to implement
//    the IALibInferenceLTFactSet class.
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ---   ------ --- ------ -----------
// $xn = yymmdd iii pnnnnn (abstract of feature/defect)
// $a1 = 960920 tlw F00296 Add signatures to find() and first/last/next/previousLTFact()
//                         to return a pointer to a LTFact
// $a2 = 960923 tlw F00337 Add updateLTFact() method
// $a3 = 961011 tlw F00414 Add currentLTFact() method
// $a4 = 961029 tlw F00445 Support new IALibrary class; constructor and destructor
//                         are now private
// $a5 = 961023 tlw F00445 Change method names so no duplicates
// $a6 = 961121 tlw F00445 Add size method
// $a7 = 961210 tlw D00502 Make copy constructor and assignment operator private
// $a8 = 970107 tlw D00559 Add extractVersion method
// $a9 = 970124 tlw D00600 Default setLTFactsMetadataName() parm to '0'
// $aa = 970519 jeg F00813 Support for Microsoft Visual C++ compiler
//
//******************************************************************************

#include <iatk/types.h>
#include <iatk/ltfact.h>
/*
#ifndef IATK_LTFACTSETCONSTANTS
#define IATK_LTFACTSETCONSTANTS
char* const ACTIVEFS = "A";  	     // Active LTFactSet
char* const INACTIVEFS = "I";      // Inactive LTFactSet
#endif
*/
class IALibrary;                                                                                        //$aa
class IALibInferenceLTFactSetProv;                                                                      //$aa

class IAEXPCLASS IALibInferenceLTFactSet {                                                              //$aa

  public:

    // Prototype for member functions
    const char* extractName() const;           // Get the LTFactSet Name
    const char* extractID() const;             // Get the LTFactSet ID

    void setMetadataName(const char mdn='0');            // Set the LTFactSet metadata name
    char extractMetadataName() const;                    // Get the LTFactSet metadata name

    void setMetadata(const char* mdata);               // Set the LTFactSet-level metadata
    const char* extractMetadata() const;               // Get LTFactSet-level metadata

    unsigned int extractVersion() const;               // Get LTFactSet version                           $a8

    void setLTFactsMetadataName(const char ltfmdname='0');  // Set the LTFacts-level metadata name        $a9
    const char extractLTFactsMetadataName() const;   // Get the LTFacts-level metadata name

    void activate();                 // Activate the LTFactSet
    void inactivate();               // Inactivate the LTFactSet

    IABool isActive() const;               // Is the LTFactSet active?

    int get();                     // Get LTFacts and the named metadata from Store
                                   // Return value = 0 indicates success
                                   // Return value != 0 indicates failure
    int getWithoutMetadata();      // Get only the LTFacts from Store
                                   // Return value = 0 indicates success
                                   // Return value != 0 indicates failure

    void del();                    // Delete the LTFactSet from Store
    IABool delMetadata(char mdname);   // Delete the LTFactSet Metadata from Store
    void delLTFactsMetadata(char mdname);      // Delete the LTFacts Metadata from Store

    int put();                     // Put LTFacts and the named metadata to Store
                                   // Return value = 0 indicates success
                                   // Return value > 0 indicates position of LTFact that failed validation;
                                   //   (no LTFacts have been put)
    int putWithoutMetadata();      // Put only the LTFacts to Store
                                   // Return value = 0 indicates success
                                   // Return value > 0 indicates position of LTFact that failed validation;
                                   //   (no LTFacts have been put)

    IABool find(const char* ruleName, IALibInferenceLTFact& fact);  // Find a LTFact given its name and return a reference to it
    IALibInferenceLTFact* find(const char* factName);  // Find a LTFact given its name and return a pointer to it $a1

    void addLTFact(const IALibInferenceLTFact& fact);              // Add a LTFact to the end of the LTFactSet, positioning cursor to added LTFact
    void addLTFactAsNext(const IALibInferenceLTFact& fact);        // Add a LTFact (after the current position) to the LTFactSet, positioning cursor to added LTFact
    void addLTFactAsPrevious(const IALibInferenceLTFact& fact);    // Add a LTFact (before the current position) to the LTFactSet, positioning cursor to added LTFact
    int updateLTFact(IALibInferenceLTFact& fact);    // Update the LTFact at the current cursor position, keeping the cursor at the current LTFact $a2
    IABool deleteLTFact();                   // Delete LTFact at current cursor position by setting "Delete" field

    IABool firstLTFact(IALibInferenceLTFact& fact);      // Position cursor to first LTFact in the LTFactSet and return reference to LTFact
    IALibInferenceLTFact* firstLTFact();      // Position cursor to first LTFact in the LTFactSet and return pointer to LTFact $a1$a5
    IABool lastLTFact(IALibInferenceLTFact& fact);       // Position cursor to last LTFact in the LTFactSet and return reference to LTFact
    IALibInferenceLTFact* lastLTFact();       // Position cursor to last LTFact in the LTFactSet and return pointer to LTFact $a1$a5
    IABool nextLTFact(IALibInferenceLTFact& fact);       // Position cursor to next LTFact in the LTFactSet and return reference to LTFact
    IALibInferenceLTFact* nextLTFact();       // Position cursor to next LTFact in the LTFactSet and return pointer to LTFact $a1$a5
    IABool previousLTFact(IALibInferenceLTFact& fact);   // Position cursor to previous LTFact in the LTFactSet and return reference to LTFact
    IALibInferenceLTFact* previousLTFact();   // Position cursor to previous LTFact in the LTFactSet and return pointer to LTFact $a1$a5
    IABool currentLTFact(IALibInferenceLTFact& fact) const;   // Return reference to current LTFact              $a3
    IALibInferenceLTFact* currentLTFact();   // Return pointer to current LTFact                           $a3$a5

    int size() const;  // Return the number of undeleted facts in the LTFactSet                            $a6

    void debug_print(ostream& outfile) const;   // Print the LTFactSet - debugging

  private:

    IALibInferenceLTFactSet (IALibInferenceLTFactSetProv* lsimpl, IALibrary* _lib);                     //$aa

    IALibInferenceLTFactSetProv *impl;   // Implementation-specific private object pointer              //$aa
    IALibrary                   *lib;                                                                   //$aa

    ~IALibInferenceLTFactSet();         // Destructor                                                      $a4

    friend class IALibrary;                                                                              //$a4

};
#endif
