#ifndef IATK_PROCTOKN_H
#define IATK_PROCTOKN_H

/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM AgentBuilder                                                           */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/ae-api/api/proctokn.h, tk_aeapi, abe.l6
// VERSION: 1.6 97/04/02 12:42:07
//
// INCLUDE AS:  <iatk/proctokn.h>
//
// PURPOSE:  Defines format of tokens associated with the named
//           sensor/effector procedures provided by Adapters.
//
// CLASSES DECLARED:
//    IAProcToken = Enum of types of adapter named procedures.
//
// NOTES:
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   =====  === ====== ===========
// New = 960622 JMG f82    Moved from iatk/types.h.
//
//******************************************************************************

//##############################################################################
//
// Class IAProcToken
//
// Token assigned by adapters for representing a named procedure.  We
// permit this token to be either an integer or a pointer.  This lets
// Adapters use it either as an ordinal, or a function pointer.
//
//##############################################################################

enum IAProcTokenType {IAIntProcToken, IAPtrProcToken};

struct IAProcToken {
   IAProcToken()
      : type(IAIntProcToken), intToken(0) {}
      // Use this zero-arg constructor to get an integer token with
      // value zero.  (Can't specifiy IAProcToken(0) since compiler
      // can disambiguate integer and pointer.)

   IAProcToken(long  _intvalue)
      : type(IAIntProcToken), intToken(_intvalue) {}

   IAProcToken(void *_ptrvalue, int wantnullptr=0)
      : type(IAPtrProcToken), ptrToken(_ptrvalue) {}
      // Use this constructor with a dummy 2nd arg to get 0/NULL used
      // to make a pointer token.

   operator long()  const {return intToken;}
   operator void*() const {return ptrToken;}
      // Since a given adapters will probably use one form or the other
      // exclusively, we don't do any checking on the extraction operators.

   IAProcTokenType tokenType() const {return type;}

   IAProcTokenType  type;          // Which kind this one is
   union {
      long          intToken;      // Integer form of a procedure token
      void          *ptrToken;     // Pointer form of a procedure token
   };
};

#endif // IATK_PROCTOKN_H
