#ifndef IATK_RULE_H
#define IATK_RULE_H
/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM Agent Building Environment                                             */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/library/c/rule.h, tk_library, abe.l6
// VERSION: 1.10 97/06/19 18:11:52
//
// INCLUDE AS:  #include <iatk/rule.h>
//
// PURPOSE:  Define the IALibInferenceRule object.
//
// DESCRIPTION:
//    An IALibInferenceRule (Rule) represent a rule
//    that is normally build by a rule editor and then used for
//    inferencing.  Rules are grouped into RuleSets, an object
//    that contains them.
//
// CLASSES DECLARED:
//    IALibInferenceRule - Rule
//
// NOTES:
//    IALibInferenceRuleImp is the related class used to implement
//    the IALibInferenceRule class.
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ---   ------ --- ------ -----------
// $xn = yymmdd iii pnnnnn (abstract of feature/defect)
// $a1 = 961029 tlw F00445 Support new IALibrary class; constructor
//                         is now private
// $a2 = 961210 tlw D00524 Add destructor-like method delete_
// $a3 = 970116 tlw D00577 Add getImpl method
// $a4 = 970519 jeg F00813 Support for Microsoft Visual C++ compiler
//
//******************************************************************************

#include <iatk/types.h>
/*
#ifndef IATK_RULECONSTANTS
#define IATK_RULECONSTANTS
char* const ACTIVER = "A";                 // Rule is active
char* const INACTIVER = "I";               // Rule is inactive
const IABool DELETEDR = true;              // Rule deleted
const IABool NOTDELETEDR = false;          // Rule not deleted
#endif
*/
class IALibrary;                                                                                        //$a4
class IALibInferenceRuleProv;                                                                           //$a4
class IALibInferenceRuleImp;

class IAEXPCLASS IALibInferenceRule {                                                                   //$a4

  public:

     IALibInferenceRule(const IALibInferenceRule &from);  // Copy constructor

     IALibInferenceRule & operator=(const IALibInferenceRule &from);  // Assignment operator

    // The destructor is public only to allow the java interface to have access                         //$a4
    // Users should delete the object via the IALibrary delete_IALibInferenceRule() function.           //$a4
     ~IALibInferenceRule();                 // Destructor

    // Prototype for member functions
    const char* extractName() const;       // Get the Rule Name
    const char* extractID() const;         // Get the Rule ID
    IABool isActive() const;               // Is the Rule active?
    IABool isDeleted() const;              // Is the Rule deleted?

    void setName(const char* ruleName);     // Set the Rule name
    void setID(const char* ruleID);         // Set the Rule ID
    void activate();                 // Activate the Rule
    void inactivate();               // Inactivate the Rule
    void del();                      // Delete the Rule
    void undelete();                 // Undelete the Rule

    const char* extractKif() const;               // Get KIF part of Rule
    void setKif(const char* kifPart);     // Store string into KIF part of Rule

    const char* extractMetadata() const;                // Get metadata part of Rule
    void setMetadata(const char* metadataPart);   // Store string into metadata part of Rule

    IABool validate() const;          // Is the Rule valid?

    void delete_();                   // Destroy this object                                              $a2

    void debug_print(ostream& outfile) const;      // Print the Rule - debugging

    IALibInferenceRuleImp* getImpl() const;  // get pointer to impl object                              //$a4

  private:

    IALibInferenceRule(IALibInferenceRuleProv* rimpl, IALibrary* _lib);                                 //$a4

    IALibInferenceRuleProv *impl;   // Implementation-specific private object pointer                   //$a4
    IALibrary              *lib;                                                                        //$a4

    friend class IALibrary;                                                                              //$a1
    friend class IALibInferenceRuleSet;                                                                 //$a4

};
#endif
