#ifndef IATK_TYPES_H
#define IATK_TYPES_H

/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM Agent Building Environment                                             */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/ae-api/api/types.h, tk_aeapi, abe.l6
// VERSION: 1.17 97/06/20 16:52:09
//
// INCLUDE AS:  <iatk/types.h>
//
// PURPOSE:  Defines basic data types used by toolkit frameworks.
//
// DATA TYPES DECLARED:
//    IABool     = Boolean values used in toolkit APIs.
//    IARC       = Return code from toolkit APIs.
//    IAProcType = Enum of types of adapter named procedures.
//
// NOTES:
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   =====  === ====== ===========
// New = 960607 JMG f10    Initial implementation of adapter/engine intf
// --- = 960622 JMG f82    Clean up; split IAProcToken into separate header.
// $E1 = 960908 JMG f307   Allow adapters to be built using MS Visual C++
// $B1 = 961009 JMG d408   Make MS Visual C++ API library a DLL
//
// $B2 = 970416 JEG f771   Implement event completion status and
//                         agent alert mechanism
//
// $B3 = 970619 JMG f876   Prepare for use of Borland C++ on Win32
//
//******************************************************************************

typedef int IABool;
   // IABools have the usual values of 0 for false and 1 for true.  We don't
   // supply definitions of "false" and "true" because many different compiler
   // and or library header files will define them and there is no good way
   // for us to tell if they've been defined already.

typedef int IARC;

enum IAProcType {IANoProc, IABooleanSensorProc, IASensorProc,
                 IAEffectorProc};

enum IAEventCompletionStatus {IAEventOK, IARetryEvent, IAEventFailed};          //@B2

enum IAAlertLevel {IAWarning, IADegraded, IAFatal};                             //@B2


// Support compilers and their pre-defined macro "signatures":
//
// IBM VisualAge C++ on Win95/NT:
//    defines __IBMCPP__, __WINDOWS__, _WIN32
//
// Microsoft Visual C++ on Win95/NT:
//    defines _MSC_VER, _WIN32
//
// Borland C++ on Win95/NT:
//    defines __BORLANDC__, __WIN32__
//
// IBM Cset++ on AIX:
//    defines __xlC__
//
// IBM VisualAge C++ on OS/2:
//    defines __IBMCPP__, __OS2__
//


#if defined(__OS2__)                                                            //@B3
   #define IACALLBK                                                             //@E1
   #define IACLINK  _System                                                     //@E1
#elif defined(_WIN32) || defined(__WIN32__)                                     //@B3
   #define IACALLBK __stdcall                                                   //@E1
   #define IACLINK  __stdcall                                                   //@E1
#else                                                                           //@E1
   #define IACALLBK                                                             //@E1
   #define IACLINK                                                              //@E1
#endif                                                                          //@E1

#ifndef IAEXPORT1
#if defined(__OS2__)                                                            //@B3
  #define IAEXPORT1
  #define IAEXPORT2 _Export
#elif defined(_WIN32) || defined(__WIN32__)                                     //@B3
  #define IAEXPORT1 __declspec(dllexport)
  #define IAEXPORT2
#else
  #define IAEXPORT1
  #define IAEXPORT2
#endif
#endif

#if defined(_WIN32) || defined(__WIN32__)                                       //@B3
#if defined(_MSC_VER) || defined(__BORLANDC__)                                  //@B3
#ifdef _IA_BUILD                                                                //@B1
#define IAEXPCLASS __declspec(dllexport)                                        //@B1
#else                                                                           //@B1
#define IAEXPCLASS __declspec(dllimport)                                        //@B3
#endif                                                                          //@B1
#else                                                                           //@B3
#define IAEXPCLASS                                                              //@B3
#endif                                                                          //@B3
#else                                                                           //@B1
#define IAEXPCLASS                                                              //@B1
#endif                                                                          //@B1


// Use IAPURE when declaring pure virtual destructors.  XLC on AIX issues
// an annoying warning for pure virtual destructors, so we (sadly) skip
// making them pure virtual on AIX.

#ifdef __xlC__
#define IAPURE
#else
#define IAPURE = 0
#endif

#endif // IATK_TYPES_H
