#ifndef IATK_UNIQUEID_H
#define IATK_UNIQUEID_H

/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM Agent Building Environment                                             */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/ae-api/api/uniqueid.h, tk_aeapi, abe.l6
// VERSION: 1.11 97/06/03 10:50:12
//
// INCLUDE AS:  #include <iatk/uniqueid.h>
//
// PURPOSE:  Defines the IAUniqueId class, used within the toolkit as
//           a "serial number" for events and possibly other things.
//
// CLASSES DECLARED:
//    IAUniqueId      - Unique identifier for events and maybe other things
//
// THREADING NOTES:
// - Concurrency control is the caller's responsibility; the implementation
//   of these classes does not perform its own locking.  All of the classes
//   require exclusive access to the object being operated upon (the "this"
//   object) as well as any others identified in function parameter lists.
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// New = 960622 JMG f82    Newly split from old apiobjs.h.
// $A1 = 960807 WES f256   Change int results to IABool
// $B1 = 961009 JMG d408   Make MS Visual C++ API library a DLL
// $A2 = 970512 WES        hashCode needed
//
//******************************************************************************

#include <string.h>
#include <time.h>
#include <iatk/types.h>

//##############################################################################
//
// Class IAUniqueId
//
// An IAUniqueId object is a "serial number" that can be used to distinguish
// one IATK object (trigger event, adapter etc.) from another within the
// scope of a particular Agent instance.
//
//##############################################################################

class IAUniqueIdPvt;
class ostream;

class IAEXPCLASS IAUniqueId {                                                   //@B1
 public:
   //---------------------------------------------------------------------------
   // Default constructor, Destructor and Assignment operator
   //---------------------------------------------------------------------------

   IAUniqueId();
      // IAUniqueId isn't valid (isValid returns 0) after the constructor
      // runs; generate() must be called to make the UniqueId valid.

   ~IAUniqueId();

   IAUniqueId(const IAUniqueId &from);
   IAUniqueId & operator=(const IAUniqueId &rhs);


   //---------------------------------------------------------------------------
   // Equality and ordering operators
   //---------------------------------------------------------------------------

   IABool operator==(const IAUniqueId &rhs) const;                        //@A1
   IABool operator!=(const IAUniqueId &rhs) const;                        //@A1
   IABool operator< (const IAUniqueId &rhs) const;                        //@A1
      // operator< is defined to permit IAUniqueIds to be used as keys
      // on collections that require a key-ordering relation. This operator
      // defines an arbitrary ordering that does not necessarily correspond
      // to a ordering according to time of generation.


   void generate();
      // Sets this IAUniqueId to a unique value that will not be generated
      // generated again in this Agent instance.  No guarantees are made
      // about uniqueness across different agents, or across different
      // instances of a particular agent.

   // hashcodes are > 0
   long hashCode() const;                                                 //@A2

   IABool isValid() const;                                                //@A1
      // Returns non-zero if this IAUniqueId contains a valid unique ID
      // as generated by IAUniqueId::generate, otherwise returns zero.

   const char * asString() const;
      // Returns string representation of the UniqueId.

   //---------------------------------------------------------------------------
   // Debugging help
   //---------------------------------------------------------------------------

   void dbgInfo(ostream &o) const;

 private:
   IAUniqueIdPvt *p;     // Impl-specific private object.
};


#endif // IATK_UNIQUEID_H
