/*** Copyright ****************************************************************/
/*                                                                            */
/* IBM AgentBuilder                                                           */
/*                                                                            */
/* (C) Copyright IBM Corp. 1996                                               */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.          */
/*                                                                            */
/******************************************************************************/

//******************************************************************************
//
// FILE:    src/sampkif/utilkif.h , toolkit.a2
// VERSION: 1.3 96/06/20 15:52:04
//
// PURPOSE:  Implement the IABnode and IAKString methods.
//
// DESCRIPTION:
//    IABnode  represents  a node in a Binary tree
//    IAKString describes an object pointing to the top of a binary tree of 'IABnode' nodes,
//    which is the internal representation of a KIF-formatted Rule (string)
//
// CLASSES DECLARED:
//    IABnode
//    IAKString
//
// NOTES:
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ---   ------ --- ------ -----------
// $xn = yymmdd iii pnnnnn (abstract of feature/defect)
//
//******************************************************************************

#ifndef _KIFONLY_HPP
#define _KIFONLY_HPP

#include <iatk/types.h>          // includes IABool


class AString;




// Class definition
class IABnode {
  public:
          IABnode(const char* instr);
          IABnode();
          ~IABnode();
          void PrintBnodeObject();
          const  char *  GetBnodeData();
          IABnode *& GetBnodeRightChild();
          IABnode *&  GetBnodeLeftChild();
          void SetBnodeData(const char * Bdata);
          void SetBnodeRightChild(IABnode* Bright);
          void SetBnodeLeftChild(IABnode* Bleft);
  private:
          AString * data;
          IABnode* right;
          IABnode* left;
      };

class IAKString {
// IAKString will create objects that will be able to translate a rule from its KIF format (AString)
// to a binary tree format understood by the K-Engines, by using methods
   public:
      IAKString(const char*  instr);
      IAKString();
     ~IAKString();
      IAKString & operator=(const char* s);
      IAKString & operator=( IABnode* bt);
      const char *  GetName() ;
      const char *  GetKIFed();
      IABnode*  GetBinTop();
      IABnode*  GetAntecedent();
      IABnode*  GetConsequent();
      void    SetName( const char * InName);
      void    SetKIFed( const char * KIFstr);
      void    SetBinTop(IABnode* Btop);
      void    SetAntecedent(IABnode* left);
      void    SetConsequent(IABnode* right);
      IABool PrintRuleTree(IABnode* top, char * buffer, int buff_len);
      IABool PrintAntecedentTree(IABnode* top,char * buffer, int buff_len);
      IABool PrintConsequentTree(IABnode* top,char * buffer, int buff_len);
      void    ShowAllTreeStructure(IABnode* top);
      void    KIFToBin();
      void    PrintKIFed();
      void    PrintKStringObject();

   private:
      AString * Name;
      AString * KIFed;
      IABnode*  binTree;
      IABnode*  LeftSubtree;
      IABnode*  RightSubtree;
      void Parser(AString& mystr, IABnode* &ptr_parent);
      void StringSplit(AString mystr,AString &left, AString &right);
      void DeleteBinTree( IABnode* &top);
      void PrintTreeA(IABnode* top,AString &s);
      void FindConsClosedParentheses(AString str,unsigned &numOfClosedPar,unsigned &indxOfLastClosedPar);
};



//----General External Utility procs ----------------------------



void PrintTree(IABnode* top,AString &s,char d);
     // prints out a binary tree prefix format

void PrintWholeTree(IABnode* top, AString &s);
     // prints out a binary tree prefix format

void DeleteTree(IABnode* &top);
    // deletes all nodes of a binary tree exept the root one


AString KIFaNode(AString str);
    // converts to KIF the string of a node


void KIFaTree(IABnode* top,AString& str,IABool&  success);
   // recursivelly traverses the tree and invokes the KIFaNode();

IABool  BinToKIF(IABnode* top,char* buffer,int buff_len);
   // KIFs the whole binary tree (Rule)

IABool CopyAstrToCharptr(AString s, char * buffer, int buff_len);


#endif
