#ifndef SAMPMAIL_H
#define SAMPMAIL_H
/*************************************************************************
** File:         sampmail.h
** Description:  Header file for a sample email adapter
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/

#include <iatk/adapter.h>         // IAAdapter class
#include <i0string.hpp>           // I0String class

// Note: This adapter generates no events and handles no sensors.  Therefore
// the inherited versions of the following member functions provided
// by the IAAdapter class are adequate for this adapter, and are not
// defined here.  See the doc for adapter creation or the header files
// for the IAAdapter class for information on their function and content.
//     start
//     stop
//     shutdown
//     eventComplete
//     testCondition
//     answerQuery

class SampMailAdapter : public IAAdapter {
 public:
   SampMailAdapter();
   ~SampMailAdapter() {}
   IABool IACALLBK identify();
   void IACALLBK performAction(IAProcToken token, const IAEventHeader &hdr,
      const char *binding);

 private:
   // Tokens for our Sensors
     // none
   // Tokens for our Effectors
   static const int EMailFromAddressIs_Token;
   static const int EMail_Token;
   static const int EMailWithSubject_Token;
   static const int EMailRelayHostIs_Token;

   static I0String fromAddress;       // EMail address for "From:" field
   static I0String tempEmailFileName; // File for sendmail to send
   static I0String relayHost;         // Host name to relay mail through

   // Function used by performAction to conduct SMTP dialog
   int sendCommand(const I0String &cmd,
                   int sock,
                   const I0String &expectedCodes,
                   char* buf,
                   int bufSize) throw(int);


};  // end of class SampMailAdapter
#endif
