package ibm.AgentBuilder.samples.agent;
import ibm.AgentBuilder.aeapi.IAAgent;
import ibm.AgentBuilder.aeapi.IAAdapterHandle;
import ibm.AgentBuilder.aeapi.IAEngineHandle;
import ibm.AgentBuilder.aeapi.IALibraryHandle;
import ibm.AgentBuilder.misc.IAError;
import java.io.*;
import java.util.StringTokenizer;
import java.util.Properties;
import java.util.Vector;
import java.util.Date;
import java.util.Enumeration;

public class JavaAgent extends IAAgent   {
  private String filename;
  private static String thisOS=System.getProperty("os.name");
  private Vector libraries=new Vector();
  private Vector adapters=new Vector();
  private Vector libraryStrings=new Vector();
  private Vector adapterStrings=new Vector();
  private Vector engineStrings=new Vector();
  private Properties engines=new Properties();
  private IAEngineHandle firstEngine;
  private Vector conductSets=new Vector();
  private boolean agentStopped=false;
  private boolean check=false;
  private boolean daemon=false;
  private boolean verbose=false;
  private boolean monitor=false;
  private boolean exit=false;
  private boolean loadcs=false;
  long startTime;

  public static void main(String args[])
  {
    JavaAgent agent = new JavaAgent(args);

    // calls agent.addAdapter, agent.addEngine, agent.addLibrary
    if (!agent.parseConfigFile()) return;

    if (!agent.loadAll()) return;

    // calls agent.connect
    agent.makeConnections();

    if (agent.verbose) agent.dbgInfo(System.out);

    // calls agent.loadConductSet
    if (!agent.loadPreConfiguredConductSets()) return;

    // start agent if no errors so far, currently check is always false
    if (!agent.check) {
      // call agent.start
      try {
        agent.start();
      }
      catch (IAError e) {
          agent.outputError("Error starting agent: "+ e.getMessage());
          return;
      }

     // get agent start time and set up monitor if indicated on options
     Date oldDate=new Date();
     agent.startTime=oldDate.getTime();
     if (agent.monitor) {
        StatsMonitor aMonitor=new StatsMonitor(agent);
        aMonitor.start();
     }
    }
    else System.exit(0);

    // manage interactive input from user
    agent.simpleManager();

    // call agent.stop
    agent.stopAgent();

    System.exit(0);  // Force process to end regardless of other threads
  }

// constructor
JavaAgent(String args[])
   {
     if (!parseCommandLine(args)) System.exit(1);
   }


public void simpleManager()
   {

    String line;
    StringTokenizer tokens;
    String currentToken;

   try {
     while (!agentStopped)
       {
        StringBuffer console=new StringBuffer();
        char ch;

        // if in daemon mode wait for ctrl-C
        // once conductset loaded and agent doing real work don't block
        // on read in OS/2 because garbage collection won't run
        if ( daemon || (thisOS.equals("OS/2")  && loadcs)) {
           outputInfo("Waiting for signal to terminate");
           // wait will always timeout in this agent
           while (!waitForStopAgent(3000)) {
               // terminate signal only will end this agent in daemon mode
              }
        }

        // otherwise wait for command
        else {
          outputInfo("Enter 'quit' , 'stats' , 'loadcs name [selector]' , " +
          " 'unloadcs name [selector]' or 'reset [selector]' " );
          while ((ch=(char)System.in.read()) != '\n')
              console.append(ch);
         }

        // tokenize the input
        tokens=new StringTokenizer(console.toString());

        // first token is the keyword
        if (tokens.hasMoreTokens()) {
           currentToken=tokens.nextToken();

           if (currentToken.equals("quit")) {
              return;
           }

          else if (currentToken.equals("stats")) {
            // throughput since beginning of agent
            outputInfo("Processing " + getStats() +
                               " events per second");
          }

          else if (currentToken.equals("loadcs")) {
           // pass engine, conductsetname, and selector if one exists
             if (tokens.countTokens() < 1)
                outputInfo("Error in loadcs.  Format is loadcs "+
                           "conductsetname [selector]");
             else if (setupLoadConductSet(firstEngine,tokens.nextToken("\n")))
                 loadcs=true;
             else outputInfo("Error in loadcs.  Format is loadcs "+
                           "conductsetname [selector]");
          }

          else if (currentToken.equals("unloadcs")) {
            // pass engine, conductsetname, and selector if one exists
            if (tokens.countTokens() < 1)
               outputInfo("Error in unloadcs.  Format is unloadcs "+
                           "conductsetname [selector]");
            else if (!setupUnloadConductSet(firstEngine,tokens.nextToken("\n")))
                outputInfo("Error in unloadcs.  Format is unloadcs "+
                            "conductsetname [selector]");
          }
          else if (currentToken.equals("reset")) {
            // pass engine, conductsetname, and selector if one exists
            String resetParm=new String("");
            if (tokens.hasMoreTokens())
               resetParm=tokens.nextToken("\n");
            if (!setupReset(resetParm))
                outputInfo("Error in reset.  Format is reset [selector]");
          }
        } // if tokens.hasMoreTokens()
       }  // while
   }
   catch (Exception e) {
      outputError("Agent program exception "+e.getMessage());
      e.printStackTrace();
     }
 }   // end start()



synchronized void stopAgent()
  {

// IAAgent stop

    try {
      stop();
    }
    catch (IAError e) {
       outputError(e.getMessage());
       return;   // what if we get an error on Agent Stopped?
    }
    agentStopped=true;
    notify();
  }


synchronized boolean waitForStopAgent(long millisec)
  {
   try {
     wait(millisec);
   }
   catch (InterruptedException e) {}
   return agentStopped;
  }


float getStats() {
   Date newDate=new Date();
   long newTime=newDate.getTime()-startTime;

// use of IAAgent eventsCompleted method

   return ((float)eventsCompleted()/(newTime))*1000;

}

boolean parseCommandLine(String args[])
  {
    if (args.length==0) {
       outputError("No options specified.  "+
             "Issue \"iagent -?\" for usage information.");
      return false;
    }
    for (int i=0;i<args.length;i++) {
      if (args[i].charAt(0)!='-' && args[i].charAt(0)!='/'
                                 || args[i].length() < 2) {
         return false;
       }
      String rest=args[i].substring(1);
      while (rest.length() > 0) {
        switch(rest.charAt(0)) {
          // don't allow this option for now - because we can't exit cleanly with
          // after adding Java adapters until JDK 1.1 implemented
          // case 'c':
          // check=true;
          // break;
          case 'd':
            daemon=true;
            rest=new String(rest.substring(1));
            break;
          case 'v':
            verbose=true;
            rest=new String(rest.substring(1));
            break;
          case 'm':
            monitor=true;
            daemon=true;
            rest=new String(rest.substring(1));
            break;
          case '?':
             outputInfo("Usage: iagent [options]\n"+
                      "options: -fFILENAME name of configuration file\n"+
                      "                    Default is iagent.cfg.\n"+
         //           "         -c         Check configuration file,\n"+
         //           "                    but don't start agent.\n"+
                      "         -d         Run as background program.\n"+
                      "         -v         Print agent registry.\n"+
                      "         -m         Activate throughput monitoring.\n"+
                      "         -?         Print usage information.\n");
             return false;
          case 'f':
             // case of filename not immediately following -f
             if (rest.length()==1) {
                if (++i>args.length) {
                outputError(" Configuration filename is missing");
                return false;
                }
                else {
                   filename=args[i];
                }
             }
             else {
                filename=rest.substring(1);
             }
             rest=new String("");
             break;

          default:
             outputError("Unknown option \""+rest.charAt(0)+"\" specified.  "+
                 "Issue \"iagent -?\" for usage information.");
             return false;
        }   // switch
      }    // while
    }    // for
    return true;
  }

void makeConnections() {
// connect all engines and adapters
 try {
    for (Enumeration allEngines=engines.elements();allEngines.hasMoreElements();) {
      IAEngineHandle anEngine=(IAEngineHandle)allEngines.nextElement();
      for (Enumeration allAdapters=adapters.elements();allAdapters.hasMoreElements();)
// IAAgent connect
          connect((IAAdapterHandle)allAdapters.nextElement(),anEngine);
    }
 }
 catch (IAError e)
    {
     outputError("Error in make connections: "+e.getMessage());
     System.exit(1);
    }
}

boolean loadAll() {
      for (Enumeration allEngStrings=engineStrings.elements();
                       allEngStrings.hasMoreElements();)
          if (!loadEngine((String)allEngStrings.nextElement())) return false;
      for (Enumeration allLibStrings=libraryStrings.elements();
                       allLibStrings.hasMoreElements();)
          if (!loadLibrary((String)allLibStrings.nextElement())) return false;
      for (Enumeration allAdptStrings=adapterStrings.elements();
                       allAdptStrings.hasMoreElements();)
          if (!loadAdapter((String)allAdptStrings.nextElement())) return false;
      return true;
}

boolean loadPreConfiguredConductSets() {
    boolean anyLoaded=false;
    for (Enumeration allConductSets=conductSets.elements();
            allConductSets.hasMoreElements();) {
            StringTokenizer tokens=new StringTokenizer((String)allConductSets.nextElement());
                               // engine handle for engine name
            if (setupLoadConductSet((IAEngineHandle)engines.get(tokens.nextToken()),
                           tokens.nextToken("\n")))
                  anyLoaded=true;
        }
   if (anyLoaded) loadcs=true;
   return true;
}

boolean setupLoadConductSet(IAEngineHandle eHandle, String parms) {

   String rest=parms.trim();
   StringTokenizer csParms=new StringTokenizer(rest);
   String csname;
   String selector;
   StringBuffer result=new StringBuffer();
   int endQuote;
   // get csname
   if (csParms.countTokens() < 1 ) {
       outputError("Unload ConductSet Error, no conductset name");
       return false;
   }
   if (rest.charAt(0)!='"') {                  // fast path
      csname=csParms.nextToken();
      if (csParms.hasMoreTokens()) selector=csParms.nextToken("\n");
      else selector="";
      }
   else {
     result=new StringBuffer();
     endQuote=indexOfQuote(rest.substring(1),result)+1;
     if (endQuote < 1) {
        outputError("Missing trailing quote for conductset name : "+
                     rest);
        return false;
     }
     csname=new String(result);
     if (rest.charAt(endQuote+1)!=' ') {
        outputError("Load conduct set Error: "+
                    "Invalid character \"" + rest.charAt(endQuote+1) +
                     "\" following quoted string");
        return false;
     }

     if (rest.length() > endQuote)
        selector=rest.substring(endQuote+1).trim();
     else selector="";
   }
   if (selector.length() > 0) {
      csParms=new StringTokenizer(selector);
      if (selector.charAt(0)!='"') {
         if (csParms.countTokens() > 1)  {
            outputError("Load conduct set Error: "+
                        "Selectors with blanks must be enclosed in quotes. "+
                        "Invalid selector: "+selector);
            return false;
         }
      }
      else {
         result=new StringBuffer();
         endQuote=indexOfQuote(selector.substring(1),result)+1;
         if (endQuote < 1) {
            outputError("Load conduct set Error: "+
                        "Missing trailing quote for selector "+
                        "Invalid selector: "+selector);
            return false;
         }
         if (selector.length() > endQuote +1 &&
            !(selector.substring(endQuote+1).trim().equals(""))) {
            System.out.println("Load conduct set Error: Extraneous string "+
                   "after final quote: "+selector.substring(endQuote+1));
            return false;
         }
         selector=new String(result);
      }
    }
  try {
//IAAgent loadConductSet
     loadConductSet((IALibraryHandle)libraries.elementAt(0),eHandle,
                    csname,selector);
   }
   catch (IAError e) {
      outputError("loadConductSet name= "+csname+ " "+
                         e.getMessage());
      return false;
   }
   return true;
}

boolean setupUnloadConductSet(IAEngineHandle eHandle, String parms) {
   String rest=parms.trim();
   StringTokenizer csParms=new StringTokenizer(rest);
   String csname;
   String selector;
   StringBuffer result=new StringBuffer();
   int endQuote;
   // get csname
   if (csParms.countTokens() < 1 ) {
       outputError("Unload ConductSet Error, no conductset name");
       return false;
   }
   if (rest.charAt(0)!='"') {                  // fast path
      csname=csParms.nextToken();
      if (csParms.hasMoreTokens()) selector=csParms.nextToken("\n");
      else selector="";
      }
   else {
     result=new StringBuffer();
     endQuote=indexOfQuote(rest.substring(1),result)+1;
     if (endQuote < 1) {
        outputError("Missing trailing quote for conductset name");
        return false;
     }
     csname=new String(result);
     if (rest.charAt(endQuote+1)!=' ') {
        outputError("unLoad conduct set Error: "+
                    "Invalid character \"" + rest.charAt(endQuote+1) +
                     "\" following quoted string");
        return false;
     }
     if (rest.length() > endQuote)
        selector=rest.substring(endQuote+1).trim();
     else selector="";
   }
   if (selector.length() > 0) {
      csParms=new StringTokenizer(selector);
      if (selector.charAt(0)!='"') {
         if (csParms.countTokens() > 1) {
            outputError("Unload conduct set Error: "+
                        "Selectors with blanks must be enclosed in quotes");
            return false;
         }
      }
      else {
         result=new StringBuffer();
         endQuote=indexOfQuote(selector.substring(1),result)+1;
         if (endQuote < 1) {
            outputError("unLoad conduct set Error: "+
                        "Missing trailing quote for selector");
            return false;
         }
         if (selector.length() > endQuote +1 &&
            !selector.substring(endQuote+1).trim().equals("")) return false;
         selector=new String(result);
      }
    }
   try {
//IAAgent unloadConductSet
     unloadConductSet((IALibraryHandle)libraries.elementAt(0),eHandle,
                    csname,selector);
   }
   catch (IAError e) {
      outputError("unloadConductSet name= "+csname+ " "+
                         e.getMessage());
      return false;
   }
   return true;
}

boolean setupReset(String input) {
   String selector=new String(input);
   StringBuffer result=new StringBuffer();
   if (selector.length() > 0) {
      StringTokenizer csParms=new StringTokenizer(selector);
      if (selector.charAt(0)!='"') {
         if (csParms.countTokens() > 1) {
            outputError("reset Error: "+
                        "Selectors with blanks must be enclosed in quotes");
            return false;
         }
      }
      else {
         result=new StringBuffer();
         int endQuote=indexOfQuote(selector.substring(1),result)+1;
         if (endQuote < 1) {
            outputError("Reset set Error: "+
                        "Missing trailing quote for selector");
            return false;
         }
         if (selector.length() > endQuote +1 &&
            !selector.substring(endQuote+1).trim().equals("")) return false;
         selector=new String(result);
      }
    }
   try {
//IAAgent reset
     reset(selector);
   }
   catch (IAError e) {
      outputError("reset selector= "+selector+ " "+
                         e.getMessage());
      return false;
   }
   return true;
}

int indexOfQuote(String parm, StringBuffer result) {
   int start=0;
   int pos;
   while ((pos=parm.indexOf('"',start)) >= 0) {
      result.append(parm.substring(start,pos));
      if (parm.length()==pos+1) break;   // end quote
      else if (parm.charAt(pos+1)=='"') {
         // append one quote
         result.append(parm.charAt(pos+1));
         if (parm.length() < pos+3) return -1;   // no end quote
         start=pos+2;
      }
      else break;
   }
   return pos;
}

boolean parseConfigFile() {
    String line;
    StringTokenizer tokens;
    String currentToken=new String();
    String selector;
    StringTokenizer temp;
    int levels;
    IAEngineHandle eHandle;
    IALibraryHandle lHandle;
    IAAdapterHandle aHandle;

    try {

      BufferedReader input = new BufferedReader(new FileReader(filename));
      boolean endConfig=false;

       while (input.ready() && !endConfig) {
           line=input.readLine();
           tokens=new StringTokenizer(line);
           // get keyword
           if (tokens.hasMoreTokens())
              currentToken=tokens.nextToken().toLowerCase();

           // ignore blank lines and comments
           if (line.trim().length()==0 || line.charAt(0)=='*' ||
               line.charAt(0)=='#') {
           // go to next line
           }

           else if (currentToken.equals("adapter")) {
                 if (tokens.countTokens() < 2 ) {
                    configError(line," adapter statement format is:  "+
                         "adapter domain dllname [parms]");
                    return false;
                 }
                adapterStrings.addElement(tokens.nextToken("\n"));
           }

           else if (currentToken.equals("engine")) {
                if (tokens.countTokens() < 2 ) {
                    configError(line," engine statement format is:  "+
                            "engine type dllname [parms]");
                    return false;
                }
                engineStrings.addElement(tokens.nextToken("\n"));
           }

           else if (currentToken.equals("library")) {
               if (tokens.countTokens() < 4 || tokens.countTokens() > 5) {
                   configError(line," library statement format is:  "+
                          "library libtype dllnam map collector levels");
                  return false;
               }
               libraryStrings.addElement(tokens.nextToken("\n"));
           }

           else if (currentToken.equals("conductset")) {
               if (tokens.countTokens() < 2 )  {
                  configError(line," conductset statement format is:  "+
                         "conductset engine-type conductset-name [selector]");
                  return false;
               }
               // parse them later when loading
               conductSets.addElement(tokens.nextToken("\n"));
           }

          else if (currentToken.equals("run"))
             break;
          else {
             configError(line,"Unrecognized keyword \""+currentToken+ "\"");
             return false;
          }
       }  // while
    }
    catch (FileNotFoundException e) {
       outputError("Configuration file \""+e.getMessage() +"\" not found.");
       return false;
    }
    catch (Exception e) {
      outputError("Add component exception "+e.getMessage());
      e.printStackTrace();
      return false;
    }

   // Make sure we have one library, one engine, and one adapter
   if (libraryStrings.size() < 1 ) {
       outputError("Error: No libraries are configured.");
       return false;
   }
   else if (engineStrings.size() < 1 ) {
       outputError("Error: No engines are configured.");
       return false;
   }
   else if (adapterStrings.size() < 1 ) {
       outputError("Error: No adapters are configured.");
       return false;
   }
   else return true;
}


boolean loadAdapter(String adapterInfo) {

   StringTokenizer tokens=new StringTokenizer(adapterInfo);
   // next token is adapter domain
   String domain=tokens.nextToken();

   // next token is adapter module name
   String moduleName=tokens.nextToken();

   // next token is parms for the adapter (if they exist)
   String parms;
   if (tokens.hasMoreTokens())
       parms=tokens.nextToken("\n");
   else parms=new String("");

   // add the adapter
   IAAdapterHandle aHandle=new IAAdapterHandle();

//IAAgent addAdapter
   try {
      addAdapter(domain,moduleName,parms,aHandle);
      adapters.addElement(aHandle);
   }
   catch (IAError e) {
      outputError("Add adapters error: "+e.getMessage());
      return false;
   }
   return true;
}

boolean loadEngine(String engineInfo) {

   StringTokenizer tokens=new StringTokenizer(engineInfo);
   // next token is engine domain
   String type=tokens.nextToken();

   // next token is engine module name
   String moduleName=tokens.nextToken();

   // next token is parms for the engine (if they exist)
   String parms;
   if (tokens.hasMoreTokens())
       parms=tokens.nextToken();
   else parms=new String("");

   // add the engine
   IAEngineHandle eHandle=new IAEngineHandle();

   // need to save first engine handle for interactive load conduct sets
   if (engines.isEmpty()) firstEngine=eHandle;

//IAAgent addEngine
    try {
       addEngine(type,moduleName,parms,eHandle);
       engines.put(type,eHandle);
    }
    catch (IAError e) {
      outputError("Add engines error: "+e.getMessage());
      return false;
    }
    return true;
}

boolean loadLibrary(String libraryInfo) {
   StringTokenizer tokens=new StringTokenizer(libraryInfo);

   // next token is library type
   String type=tokens.nextToken();

   // next token is library module name
   String moduleName=tokens.nextToken();

   // next token is library map
   String map=tokens.nextToken();
   if (map.charAt(map.length()-1)!=File.separatorChar)
       map=new String(map+File.separator);

   // next token is library collector
   String collector=tokens.nextToken();

   // next token is library levels
   int levels;
     if (tokens.hasMoreTokens()) {
        String intString=tokens.nextToken();
        try {
           levels=Integer.parseInt(intString);
        }
       catch (NumberFormatException e) {
            outputError("Error in library levels. Level= "+intString+".  This "+
                  "must be an integer Value");
           return false;
       }
     }
     else levels=2;
   // add the library
   IALibraryHandle lHandle=new IALibraryHandle();

//IAAgent addLibrary
   try {
      addLibrary(map,collector,moduleName,type,levels,lHandle);
      libraries.addElement(lHandle);
   }
   catch (IAError e) {
      outputError("Add library error: "+e.getMessage());
      return false;
   }
   return true;
}

void configError(String line,String message) {
   outputError("Error on line: "+line+"\n"+message);
}

// Wrappers for messages to user
void outputError(String msg) { System.err.println(msg); }

void outputInfo(String msg) { System.out.println(msg); }

} // end class JavaAgent

class StatsMonitor extends Thread {
   private long waitTime;
   private JavaAgent agent;

   StatsMonitor(JavaAgent theAgent)
     {
       agent=theAgent;
       waitTime=3*1000;
     }

public void run() {
// waitForStopAgent will never return true in this implementation
// because we are running in daemon mode until a termination signal and
// stopAgent won't ever be called in daemon mode
     while (!agent.waitForStopAgent(waitTime)) {
       agent.outputInfo("Processing " + agent.getStats() +
                               " events per second");
     }
   }

}
