/*************************************************************************
** File:         RemClient.java
** Description:  Client Driver for Sample Reminder Agent
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/

/* RemClient.java
 * This class provides a command line input to the RemAgent socket interface
*/
package ibm.AgentBuilder.samples.ReminderAgent;
import java.net.*;
import java.io.*;
import java.util.*;

public class RemClient {
   public static boolean debug = true;
   private static String IPAddr;

   public static void main(String args[]) {
      BufferedReader  cmdin = new BufferedReader(new InputStreamReader(System.in));
      String command = null;

      if (args.length == 1)
         IPAddr = args[0];  //set IP address from input argument
      else {
         System.out.println("Exception: IP address required as input argument");
         System.exit(1);
        }

    try {
      System.out.println(" Enter command to send to server (QUIT to leave, ? for list of cmds): ");
      while ((command = cmdin.readLine()) != null) {
         StringTokenizer st = new StringTokenizer(command, " ");
         String cmd = st.nextToken().toUpperCase();
         if (cmd.equals("QUIT")) break;
         else if (cmd.equals("?")) help();
         else send(command);
         }
      }
      catch (Exception e) {
         fail(e, "IOException sending data to server");
      }
   }

   public static void fail(Exception e, String msg) {
      System.err.println(msg + ": " + e);
      System.exit(1);
   }

   public static void send(String socketData) {
    Socket client_socket;
    PrintWriter out;

    try {
       client_socket = new Socket(IPAddr, 6789);  // Connect, providing IP addr and port

       out = new PrintWriter(client_socket.getOutputStream(), true); // Get output stream
       if (debug)  System.out.println("About to write data to socket:" + socketData );
       out.println(socketData);                                // Write data
       receive(client_socket);                                 // invoke recieve to get response
       out.close();                                            // Close DataOutputStream
       if (debug)  System.out.println("Closed DataOutputStream");
      }
     catch (Exception e) {
         fail(e, "Exception sending data to server");
      }
   if (debug)  System.out.println("Exiting Send routine");
   }

   public static void receive(Socket client_socket) {
     BufferedReader in;
     String line;

     try {
       in = new BufferedReader(new InputStreamReader(client_socket.getInputStream())); // Get input stream
       if (debug)  System.out.println("About to receive data from socket");
       while ((line = in.readLine()) != null) {                  // receive on input stream
          System.out.println("Received following response: " + line);// print what was received
       }
       if (debug)  System.out.println("End of receiving responses from server");
       in.close();                                               // close input stream
       if (debug)  System.out.println("Closed BufferedReader");
     }
       catch (Exception e) {
        fail(e, "Exception receiving response data from server");
       }
   }

   public static void help() {
      System.out.println("QUIT                            - to terminate the client program");
      System.out.println("Get:UserID                      - to retrieve all reminders for user");
      System.out.println("Put:UserID:RemString            - to save reminders ");
      System.out.println("Remove:UserID:RemID             - to remove a reminder");
      System.out.println("Modify:UserID:RemString         - to replace a reminder ");
      System.out.println("GetMsgs:UserID                  - to get all messages for user");
      System.out.println("                                  and mark them as read");
      System.out.println("GetMsgsNoMark:UserID            - to get all message for user");
      System.out.println("                                  but NOT mark them as read");
      System.out.println("DelMsgs:UserID:MsgID,MsgID,...  - to delete specified messages for user");
      System.out.println("MarkMsgsAsNew:UserID:MsgID,...  - to mark specified user's messages as unread");
      System.out.println("StatusMsgs:UserID               - to query if any new messages exist for user");
      System.out.println("where RemString is of the format: ID$nnn$tokenStr");
      System.out.println("       ID       is a keyword");
      System.out.println("       nnn      is the reminder id (use 000 for put, else use ");
      System.out.println("                the reminder id that is returned by the Get command)");
      System.out.println("       tokenStr is of the format: key=value#key=value#...");
      System.out.println("         key    is for documentation of positional data, and");
      System.out.println("         value  is positional data, as follows:");
      System.out.println("           typeOfEvent=Birthday#nameOfEvent=Sam's Birthday#month=July#day=4");
      System.out.println("           #year=1997#count=5#via=E-Mail#user=Abe#emailAddr=user@address.com");
      System.out.println("        (via=log or E-Mail, for log you must explicitly go to the reminder)");
      System.out.println("        (  client and getMsgs to retrieve reminder messages.)");
      System.out.println("        (count is number of days BEFORE event date (when notifications begin))");
      }

}
