/*************************************************************************
** File:         Message.java
** Description:  Reminder Message Class used by RemAgent - sample agent for reminders
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
// Description: The Reminder Message Class
//
//-----------------------------------------------------------------------------
package ibm.AgentBuilder.samples.ReminderAgent;
import java.util.*;

public class RemMessage
{
    //-------------------------------------------------------------------------
    // Instance variables
    //-------------------------------------------------------------------------
  private char    theStatus;         // Status of the message
  private String  theID;             // Unique ID of the message
  private String  theDate;           // The date the message was created
  private String  theTime;           // The time the message was created
  private String  theTimeZone;       // The time zone the message was created
  private String  theMessage;        // The message!
  private boolean debug = false;     // Debug variable

    //-------------------------------------------------------------------------
    // Constructor  (default)
    //-------------------------------------------------------------------------
  public RemMessage( )
    {

    }

    //-------------------------------------------------------------------------
    // Constructor from aLine
    //
    // Description: Does no error checking, assumes line was created by print()
    //-------------------------------------------------------------------------
  public RemMessage( String aLine )
    {
    char status;
    String    msgID, date, time, timeZone, message, blank = " ";

    int start= 2;
    int end = 0;
    if ( debug )
      System.out.println( "    The line is      : " + aLine );

    status = aLine.charAt( end );

    end  = aLine.indexOf( blank, start );
    msgID = aLine.substring( start, end);

    start = ++end;
    end  = aLine.indexOf( blank, start );
    date = aLine.substring( start, end );

    start = ++end;
    end  = aLine.indexOf( blank, start );
    time = aLine.substring( start, end );

    start = ++end;
    end  = aLine.indexOf( blank, start );
    timeZone = aLine.substring( start, end );

    start = ++end;
    message = aLine.substring( start );

    if ( debug )
      {
      System.out.println( "    The line is      : " + aLine );
      System.out.println( "    The status is    : " + status );
      System.out.println( "    The ID is        : " + msgID );
      System.out.println( "    The date is      : " + date );
      System.out.println( "    The time is      : " + time );
      System.out.println( "    The time zone is : " + timeZone );
      System.out.println( "    The message is   : " + message );
      }

    theStatus   = status;
    theID       = new String( msgID );
    theDate     = new String( date );
    theTime     = new String( time );
    theTimeZone = new String( timeZone );
    theMessage  = new String( message );

    }

    //-------------------------------------------------------------------------
    // Method: print()
    //-------------------------------------------------------------------------
  public String print( )
    {
    String aString = new String( theStatus   + " " +
                                 theID       + " " +
                                 theDate     + " " +
                                 theTime     + " " +
                                 theTimeZone + " " +
                                 theMessage  + "\n" );
    return( aString );
    }

    //-------------------------------------------------------------------------
    // Method: setStatus( char )
    //-------------------------------------------------------------------------
  public void setStatus( char aStatus )
    {
    if ( aStatus == 'n' )
      theStatus  = aStatus;
    else
      theStatus = 'o';
    }

    //-------------------------------------------------------------------------
    // Method: status( )
    //-----------------------------------------------------------------------
  public char status( )
    {
    return( theStatus );
    }

    //-------------------------------------------------------------------------
    // Method: msgID( ) - return the message ID
    //-------------------------------------------------------------------------
  public String msgID( )
    {
    return( theID );
    }

    //-------------------------------------------------------------------------
    // Method: markRead()
    //-------------------------------------------------------------------------
  public void markRead( )
    {
    theStatus = 'o';
    }

    //-------------------------------------------------------------------------
    // Method: setDate()
    //-------------------------------------------------------------------------
  public void setDate( )
    {
    GregorianCalendar today = new GregorianCalendar();
    String month, day;
    Integer year;

    month = monthToString( today.get(Calendar.MONTH) );
    day   = dayToString( today.get(Calendar.DAY_OF_MONTH) );
    year  = new Integer( today.get(Calendar.YEAR));

    theDate = month + "/" + day + "/" + year.toString();
    theTime = hourToString( today.get(Calendar.HOUR_OF_DAY) ) + ":" +
              minuteToString( today.get(Calendar.MINUTE) );
    theTimeZone = "EDT";

    }

    //-------------------------------------------------------------------------
    // Method: monthToString()
    // Convert month (int 0-11) to string ("01 - "12")
    //-------------------------------------------------------------------------
  String monthToString(int m) {
    switch (m) {
      case 0: return "01";
      case 1: return "02";
      case 2: return "03";
      case 3: return "04";
      case 4: return "05";
      case 5: return "06";
      case 6: return "07";
      case 7: return "08";
      case 8: return "09";
      case 9: return "10";
      case 10: return "11";
      case 11: return "12";
      default: return "01";
      }
   }

    //-------------------------------------------------------------------------
    // Method: dayToString()
    // Convert day (int 0-31) to string ("01" - "31")
    //-------------------------------------------------------------------------
  String dayToString(int m) {
    switch (m) {
      case 1: return "01";
      case 2: return "02";
      case 3: return "03";
      case 4: return "04";
      case 5: return "05";
      case 6: return "06";
      case 7: return "07";
      case 8: return "08";
      case 9: return "09";
      case 10: return "10";
      case 11: return "11";
      case 12: return "12";
      case 13: return "13";
      case 14: return "14";
      case 15: return "15";
      case 16: return "16";
      case 17: return "17";
      case 18: return "18";
      case 19: return "19";
      case 20: return "20";
      case 21: return "21";
      case 22: return "22";
      case 23: return "23";
      case 24: return "24";
      case 25: return "25";
      case 26: return "26";
      case 27: return "27";
      case 28: return "28";
      case 29: return "29";
      case 30: return "30";
      case 31: return "31";
      default: return "01";
      }
   }

    //-------------------------------------------------------------------------
    // Method: hourToString()
    // Convert day (int 0-31) to string ("01" - "31")
    //-------------------------------------------------------------------------
  String hourToString(int m) {
    switch (m) {
      case 0: return "00";
      case 1: return "01";
      case 2: return "02";
      case 3: return "03";
      case 4: return "04";
      case 5: return "05";
      case 6: return "06";
      case 7: return "07";
      case 8: return "08";
      case 9: return "09";
      case 10: return "10";
      case 11: return "11";
      case 12: return "12";
      case 13: return "13";
      case 14: return "14";
      case 15: return "15";
      case 16: return "16";
      case 17: return "17";
      case 18: return "18";
      case 19: return "19";
      case 20: return "20";
      case 21: return "21";
      case 22: return "22";
      case 23: return "23";
      default: return "00";
      }
   }

    //-------------------------------------------------------------------------
    // Method: minuteToString()
    // Convert day (int 0-31) to string ("01" - "31")
    //-------------------------------------------------------------------------
  String minuteToString(int m) {
    switch (m) {
      case 0: return "00";
      case 1: return "01";
      case 2: return "02";
      case 3: return "03";
      case 4: return "04";
      case 5: return "05";
      case 6: return "06";
      case 7: return "07";
      case 8: return "08";
      case 9: return "09";
      case 10: return "10";
      case 11: return "11";
      case 12: return "12";
      case 13: return "13";
      case 14: return "14";
      case 15: return "15";
      case 16: return "16";
      case 17: return "17";
      case 18: return "18";
      case 19: return "19";
      case 20: return "20";
      case 21: return "21";
      case 22: return "22";
      case 23: return "23";
      case 24: return "24";
      case 25: return "25";
      case 26: return "26";
      case 27: return "27";
      case 28: return "28";
      case 29: return "29";
      case 30: return "30";
      case 31: return "31";
      case 32: return "32";
      case 33: return "33";
      case 34: return "34";
      case 35: return "35";
      case 36: return "36";
      case 37: return "37";
      case 38: return "38";
      case 39: return "39";
      case 40: return "40";
      case 41: return "41";
      case 42: return "42";
      case 43: return "43";
      case 44: return "44";
      case 45: return "45";
      case 46: return "46";
      case 47: return "47";
      case 48: return "48";
      case 49: return "49";
      case 50: return "50";
      case 51: return "51";
      case 52: return "52";
      case 53: return "53";
      case 54: return "54";
      case 55: return "55";
      case 56: return "56";
      case 57: return "57";
      case 58: return "58";
      case 59: return "59";
      default: return "00";
      }
   }

}
