// -----------------------------------------------------------------------
// (C) Copyright IBM Corp, 1996
//
// File:        MonitorThread.java
// Version:     1.0 (sample) 6 August 1996
// Description: This class contains code designed to run as a thread
//              when a monitor event is scheduled.
//
// DISCLAIMER OF WARRANTIES:
// -------------------------
// The following [enclosed] code is sample code created by IBM
// Corporation.  This sample code is not part of any standard IBM product
// and is provided to you solely for the purpose of assisting you in the
// development of your applications.  The code is provided "AS IS",
// without warranty of any kind.  IBM shall not be liable for any damages
// arising out of your use of the sample code, even if they have been
// advised of the possibility of such damages.
// -----------------------------------------------------------------------
package ibm.AgentBuilder.samples.stock;

import ibm.AgentBuilder.Adapter.*;
import ibm.AgentBuilder.aeapi.*;

import ibm.AgentBuilder.samples.stock.*;
// import MonitorSC;
// import ParseSC;
// import StockAdapter;

// As needed
import java.util.Properties;
import java.util.StringTokenizer;

public class MonitorThread implements Runnable {


   // -----------------------------------------------------
   // Constructor()
   // -----------------------------------------------------
   public MonitorThread(StockAdapter iStockAdapter,
                        MonitorSC    iMonitorObj,
                        ParseSC      iParseObj,
                        String       iStockSymbols ) {

      stockAdapter = iStockAdapter;
      parseObj = iParseObj;
      monitorObj = iMonitorObj;
      stockSymbols = iStockSymbols;

      thisThread = new Thread(this);
   }

   // -----------------------------------------------------
   // Start()
   // -----------------------------------------------------
   public void start() {
      thisThread.start();
   }

   // -----------------------------------------------------
   // Our Run method
   // -----------------------------------------------------
   public void run() {

      String      key, keyValue;
      IAAtom      myAtom = null;
      IAFactSet   myFactSet = null;
      Properties  listOfFacts = null;

      // Connect to our intended source of information.
      // This should be indicated in the URL.
      // Obtain our page of HTML data.
      String rawHTML = monitorObj.startMonitor(stockSymbols);

      // Parse the data by passing it to our parseObj.
      try {
         listOfFacts = parseObj.parseHTML(stockSymbols, rawHTML);
      } catch (Exception e) {
         System.out.println("MonitorThread(): Exception while parsing. " + e.getMessage());
         Thread ct = Thread.currentThread();
         ct.stop();
      }

      // Create a notification event if we were able to parse the data

      StringTokenizer st = new StringTokenizer(stockSymbols);

      // create a fact set to use for the TriggerEvent
      try {
         myFactSet = new IAFactSet();
      } catch (Exception e) {
         System.out.println("MonitorThread(): Exception while creating fact. " + e.getMessage());
         Thread ct = Thread.currentThread();
         ct.stop();
      }

      // create an atom to use to build each fact
      try {
         myAtom = new IAAtom();
         myAtom.setPredicate("StockPrice");

        // add each stock symbol and quote as a fact to our fact set to be
        // passed on the TriggerEvent
        while(st.hasMoreTokens()) {

           // Obtain the key/value to add to our atom.
           key = st.nextToken();
           keyValue = listOfFacts.getProperty(key);

           // Clear any current atom terms.
           myAtom.setTerms("");


           // Build our atom.
           myAtom.addTerm(key);
           myAtom.addTerm(keyValue);

           // Add it to the current fact set.
           myFactSet.add(myAtom);

        }
      } catch (Exception e) {
         System.out.println("MonitorThread(): Exception while creating atom. " + e.getMessage());
         Thread ct = Thread.currentThread();
         ct.stop();
      }

      // now notify the agent of our StockPriceEvent
      try {
         IATriggerEvent stockPriceEvent = new IATriggerEvent();
         stockPriceEvent.setType("StockPriceEvent");
         stockPriceEvent.setFactSet(myFactSet);
         stockAdapter.notify(stockPriceEvent);
      } catch (Exception e) {
         // if StockAdapter is now not started, an exception on notify is expected
         if (stockAdapter.isStarted())
           {
             System.out.println("MonitorThread(): Exception while performing notify. " + e.getMessage());
           }
      }

   }

   // -----------------------------------------------------
   // Java Class Private Data Area:
   // -----------------------------------------------------


   // Our monitor, parser and adapter objects.
   private String         stockSymbols;
   private MonitorSC      monitorObj;
   private ParseSC        parseObj;
   private StockAdapter   stockAdapter;
   private Thread         thisThread;

}  // end of java class MonitorThread



