// -----------------------------------------------------------------------
// (C) Copyright IBM Corp, 1996
//
// File:        ParseQuoteYahooCom.java
// Version:     1.0 (sample) 6 August 1996
//
// Description: This takes as input a string of stock ticker symbols separated
//               by white space and a raw HTML page in a very specific
//               format and find stock prices for each ticker symbol if
//               available and returns properties object with each entry
//               containing a ticker symbol and price as a string in
//               decimal format ( ddd.ddd )
//
// DISCLAIMER OF WARRANTIES:
// -------------------------
// The following [enclosed] code is sample code created by IBM
// Corporation.  This sample code is not part of any standard IBM product
// and is provided to you solely for the purpose of assisting you in the
// development of your applications.  The code is provided "AS IS",
// without warranty of any kind.  IBM shall not be liable for any damages
// arising out of your use of the sample code, even if they have been
// advised of the possibility of such damages.
// -----------------------------------------------------------------------
package ibm.AgentBuilder.samples.stock;

import java.util.*;

class StockPriceException extends Exception {
   StockPriceException() {super();}
   StockPriceException(String s) {super(s);}
   }

public class ParseQuoteYahooCom extends ParseSC {

  Properties  parseHTML(String companyTokens, String rawHTML) {

    Properties quoteList=new Properties();
    String parseString=new String(rawHTML);
    // tokenize strings of stock ticker symbols
    StringTokenizer stocks=new StringTokenizer(companyTokens);
    String tokens[] = new String[stocks.countTokens()];
    int       currentIndex;
    int       beginTableIndex;
    int       nextIndex;
    int       endTableIndex;
    int       priceIndex;
    int       endPriceIndex;
    int       numeratorIndex=-1;
    int       denominatorIndex=-1;
    int       endNumeratorIndex=-1;
    int       endDenominatorIndex=-1;
    int       endFractionIndex;
    int       endTableRowIndex;
    int       badSymbolIndex;
    String    quotes="Quotes";
    String    test="Here are my Quotes";
    String   nextItem = "<";
    boolean   hasFraction;

    //--------------- Validate that we got the page successfully ------------//
    if (parseString.equals("")) return quoteList;

    StringTokenizer hdr = new StringTokenizer(parseString);
    int rsp = 200;           // Assume OK response

    // If response header received, extract response code
    try {
       if (hdr.nextToken().substring(0,5).toUpperCase().equals("HTTP/"))
          rsp = Integer.parseInt(hdr.nextToken());
    } catch (Exception e) {}

    hdr = null;  // allow garbage collection, html page could be very large

    // If Not OK response, issue message and exit
    if (rsp != 200) {
       System.out.println("StockAdapter Exception: " +
                          "Unable to obtain stock prices, HTTP " +
                          "response code = " + String.valueOf(rsp) + ".");
       return quoteList;
       } // End of error response
    //-------------------- End of validating receipt of page ----------------//

    // Make sure we get to stock quote table - This particular HTML page
    // has U.S Markets

    currentIndex=parseString.indexOf("U.S. Markets");
    if ( currentIndex > -1 ) {
       beginTableIndex=parseString.indexOf("<table",currentIndex);

       // Did we find a table?
       if (beginTableIndex > -1) {
           endTableIndex=parseString.indexOf("</table",currentIndex);

          // go through each stock name in list that was passed to us
          for (int i=0; i < tokens.length; i++) {
              // reinitialize indexes
              priceIndex=-1;   // initialize
              endPriceIndex=-1;  // initialize
              endFractionIndex=-1;
              hasFraction=false;
           // Convert to uppercase as was done in inquiry
              tokens[i]=stocks.nextToken().toUpperCase();
           // Get first instance of ticker symbol
              nextIndex=parseString.indexOf(tokens[i],beginTableIndex);
           // Get end of row
              endTableRowIndex=parseString.indexOf("</tr",nextIndex);
              badSymbolIndex=parseString.indexOf("No such symbol",nextIndex);
              if (badSymbolIndex > -1 && endTableRowIndex > badSymbolIndex)
                 // set it up so we go to next token
                 {nextIndex=-1;}
           // get second occurrence of company string
              if (nextIndex > -1 && nextIndex < endTableIndex ) {
                nextIndex=parseString.indexOf(tokens[i],nextIndex);
               }

           // the price is highlighted
              if (nextIndex > -1 && nextIndex < endTableIndex)  {
                nextIndex=parseString.indexOf("<strong>",nextIndex);
               }
           // get index of price
              if (nextIndex > -1 && nextIndex < endTableIndex)  {
                priceIndex=nextIndex+8;
               }
           // get ending index of price
              if (nextIndex > -1 && nextIndex < endTableIndex)  {
                endPriceIndex=parseString.indexOf(nextItem,priceIndex);
               }
           // need to get fractional portion of the price
              if (endPriceIndex > -1 && endPriceIndex < endTableIndex)
                 {  endFractionIndex=parseString.indexOf("</strong",endPriceIndex); }
           // no fraction found
              if (endFractionIndex==-1 || endFractionIndex==endPriceIndex) {
                 hasFraction=false;
               }
           // fraction found get numerator and denominator tagged as super and
           // subscripts
              else {
                   hasFraction=true;
                   numeratorIndex=parseString.indexOf("<sup>",endPriceIndex)+5;
                   if (numeratorIndex < 4) hasFraction=false;
                   else endNumeratorIndex=parseString.indexOf(nextItem,numeratorIndex);
                   denominatorIndex=parseString.indexOf("<sub>",endPriceIndex)+5;
                   if (denominatorIndex < 4) hasFraction=false;
                   else endDenominatorIndex=parseString.indexOf(nextItem,denominatorIndex);
                 }
           // Now add the price to the Properties
              if (endPriceIndex > -1 && endPriceIndex < endTableIndex)  {
                try {
                    // convertToRealString takes ddd d/d and makes a ddd.ddd
                    // format string
                      if (hasFraction) {
                        quoteList.put(tokens[i],convertToRealString(
                         parseString.substring(priceIndex,endPriceIndex).trim()+" "+
                         parseString.substring(numeratorIndex,endNumeratorIndex).trim()+"/"+
                         parseString.substring(denominatorIndex,endDenominatorIndex).trim()));
                         }
                      else {
                        quoteList.put(tokens[i],parseString.substring(priceIndex,endPriceIndex).trim());
                       }
                    }
                 // Output messages for bad stock prices
                 catch (StockPriceException e)
                    {
                     System.out.println("StockAdapter Exception: " +
                         e.getMessage());
                    }
                 catch (NumberFormatException e)
                    {
                     System.out.println("StockAdapter Exception: " +
                         e.getMessage());
                    }
               }
           } // for
        } // if beginTableIndex
       else  { //  Couldn't find table
          System.out.println("StockAdapter Exception: " +
                             "Unable to find stock quotes, content of page" +
                             " appears to have changed.");
        } // End couldn't find table
     } // if currentIndex
    else { // Couldn't find U.S. Markets
       System.out.println("StockAdapter Exception: " +
                          "Unable to find U.S. Markets quotes, content of " +
                          "page appears to have changed.");
     } // End couldn't find U.S. Markets

    return quoteList;
 }  // ParseHTML


 String  convertToRealString(String price) throws StockPriceException {

        // takes a string in the format ddd d/d and makes a string ddd.ddd
        // 109 1/4 becomes 109.25
        StringTokenizer fracPrice=new StringTokenizer(price);
        String digits="0123456789";
        if (fracPrice.countTokens() < 1 || fracPrice.countTokens() > 2) {
             throw  new StockPriceException("Price \"" + price + "\" does " +
                        "not have the required 2 tokens, it has " +
                        fracPrice.countTokens() + " tokens.");
         }
        String intPart=fracPrice.nextToken();

        // Process integer portion of Stock Price
        // Check for valid number
        for (int i=0;i < intPart.length();i++) {
           if (digits.indexOf(intPart.substring(i,i+1))==-1)  {
            throw new StockPriceException("Price \"" + price + "\" is " +
                        "not numeric.");
            }
          }
         // make a float of left side of decimal point
         float leftSide=Float.valueOf(intPart).floatValue();

         // Do we have a fraction?
         // no, we're all done
         if (!fracPrice.hasMoreTokens()) {
            return String.valueOf(leftSide);
          }
         // Process fractional portion of Stock Price
         else  {
            String fraction=fracPrice.nextToken();
            int divIndex=fraction.indexOf("/");
            String numerator=fraction.substring(0,divIndex);
            String denominator=fraction.substring(divIndex+1);
        // Check for valid number in numerator
         for (int i=0;i < numerator.length();i++) {
             if (digits.indexOf(numerator.substring(i,i+1))==-1)  {
              throw new StockPriceException("Error in numerator: Price= " +
                                             price);
              }
          }
         // make a float of the numerator
         float numer=Float.valueOf(numerator).floatValue();
        // Check for valid number in denominator
         for (int i=0;i < denominator.length();i++) {
             if (digits.indexOf(denominator.substring(i,i+1))==-1)  {
              throw new StockPriceException("Error in denominator: Price= " +
                                             price);
              }
          }
         // make a float of the denominator
         float denom=Float.valueOf(denominator).floatValue();
         return String.valueOf(leftSide+numer/denom);
          }  // else
 } // convertToRealString

}  // ParseSC1
