// -----------------------------------------------------------------------
// (C) Copyright IBM Corp, 1996
//
// File:        QuoteYahooCom.java
// Version:     1.0 (sample) 6 August 1996
//
// Description: This class obtains raw HTML pages from a web site that
//              contains stock price information.  It handles going through
//              a proxy server (if required).  The class is loaded dynamically
//              based on the contents of the ruleset.  It is created and
//              invoked from the monitor thread.
//
// DISCLAIMER OF WARRANTIES:
// -------------------------
// The following [enclosed] code is sample code created by IBM
// Corporation.  This sample code is not part of any standard IBM product
// and is provided to you solely for the purpose of assisting you in the
// development of your applications.  The code is provided "AS IS",
// without warranty of any kind.  IBM shall not be liable for any damages
// arising out of your use of the sample code, even if they have been
// advised of the possibility of such damages.
// -----------------------------------------------------------------------
package ibm.AgentBuilder.samples.stock;

import ibm.AgentBuilder.Adapter.*;

import java.net.*;
import java.util.StringTokenizer;
import java.io.*;

public class QuoteYahooCom extends MonitorSC {

   // Establish target of connect.  Input is from the ruleset.
   public void setURL(String pList) {
      rawURL = pList;
   }

   // Establish proxy...
   public void setProxy(String pList) {
      proxyServer = pList;
   }

   // Establish proxy port...
   public void setProxyPort(int iInt) {
      proxyPort = iInt;
   }

   // Connect and return a buffer full of raw HTML text.
   public String startMonitor(String pList) {

      String              rawHTML = "";

      String              connectServer = "";
      String              connectSelector = "";
      int                 connectPort = 80;

      Socket              mySock;
      PrintWriter         printOutWriter;
      BufferedReader      inStream;

      String              CRLF = "\r\n";

      // Parse rawURL if needed.
      if (webSelector.equals("")) {
         StringTokenizer tok = new StringTokenizer(rawURL,"/");
         webServer = tok.nextToken();
         webSelector = "";
         while ( tok.hasMoreTokens()) {
            webSelector = webSelector + "/" + tok.nextToken();
         }
         if ( webSelector.length() > 0) {
           webSelector = webSelector.substring(1);
         }
      }

      // Proxy server involved?
      if (proxyServer.equals("")) {
         connectServer = webServer;
         connectSelector = "/" + webSelector + "?SYMBOLS=" + pList.replace(' ', '+');
         connectPort = 80;
      } else {
         connectServer = proxyServer;
         connectPort = proxyPort;
         connectSelector = "HTTP://" + webServer + "/" + webSelector +
                                "?SYMBOLS=" + pList.replace(' ', '+');
      }

      // Connect and prepare to read.
      try {
         mySock = new Socket(connectServer, connectPort);
         // Bytes read from or written to the socket are ISO Latin-1
         printOutWriter = new PrintWriter(new OutputStreamWriter(
                                    mySock.getOutputStream(), "8859_1"), true);
         inStream   = new BufferedReader(new InputStreamReader(
                                    mySock.getInputStream(), "8859_1"));
      } catch (Exception e) {
         System.out.println("StockAdapter Exception: " +
                            "Connect to server " + connectServer +
                            ":" + String.valueOf(connectPort) + " - " +
                            e.getMessage());
         return "";
      }

      // Send the http command.
      printOutWriter.println("GET " + connectSelector + " HTTP/1.0" + CRLF +
                               "Accept: */*" + CRLF +
                               "User-Agent: ACTIVIST/000" +
                               CRLF + CRLF);

      // Fill our buffer with the response.
      try {
         String buf;
         rawHTML = "";
         // Read until the socket closes.
         while((buf = inStream.readLine()) != null) {
            rawHTML = rawHTML+buf;
         }

      } catch (Exception e) {
         System.out.println("StockAdapter Exception: " +
                            "Read from server " + connectServer +
                            ":" + String.valueOf(connectPort) + " - " +
                            e.getMessage());
         return "";
      }

      return rawHTML;
   }


   // Private state data.
   String rawURL;

   String proxyServer = "";
   int    proxyPort   = 80;

   String webServer = "";
   int    webPort = 80;
   String webSelector = "";

}  // end of java class QuoteYahooCom
