#ifndef ALARMBIN_H
#define ALARMBIN_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: alarmbin.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/alarmbin.h, tka_timer, abe.l6
// VERSION: 1.11.1.2 97/06/13 11:15:00
//
// INCLUDE AS:  #include <alarmbin.h>
//
// PURPOSE:  Definition of bounded interval alarm
//
// DESCRIPTION:
//    A bounded interval alarm - Interval alarm.  All non-identified
//    interval alarms start on predefined boundaries, starting at midnight
//    Bounded means that an ending time for discontinuing the alarm is
//    given when the alarm is constructed
//
// CLASSES DECLARED:
//    AlarmBoundedInterval
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $A1   960926 WES  f304   Add support for selector in alarms
// $A2   970213 WES  f651   Fix loop when interval 0
// $A3 = 970225 WES f658   Additional mapping for alarms
// $A4 = 970225 WES f664    Persistent alarms
//
//******************************************************************************


#include <alarmint.h>

class PersistentFile;

class AlarmBoundedInterval : public AlarmInterval {
   public:

      AlarmBoundedInterval():AlarmInterval(),                              //@A4
                                       endTime(),remainder(0) {}           //@A2

      // Bounded Interval Alarm ctor & selector                              @A1
      AlarmBoundedInterval (const AString& alarmId,
             Interval thisInterval,                                       //@A1
             const IntervalUnits& units,                                  //@A1
             const AString& startDate,const AString& startTime,           //@A1
             const AString& endDate, const AString& endTime,              //@A1
             const AString& selector)                                     //@A1
          throw(TimeFormatError,DateFormatError,IntervalUnitsError,
                IntervalValueError);                                      //@A3

      static AlarmUniqueId makeUid(const AString& alarmId,
             Interval thisInterval,                                       //@A3
             const IntervalUnits& units,                                  //@A3
             const AString& startDate,const AString& startTime,           //@A3
             const AString& endDate, const AString& endTime,              //@A3
             const AString& selector)                                     //@A3
      // not user-identified                                              //@A3
      { return (alarmId.numWords()==0 ?
                 AlarmUniqueId(selector,'n',(AString)thisInterval+" "+    //@A3
           units+ " "+ startDate+" "+startTime+" "+endDate+" "+endTime) :  //@A3
                AlarmUniqueId(selector,'i',alarmId)); }                   //@A2

      virtual ~AlarmBoundedInterval() {}

      // Alarm Methods

      // override to check for alarm being set after end time
      virtual boolean isExpired() const;

      // Is this a recurring or interval alarm and is the next interval past
      // the current time:
      // time < endTime
      // override needed since end time defined
      virtual boolean isRecurring(IATime currentTime) const;              //@A4

      virtual boolean isRecurring() const
             { IATime currentTime;                                        //@A4
               return isRecurring(currentTime); }                         //@A4

      // need override for remainder
      virtual int incrementAlarm(IATime currentTime);                     //@A2

      // need override for remainder
      virtual int update(IATime currentTime);                             //@A4

      virtual int incrementAlarm()
             { IATime currentTime;                                        //@A2
               return incrementAlarm(currentTime); }                      //@A2

      virtual IAFactSet getFactSet() const throw(IAError);

      // Create a copy of any type of alarm
      virtual Alarm* createCopy() const;

    // utilities possibly used by other alarms (not necessarily subclasses)

    // both 0: current time is end time - current time and date - later make
    // this exception?
    // end time 0 only - current time of given date
    // end date 0 only - todays date

    static IATime makeEndTime(const AString& thisEndDate,                  //@A1
                             const AString& thisEndTime)                  //@A1
          throw(TimeFormatError,DateFormatError);

    virtual AString className() const { return "AlarmBoundedInterval"; }  //@A3

    virtual AString classId() const { return classid; }                   //@A4

    virtual IATime getEndTime() { return endTime; }

    virtual IAAtom getRemainderFact() const throw(IAError);

    // only static variable from persistence is currently persistentVersion //@A4
    static boolean restoreStaticVars(PersistentFile& objectFile);         //@A4

    // only static variable from persistence is currently persistentVersion //@A4
    static boolean saveStaticVars(PersistentFile& objectFile);            //@A4

    //  save and restore
    virtual boolean save(PersistentFile &objFile) const;                  //@A4

    virtual boolean restore(PersistentFile &objFile);                     //@A4


protected:

      virtual void print(ostream& os) const;

      virtual long getRemainder(Interval anInterval ,const IntervalUnits& units);

private:
   IATime endTime;
   long remainder;
   static PersistentObject* alarmDefault;                                 //@A4
   static int version;                                                    //@A4
   static boolean versionAdded;
   static int persistentVersion;                                          //@A4
   static const AString classid;                                          //@A4

};

#endif //_ALARM_BOUNDED_INTERVAL
