#ifndef ALARMINT_H
#define ALARMINT_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: alarmint.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/alarmint.h, tka_timer, abe.l6
// VERSION: 1.11.1.2 97/06/13 11:17:04
//
// INCLUDE AS:  #include <alarmint.h>
//
// PURPOSE:  Definition of AlarmInterval class.
//
// DESCRIPTION:
//    An interval alarm - All non-identified
//    interval alarms start on predefined boundaries, starting at midnight.
//    May specify start time of alarm (used for calculating actual start
//    time as next interval on predefined boundary).
//
// CLASSES DECLARED:
//    AlarmInterval
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $A1   960909 WES  f304   Add support for selector in alarms
// $A2   961206 WES  f512   React to IAAtom and IAFactSet throws
// $A3   970213 WES  f651   Fix loop when interval 0
// $A4 = 970225 WES f658   Additional mapping for alarms
// $A5 = 970312 WES f664    Persistent alarms
//
//******************************************************************************

#include <alarmrec.h>

class IAError;                                                            //@A2

class AlarmInterval : public AlarmRecurring {
   public:

      AlarmInterval():                                                    //@A5
                    AlarmRecurring() {}                                   //@A5

      // Interval Alarm ctor with selector                                   @A1
      AlarmInterval(const AString& alarmId,
                    Interval thisInterval,const IntervalUnits& units,     //@A1
                    const AString& selector)                              //@A1
                    throw(IntervalUnitsError,IntervalValueError);          //@A1

      // constructor with start time
      AlarmInterval(const AString& alarmId,
                    Interval thisInterval,const IntervalUnits& units,     //@A1
              const AString& thisStartDate, const AString& thisStartTime, //@A1
                    const AString& selector)                              //@A1
          throw(TimeFormatError,DateFormatError,IntervalUnitsError,       //@A1
                IntervalValueError);                                      //@A3


      static AlarmUniqueId makeUid(const AString& anAlarmId,
             Interval thisInterval,                                       //@A4
             const IntervalUnits& units,                                  //@A4
             const AString& startDate,const AString& startTime,           //@A4
             const AString& selector)                                     //@A4
                                                                         //@A4
      { return (anAlarmId.numWords()==0 ?
           AlarmUniqueId(selector,'n',                              //@A4
           (AString)thisInterval+" "+                                     //@A4
           units+ " "+ startDate+" "+startTime) :                         //@A4
          AlarmUniqueId(selector,'i',anAlarmId)); }                         //@A2

      static AlarmUniqueId makeUid(const AString& anAlarmId,
             Interval thisInterval,                                       //@A4
             const IntervalUnits& units,                                  //@A4
             const AString& selector)                                     //@A4
      { return (anAlarmId.numWords()==0 ?
           AlarmUniqueId(selector,'n',                                    //@A4
           (AString)thisInterval+" "+units+" 0 0") :                      //@A4
          AlarmUniqueId(selector,'i',anAlarmId)); }                         //@A2

      virtual ~AlarmInterval() {}
      // Alarm Methods

      // IsRecurring same as AlarmRecurring since no end time

      virtual IAFactSet getFactSet() const throw(IAError);                //@A2

      // Create a copy of any type of alarm
      virtual Alarm* createCopy() const;

      virtual AString className() const { return "AlarmInterval";}        //@A4

      virtual AString classId() const { return classid; }                  //@A5

    // only static variable from persistence is currently persistentVersion //@A5
    static boolean restoreStaticVars(PersistentFile& objectFile);         //@A5

    // only static variable from persistence is currently persistentVersion //@A5
    static boolean saveStaticVars(PersistentFile& objectFile);            //@A5

    // no need for save and restore - inherit AlarmRecurring's

protected:

      AlarmInterval(
              Interval thisInterval,const IntervalUnits& units,           //@A1
              const AString& thisStartDate, const AString& thisStartTime, //@A1
              const AlarmUniqueId& alarmuid)                              //@A4
          throw(TimeFormatError,DateFormatError,IntervalUnitsError,       //@A1
                        IntervalValueError);                              //@A3

      // helper function adjust interval start time to proper boundary
      virtual void adjustStart(Interval thisInterval,IntervalUnits units);

      virtual IAAtom getIntervalFact() const throw(IAError);              //@A2

private:
   static PersistentObject* alarmDefault;                                 //@A5
   static const int version;                                              //@A5
   static boolean versionAdded;                                           //@A5
   static int persistentVersion;                                          //@A5
   static const AString classid;                                          //@A5


};
#endif //_ALARM_INTERVAL
