/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: alarmot.cpp
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/alarmot.cpp, tka_timer, abe.l6
// VERSION: 1.11.1.3 97/06/20 15:13:37
//
// PURPOSE:  Implement AlarmOneTime
//
// DESCRIPTION:
//    This class implements an alarm that occurs one time at a specified
//    time and then is cancelled.
//
// CLASSES IMPLEMENTED:
//    AlarmOneTime
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $A1   960909 WES  f304   Add support for selector in alarms
// $A2   960914 WES  f304   react to changes in IAFactSet
// $A3   961209 WES  f512   react to throws in IAFactSet
// $A4 = 970225 WES f658   Additional mapping for alarms
// $A5 = 970312 WES f664   Persistent alarms
//
//******************************************************************************

#include <iapl/common.h>
#include <iatk/iaerrors.h>

static char _partid_[] = "@(#) 1.11.1.3 src/adapters/timer/alarmot.cpp, tka_timer, abe.l6";

#include <alarmot.h>

// 0 indicates persistent Version hasn't been restored yet                //@A5
int AlarmOneTime::persistentVersion=0;                                    //@A5
const int AlarmOneTime::version=1;                                        //@A5
const AString AlarmOneTime::classid="ot";                                 //@A5


// no other static variables need to be saved?                            //@A5
boolean AlarmOneTime::versionAdded=                                       //@A5
                     PersistentObject::addStaticVars("AlarmOneTime",      //@A5
                     &restoreStaticVars,                                  //@A5
                     &saveStaticVars);                                    //@A5


PersistentObject * AlarmOneTime::alarmDefault=                                       //@A5
                      PersistentObject::addObject(classid,                //@A5
                                             new AlarmOneTime());         //@A5

// only static variable from persistence is currently persistentVersion   //@A5
boolean AlarmOneTime::restoreStaticVars(PersistentFile& objectFile)       //@A5
    {                                                                     //@A5
      if (!objectFile.read(persistentVersion)) return false;              //@A5
      return objectFile.readEnd();                                        //@A5
    }                                                                     //@A5

// only static variable from persistence is currently persistentVersion   //@A5
boolean AlarmOneTime::saveStaticVars(PersistentFile& objectFile)          //@A5
    {                                                                     //@A5
      if (!objectFile.write(version)) return false;                       //@A5
      return objectFile.writeEnd();                                       //@A5
    }                                                                     //@A5

// One-time alarm

AlarmOneTime::AlarmOneTime
                       (const AString& aAlarmId,
                       const AString& startDate,
                       const AString& startTime,
                       const AString& selector)
                       throw(DateFormatError,TimeFormatError):
     Alarm(startDate,startTime,
                  makeUid(aAlarmId,startDate,startTime,selector))         //@A2
 {}

// Create a copy of any type of alarm
Alarm* AlarmOneTime::createCopy() const
  {
   return new AlarmOneTime(*this);
  }

// Alarm Methods


IAFactSet AlarmOneTime::getFactSet() const
    {
      IAFactSet OneTimeFacts;
      // standard facts in this fact set
      OneTimeFacts.add(getAlarmTimeFact());                               //@A3
      OneTimeFacts.add(getWeekdayFact());                                 //@A3
      OneTimeFacts.add(getAlarmIdFact());                                 //@A1
      OneTimeFacts.add(getAlarmCategoryFact());
      return OneTimeFacts;
    }


