#ifndef ALARMOT_H
#define ALARMOT_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: alarmot.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/alarmot.h, tka_timer, abe.l6
// VERSION: 1.11.1.2 97/06/13 11:17:09
//
// INCLUDE AS:  #include <alarmot.h>
//
// PURPOSE:  Define AlarmOneTime
//
// DESCRIPTION:
//    This class implements an alarm that occurs one time at a specified
//    time and then is cancelled.
//
// CLASSES DECLARED:
//    AlarmOneTime
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $A1   960909 WES  f304   Add support for selector in alarms
// $A2   961209 WES  f512   React to throws in IAFactSet
// $A3 = 970225 WES f658   Additional mapping for alarms
// $A4 = 970312 WES f664   Persistent alarms
//
//******************************************************************************


#include <alarm.h>

class AlarmOneTime : public Alarm {
   public:

      friend ostream& operator<<(ostream& os, const AlarmOneTime& a) {a.print(os);return os;}

      // default constructor

      AlarmOneTime():Alarm() {}                                           //@A3


      AlarmOneTime(const AString& anAlarmId,
                             const AString& startDate,
                             const AString& startTime,
                             const AString& selector)
                      throw(DateFormatError,TimeFormatError);

      static AlarmUniqueId makeUid(const AString& alarmId,                //@A3
             const AString& startDate,const AString& startTime,           //@A3
             const AString& selector)                                     //@A3
      { return (alarmId.numWords()==0 ?
          AlarmUniqueId(selector,'n',startDate+" "+startTime)  :                        //@A3
          AlarmUniqueId(selector,'i',alarmId)); }                         //@A2

      virtual ~AlarmOneTime() {}

      // Alarm Methods

      // Is this a recurring or interval alarm and is the next interval past the current
      // time < endTime
      boolean isRecurring() const {return false;};

      boolean isRecurring(IATime currentTime) const { return false; }     //@A4

      virtual IAFactSet getFactSet() const throw(IAError);                //@A2

      virtual AString getEventType() const {return "OneTimeAlarm";}

      // Increment to next interval past current time and date
      // Return number of increments skipped
      int incrementAlarm() {return 0;}

      int incrementAlarm(IATime t) {return 0;}                            //@A4

      // Create a copy of any type of alarm
      virtual Alarm* createCopy() const;

      virtual AString className() const { return "AlarmOneTime";}         //@A3

      virtual AString classId() const {return classid; }                  //@A4

      virtual IntervalUnits getIntervalUnits() const {return "onetime";}  //@B2

    // only static variable from persistence is currently persistentVersion //@A4
    static boolean restoreStaticVars(PersistentFile& objectFile);         //@A4

    // only static variable from persistence is currently persistentVersion //@A4
    static boolean saveStaticVars(PersistentFile& objectFile);            //@A4

    // no need for save and restore - inherit alarm's


private:

   static PersistentObject* alarmDefault;                                            //@A4
   static const int version;                                              //@A4
   static boolean versionAdded;                                           //@A4
   static int persistentVersion;                                          //@A4
   static const AString classid;                                          //@A4

};

#endif //_ALARM_ONE_TIME_DEF
