#ifndef ALARMREC_H
#define ALARMREC_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File:  alarmrec.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/alarmrec.h, tka_timer, abe.l6
// VERSION: 1.12.1.2 97/06/13 11:18:18
//
// INCLUDE AS:  #include <alarmrec.h>
//
// PURPOSE:  DEFINITION of AlarmRecurring
//
// DESCRIPTION:
//    An abstract class for alarms that will possibly occur more than once
//
// CLASSES DECLARED:
//    AlarmRecurring
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $A1   960909 WES  f304   Add support for selector in alarms
// $A2   961031 WES  F441   Special case months and years
// $A3   961206 WES  f512   React to IAAtom and IAFactSet throws
// $A4   970213 WES  f651   Fix loop when interval 0
// $A5 = 970226 WES  f658   Additional mapping for alarms
// $A6   970409 WES  f664   Persistent Alarms
//
//******************************************************************************


#include <iapl/common.h>
#include <alarm.h>

class IAError;

class AlarmRecurring : public Alarm {
   public:

      // Recurring alarm
      virtual ~AlarmRecurring() {}

      // Alarm Methods

      // Is this a recurring or interval alarm and is the next interval past the current
      // time < endTime
      // override this if end time defined
      virtual boolean isRecurring() const
             { IATime currentTime;                                        //@A6
               return isRecurring(currentTime); }                         //@A6

      virtual boolean isRecurring(IATime currentTime) const;              //@A6

      // Increment to next interval past current time and date
      // Return number of increments skipped
      // PreCondition: alarm is incrementable not past ending time
      // Currently, this is implemented in this class

      virtual int incrementAlarm()
             { IATime currentTime;                                        //@A6
               return incrementAlarm(currentTime); }                      //@A6

      virtual int incrementAlarm(IATime currentTime);                     //@A6

      virtual int update(IATime currentTime);                             //@A6

      // Get the interval missed fact
      virtual IAAtom getIntervalsMissedFact() const throw(IAError);       //@A3

      virtual IAFactSet getFactSet() const throw(IAError)=0;              //@A3

      // Create a copy of any type of alarm
      virtual Alarm* createCopy() const=0;

      virtual AString className() const { return "AlarmRecurring";}       //@A5

      virtual AString classId() const=0;                                  //@A4

    virtual IntervalUnits getIntervalUnits() const {return theIntervalUnits;} //@B2

    // only static variable from persistence is currently persistentVersion //@A4
    static boolean restoreStaticVars(PersistentFile& objectFile);         //@A4

    // only static variable from persistence is currently persistentVersion //@A4
    static boolean saveStaticVars(PersistentFile& objectFile);            //@A4

    virtual boolean save(PersistentFile &objFile) const;                  //@A4

    virtual boolean restore(PersistentFile &objFile);                     //@A4

protected:
      // ctors

      // basis for default
      AlarmRecurring():Alarm(),                                           //@A6
          aInterval(1),theIntervalUnits("days"),startDay("") {}           //@A6

      AlarmRecurring(Interval thisInterval,const IntervalUnits& units,    //@A1
                     const AlarmUniqueId& anId)                           //@A5
                     throw(IntervalUnitsError,IntervalValueError);        //@A4
      AlarmRecurring(const AString& startDate,const AString& startTime,   //@A1
                   Interval thisInterval, const IntervalUnits& units,     //@A1
                     const AlarmUniqueId& anId)                           //@A5
             throw(DateFormatError,TimeFormatError,IntervalUnitsError,    //@A1
                   IntervalValueError);                                   //@A4

      virtual void print(ostream& os) const;

      // helper functions
      boolean isIntervalEqual(const AlarmRecurring*) const;

      Interval getInterval() const {return aInterval;}

      void setInterval(Interval I)  {aInterval=I;}

      void setIntervalUnits(IntervalUnits u) {theIntervalUnits=u;}

      void setStartDay();                                                 //@A2

      Interval aInterval;
      IntervalUnits theIntervalUnits;

private:
   AString startDay;                                                      //@A2
   static const int version;                                              //@A6
   static boolean versionAdded;                                           //@A6
   static int persistentVersion;                                          //@A6
};

#endif //_ALARM_RECURRING
