/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: alarmsda.cpp
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/alarmsda.cpp, tka_timer, abe.l6
// VERSION: 1.11.1.3 97/06/20 15:13:40
//
// PURPOSE:  implement AlarmSpecificDaily class
//
// DESCRIPTION:
//    A specific daily alarm (days of the week specified).
//    Can be set to reoccur at one specific time on specific days of the week.
//    Generates a OneTimeAlarm event.
//
// CLASSES IMPLEMENTED:
//    AlarmSpecificDaily
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $B1 = 960725 WES d219   Fix allowing day of "7" - caused infinite loop
// $A2 = 960914 WES f304   react to changes in IAFactSet
// $A3   961209 WES f512   React to throws in IAFactSet
// $A4 = 970225 WES f658   Additional mapping for alarms
// $A5 = 970312 WES f664   Persistent alarms
//
//******************************************************************************

#include <iapl/common.h>

static char _partid_[] = "@(#) 1.11.1.3 src/adapters/timer/alarmsda.cpp, tka_timer, abe.l6";

#include <iatk/iaerrors.h>
#include <alarmsda.h>

int AlarmSpecificDaily::persistentVersion=0;                              //@A5
const int AlarmSpecificDaily::version=1;                                  //@A5
const AString AlarmSpecificDaily::classid="sda";                          //@A5


// no other static variables need to be saved?                            //@A5
boolean AlarmSpecificDaily::versionAdded=                                 //@A5
        PersistentObject::addStaticVars("AlarmSpecificDaily",              //@A5
        &restoreStaticVars,                                               //@A5
        &saveStaticVars);                                                 //@A5

PersistentObject * AlarmSpecificDaily::alarmDefault=                                 //@A5
                    PersistentObject::addObject(classid,                  //@A5
                                             new AlarmSpecificDaily());   //@A5

// only static variable from persistence is currently persistentVersion   //@A5
boolean AlarmSpecificDaily::restoreStaticVars(                            //@A5
                                        PersistentFile& objectFile)       //@A5
    {                                                                     //@A5
      if (!objectFile.read(persistentVersion)) return false;              //@A5
      return objectFile.readEnd();                                        //@A5
    }                                                                     //@A5

// only static variable from persistence is currently persistentVersion   //@A5
boolean AlarmSpecificDaily::saveStaticVars(                               //@A5
                                 PersistentFile& objectFile)              //@A5
    {                                                                     //@A5
      if (!objectFile.write(version)) return false;                       //@A5
      return objectFile.writeEnd();                                       //@A5
    }                                                                     //@A5

boolean AlarmSpecificDaily::save(PersistentFile &objFile) const //@A5
  {                                                                       //@A5
    if (!AlarmRecurring::save(objFile)) return false;                     //@A5
    return objFile.write(allWeekDays);                                    //@A5
  }                                                                       //@A5

boolean AlarmSpecificDaily::restore(PersistentFile &objFile)              //@A5
  {                                                                       //@A5
    if (persistentVersion > version) {                                    //@A5
    cout << "AlarmSpecificDaily versions not compatible" <<               //@A5
          endl;                                                           //@A5
    return false;                                                         //@A5
    }                                                                     //@A5
    if (!AlarmRecurring::restore(objFile)) return false;                  //@A5
    return objFile.read(allWeekDays);                                     //@A5
  }                                                                       //@A5

// Specific Daily Alarm ctor - used multiply for daily with specific days

AlarmSpecificDaily::AlarmSpecificDaily (const AString& alarmId,
                                        const AString& startTime,         //@A1
                                        const AString& dayOfWeekString,   //@A1
                                        const AString& aSelector):        //@A1
AlarmRecurring(1,"days",
               makeUid(alarmId,startTime,dayOfWeekString,aSelector)),     //@A4
               allWeekDays(dayOfWeekString)                               //@A1
{                                                                         //@A1
     constructorCommon(startTime,dayOfWeekString);                        //@A1
}                                                                         //@A1

// Specific Daily Alarm ctor for subclasses


void AlarmSpecificDaily::constructorCommon(const AString& startTime,
                                        const AString& dayOfWeekString)
{
 if (dayOfWeekString.indexOfAnyBut("0123456")!=-1)                        //@B1
    {
      AString badWeekdayString;
      badWeekdayString=badWeekdayString+"Weekdays= "+dayOfWeekString;
      ITHROW(WeekdayValueError((char*)badWeekdayString));
    }
 Alarm::setStartTime(startTime);
 // Sets up alarm for proper day of week and if today and time expired, adds
 // 7 days
 AString weekdayString(getEventWeekday());
 if (isExpired() || dayOfWeekString.indexOf(weekdayString)==-1)
    incrementAlarm();
}


// Create a copy of any type of alarm
Alarm* AlarmSpecificDaily::createCopy() const
  {
   return new AlarmSpecificDaily(*this);
  }

// Alarm Methods

// Increment alarm different for daily - 24 hour increment but skip those
// days not used
int AlarmSpecificDaily::incrementAlarm(IATime currentTime)
  {
    int skipped=AlarmRecurring::incrementAlarm(currentTime);              //@A5
    AString weekdayString(getEventWeekday());
    while (allWeekDays.indexOf(weekdayString)==-1)
      {
       setAlarmTime(getAlarmTime().addInterval(1,"days"));
       weekdayString=(AString)(getEventWeekday());
      }
     return skipped;
  }


IAFactSet AlarmSpecificDaily::getFactSet() const
    {
      IAFactSet WeeklyFacts;
      // standard fact set
      WeeklyFacts.add(getAlarmTimeFact());                                //@A3
      WeeklyFacts.add(getWeekdayFact());                                  //@A3
      WeeklyFacts.add(getAlarmIdFact());
      WeeklyFacts.add(getAlarmCategoryFact());
      WeeklyFacts.add(getIntervalsMissedFact());                          //@A3
      return WeeklyFacts;
    }

