#ifndef ALARMSDA_H
#define ALARMSDA_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: alarmsda.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/alarmsda.h, tka_timer, abe.l6
// VERSION: 1.10.1.2 97/06/13 11:18:23
//
// INCLUDE AS:  #include <alarmsda.h>
//
// PURPOSE:  define AlarmSpecificDaily class
//
// DESCRIPTION:
//    A specific daily alarm (days of the week specified).
//    Can be set to reoccur at one specific time on specific days of the week.
//    Generates a OneTimeAlarm event.
//
// CLASSES DECLARED:
//    AlarmSpecificDaily
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $A1   960919 WES  f304   Add support for selector in alarms
// $A2   961209 WES  f512   React to throws in IAFactSet
// $A3 = 970225 WES f658   Additional mapping for alarms
// $A4 = 970312 WES f664   Persistent alarms
//
//******************************************************************************


#include <alarmrec.h>

class AlarmSpecificDaily : public AlarmRecurring {
   public:

      AlarmSpecificDaily():AlarmRecurring(),                             //@A4
                           allWeekDays("0123456") {}                     //@A4

      // Specific Daily Alarm ctor -
      AlarmSpecificDaily(const AString& alarmId,
                         const AString& startTime,                        //@A1
                         const AString& dayOfWeekString,                  //@A1
                         const AString& aSelector)                        //@A1
                   throw(TimeFormatError,WeekdayValueError);              //@A1

      static AlarmUniqueId makeUid(const AString& alarmId,
                         const AString& startTime,                        //@A3
                         const AString& dayOfWeekString,                  //@A3
                         const AString& aSelector)                        //@A3
       // not user-identified                                             //@A3
      { return (alarmId.numWords()==0 ?
                AlarmUniqueId(aSelector,'n',                             //@A3
           startTime+" "+dayOfWeekString) :                              //@A3
                AlarmUniqueId(aSelector,'i',alarmId)); }                 //@A2


      virtual ~AlarmSpecificDaily() {}

      // Alarm Methods

      // IsRecurring same as AlarmRecurring since no end time

      virtual int incrementAlarm(IATime currentTime);                     //@A4

      virtual int incrementAlarm()
             { IATime currentTime;                                        //@A4
               return incrementAlarm(currentTime); }                      //@A4

      virtual IAFactSet getFactSet() const throw(IAError);                //@A2

      virtual AString getEventType() const {return "OneTimeAlarm";}

      // Create a copy of any type of alarm
      virtual Alarm* createCopy() const;

      virtual AString className() const { return "AlarmSpecificDaily";}   //@A3

      virtual AString classId() const { return classid; }                 //@A4

    // only static variable from persistence is currently persistentVersion //@A4
    static boolean restoreStaticVars(PersistentFile& objectFile);         //@A4

    // only static variable from persistence is currently persistentVersion //@A4
    static boolean saveStaticVars(PersistentFile& objectFile);            //@A4

    // save and restore
    virtual boolean save(PersistentFile &objFile) const;                  //@A4

    virtual boolean restore(PersistentFile &objFile);                     //@A4


protected:

      virtual void constructorCommon(const AString& startTime, const AString& dayOfWeekString)
                             throw(TimeFormatError,WeekdayValueError);

private:
   AString allWeekDays;
   static PersistentObject* alarmDefault;                                 //@A4
   static const int version;                                              //@A4
   static boolean versionAdded;                                           //@A4
   static int persistentVersion;                                          //@A4
   static const AString classid;                                          //@A4
};

#endif //_ALARM_SPECIFIC_DAILY
