/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: alarmset.cpp
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/alarmset.cpp, tka_timer, abe.l6
// VERSION: 1.9.1.2 97/06/13 11:18:26
//
// PURPOSE:  Implementation of AlarmSet
//
// DESCRIPTION:
//    A Key Sorted bag of Alarm* (sorted by the time the alarm is to occur
//    next).
//
// CLASSES DECLARED:
//    AlarmSet
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $A1   960909 WES  f304   Add support for selector in alarms
// $A2   970522 WES  f      New reset function for persistence
//
//******************************************************************************

// Includes
#include <iapl/common.h>

static char _partid_[] = "@(#) 1.9.1.2 src/adapters/timer/alarmset.cpp, tka_timer, abe.l6";

#include <alarmset.h>

//class AlarmSet : public IKeySortedBag < Alarm*,IATime >

// set cursor to given alarm - match entire alarm not just key
boolean AlarmSet::findElement(const Alarm* a, ICursor &aCursor)
  {
    if (!setToFirst(aCursor)) return false;
    for (;!(elementAt(aCursor)->matches(a))&& !isLast(aCursor);setToNext(aCursor));
    return elementAt(aCursor)->matches(a);
  }

Alarm* AlarmSet::firstActiveAlarm() const                                 //@A2
  {                                                                       //@A2
    Cursor aCursor(*this);                                                //@A2
    return firstActiveAlarm(aCursor);                                     //@A2
  }

Alarm* AlarmSet::firstActiveAlarm(ICursor& aCursor) const                 //@A2
  {
    if (!setToFirst(aCursor)) return NULL;
    for (;(elementAt(aCursor)->isDeferred() &&                         //@A2
            !isLast(aCursor));                                           //@A1
          setToNext(aCursor));                                           //@A1
    if (elementAt(aCursor)->isDeferred()) return NULL;                   //@A2
    else return elementAt(aCursor);                                      //@A2
  }

// set cursor to given alarm - match entire alarm not just key and router  @A1
boolean AlarmSet::findElement                                              //@A1
                (const AlarmUniqueId& auid, ICursor &aCursor)             //@A1
  {                                                                       //@A1
    if (!setToFirst(aCursor)) return false;                              //@A1
    for (;(!elementAt(aCursor)->matches(auid) &&                         //@A2
            !isLast(aCursor));                                           //@A1
          setToNext(aCursor));                                           //@A1
    return elementAt(aCursor)->matches(auid);                            //@A2
  }                                                                       //@A1

// set cursor to given alarm - match entire alarm not just key and router  @A1
boolean AlarmSet::findMatchingElement                                     //@A2
                (const AlarmUniqueId& auid, ICursor &aCursor)             //@A2
  {                                                                       //@A2
    if (aCursor.isValid()) return false;                                 //@A2
    IATime currentKey=key(elementAt(aCursor));                           //@A2
    for (;(!elementAt(aCursor)->matches(auid) &&                         //@A2
            !isLast(aCursor) && currentKey==key(elementAt(aCursor)));   //@A2
          setToNext(aCursor));                                           //@A2
    return elementAt(aCursor)->matches(auid);                            //@A2
  }                                                                       //@A2

boolean AlarmSet::findElementForId(ICursor& aCursor,                //@A2
                   const AlarmUniqueId& alarmuid, const IATime& alarmTime)      //@A2

{
  for (locateElementWithKey(alarmTime,aCursor);aCursor.isValid() &&        //@A2
           elementAt(aCursor)->getAlarmUniqueId()!=alarmuid;
           locateNextElementWithKey(alarmTime,aCursor)) {}
  if (aCursor.isValid()) return true;
  else return false;
}

boolean AlarmSet::findFirstSelector(ICursor &aCursor,                     //@A1
                                   const AString& selector)               //@A1
  {                                                                       //@A1
    if (!setToFirst(aCursor)) return false;                              //@A1
    for (;(elementAt(aCursor)->getSelector()!=selector) &&               //@A1
            !isLast(aCursor);                                            //@A1
          setToNext(aCursor));                                           //@A1
    return (elementAt(aCursor)->getSelector()==selector);                //@A1
  }                                                                       //@A1

boolean AlarmSet::findNextSelector(ICursor &aCursor,                      //@A1     //@A1
                                   const AString& selector)               //@A1
  {                                                                       //@A1
    if (!setToNext(aCursor)) return false;                               //@A1
    for (;(elementAt(aCursor)->getSelector()!=selector) &&               //@A1
            !isLast(aCursor);                                            //@A1
          setToNext(aCursor));                                           //@A1
    return (elementAt(aCursor)->getSelector()==selector);                //@A1
  }                                                                       //@A1

// find and remove given alarm - match entire alarm not just key
boolean AlarmSet::findAndRemoveElement(const Alarm* a)
  {
    Cursor aCursor(*this);
    if (!setToFirst(aCursor)) return false;
    for (;!(elementAt(aCursor)->matches(a))&& !isLast(aCursor);setToNext(aCursor));
    if (!(elementAt(aCursor)->matches(a))) return false;
    delete elementAt(aCursor);
    removeAt(aCursor);
    return true;
  }

// delete then remove all the alarms in the set
void AlarmSet::RemoveAll()
  {
    Cursor aCursor(*this);
    if (!setToFirst(aCursor)) return;
    for (;!(isLast(aCursor));setToNext(aCursor)) delete elementAt(aCursor);
    // delete last element
    delete elementAt(aCursor);
    removeAll();
  }

// delete then remove first alarm
void AlarmSet::RemoveFirst()
  {
    delete firstElement();
    removeFirst();
  }

// delete then remove the alarm at the cursor
void AlarmSet::RemoveAt(ICursor& aCursor)
  {
    delete elementAt(aCursor);
    removeAt(aCursor);
  }


// output all the alarms in the set
void AlarmSet::print(ostream& os) const
{
    Cursor aCursor(*this);
    os << "Alarms in alarmSet:"<<endl;
    if (!setToFirst(aCursor)) return;
    for (;!isLast(aCursor);setToNext(aCursor))
     {
       os << *elementAt(aCursor) << endl;
     }
    os << *elementAt(aCursor) << endl;
}

// AlarmIdSet

      // return false if first element removed
boolean AlarmIdSet::removeAtAndResetCursor(ICursor& aCursor) {                      //@A2
// this works for sets (unique keys)
   Cursor tCursor(*this);
   tCursor.copy(aCursor);
   if (!tCursor.setToPrevious()) {
      removeAt(aCursor);
      return false;
   }
   else {
      AlarmUniqueId previousId=elementAt(tCursor)->getAlarmUniqueId();
      removeAt(aCursor);
      // cursor is now valid and ready to iterate to the next element
      locateElementWithKey(previousId,aCursor);
  }
  return true;
}
