#ifndef ALARMSET_H
#define ALARMSET_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: alarmset.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/alarmset.h, tka_timer, abe.l6
// VERSION: 1.9.1.2 97/06/13 11:18:29
//
// INCLUDE AS:  #include <alarmset.h>
//
// PURPOSE:  Definition of AlarmSet
//
// DESCRIPTION:
//    A Key Sorted bag of Alarm* (sorted by the time the alarm is to occur
//    next).
//
// CLASSES DECLARED:
//    AlarmSet
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $A1   960909 WES  f304   Add support for selector in alarms
// $A2   970304 WES  f658   New mapping for alarms
// $A3   970522 WES  f      New reset function for persistence
//
//******************************************************************************

// Openclass Includes
#include <time.h>
#include <iostream.h>
#include <iksbag.h>
#include <iksset.h>
#include <iapl/astring.h>
#include <alarm.h>
#include <istdops.h>

class TimeOps : public IStdMemOps

 {
  public :

  void assign (Alarm *& lhs,Alarm* const& rhs) const { lhs=rhs; }

  IBoolean equal (Alarm * const& lhs, Alarm* const& rhs) { return lhs==rhs; }

  IATime const& key (Alarm* const& anAlarm) const { return anAlarm-> getAlarmTime(); }

  class KeyOps
   {
     public:

     IBoolean equal (IATime const& lhs, IATime const& rhs) const
        { return lhs==rhs; }

     long compare (IATime const& lhs, IATime const& rhs) const
        { return lhs.compare(rhs); }
   } keyOps;
 };

class AlarmSet : public IGKeySortedBag < Alarm*,IATime,TimeOps> {

   public:
      friend ostream& operator <<(ostream& os, const AlarmSet& a)
         {a.print(os);return os;}
      boolean findElement(const Alarm*,ICursor&);
      boolean findAndRemoveElement(const Alarm*);

      Alarm* firstActiveAlarm() const;                                    //@A3

      Alarm* firstActiveAlarm(ICursor&) const;                            //@A3

      boolean findElement                                                 //@A2
                      (const AlarmUniqueId&, ICursor&);                   //@A2
      // starts with cursor at matching key                               //@A2
      boolean findMatchingElement                                         //@A2
                      (const AlarmUniqueId&, ICursor&);                   //@A2
      boolean findFirstSelector                                           //@A1
                      (ICursor&, const AString&);                         //@A1
      // ICursor must be valid for findNextSelector                       //@A1
      // This will look for the next element after this with given        //@A1
      // selector                                                         //@A1
      boolean findNextSelector                                            //@A1
                      (ICursor&, const AString&);                         //@A1
      // These are the exact same as the collection class counterparts
      // except they all delete the element
      void RemoveFirst();
      void RemoveAt(ICursor&);
      void RemoveAll();

      boolean findElementForId(ICursor&,const AlarmUniqueId&,             //@A2
                               const IATime&);                            //@A2

protected:
      void print(ostream& os) const;
};


class IdOps : public IStdMemOps

 {
  public:
  void assign (Alarm *& lhs,Alarm* const& rhs) const { lhs=rhs; }

  IBoolean equal (Alarm * const& lhs, Alarm* const& rhs) { return lhs==rhs; }

  AlarmUniqueId const& key(Alarm* const& anAlarm) const
           { return anAlarm-> getAlarmUniqueId(); }

  class KeyOps
   {
     public:

     IBoolean equal (AlarmUniqueId const& lhs, AlarmUniqueId const& rhs) const
        { return lhs==rhs; }

     long compare (AlarmUniqueId const& lhs, AlarmUniqueId const& rhs) const
        {
           return lhs >= rhs ?
                !(lhs==rhs) : -1 ;
        }
    } keyOps;
 };

class AlarmIdSet :
    public IGKeySortedSet < Alarm*, AlarmUniqueId, IdOps>  {

      // return false if first element removed
   public:
      boolean removeAtAndResetCursor(ICursor& aCursor);                      //@A2

};
#endif
