#ifndef _ALARMUID_H
#define _ALARMUID_H

/*************************************************************************
** (C) Copyright IBM Corp, 1997
**
** File: alarmuid.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/alarmuid.h, tka_timer, abe.l6
// VERSION: 1.1 97/06/13 11:31:38
//
// INCLUDE AS:  #include <alarmuid.h>
//
// PURPOSE:  Alarm unique identifier class definition
//
// DESCRIPTION:
//     class defining unique identifier for alarms
//
// CLASSES DECLARED:
//    AlarmUniqueId- Abstract class
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// New   970224 WES  f658  Unique identifiers for sorting alarms
//       970311 WES  f664   Persistent alarms
//******************************************************************************

// Includes
#include <iapl/astring.h>
#include <iatk/types.h>
#include <iostream.h>
#include <persobj.h>
#include <persfile.h>

class AlarmUniqueId: public PersistentObject {
   public:

       AlarmUniqueId():selector(""),typeId('d'),id("") {}

       AlarmUniqueId(const AString& aSelector, char aTypeId,
                     const AString& anId):
                     selector(aSelector),typeId(aTypeId),id(anId) {}

       ~AlarmUniqueId() {}

       AString className() const { return "AlarmUniqueId"; }

       char getType() const { return typeId; }

       boolean isDummy() const { return typeId=='d'; }

       AString getSelector() const { return selector; }

       AString getId() const { return id; }

       AString getPersistentId() const { return typeId+id; }

       void setSelector(const AString& aSelector) { selector=aSelector; }

       void setId(const AString& anId) { id=anId; }

       void setPersistentId(const AString& persistentId)
            {
              id=persistentId.subString(0,persistentId.length()-2);
              typeId=persistentId[persistentId.length()-1];
            }

       void set(const AString& aSelector,char aTypeId, const AString& anId)
          { selector=aSelector;
            typeId=aTypeId;
            id=anId; }

      IABool operator==(const AlarmUniqueId& alarmuid) const
            { return selector==alarmuid.selector && id==alarmuid.id
                     && typeIdMatches(alarmuid);  }

      IABool operator!=(const AlarmUniqueId& alarmuid) const
            { return !(*this==alarmuid); }

      IABool operator<(const AlarmUniqueId& alarmuid) const
            { return selector<alarmuid.selector ||
                 (selector==alarmuid.selector &&
                 ( id < alarmuid.id || (id==alarmuid.id &&
                         typeId < alarmuid.typeId  &&
                         !typeIdMatches(alarmuid) ) ) ); }


      IABool operator>=(const AlarmUniqueId& alarmuid) const
            { return selector > alarmuid.selector ||
                 (selector==alarmuid.selector &&
                 ( id > alarmuid.id || (id==alarmuid.id &&
                         (typeId > alarmuid.typeId ||
                         typeIdMatches(alarmuid)) ) ) ); }

      IABool typeIdMatches(AlarmUniqueId aUid) const
       {
         if (typeId==aUid.typeId || typeId=='u' || aUid.typeId=='u')
            return true;
         else return false;
       }

   AString classId() const {return classid; }

   AlarmUniqueId * createCopy() const { return new AlarmUniqueId(*this); }

    boolean save(PersistentFile &objFile) const
          {
            if (!objFile.writeWholeString(selector))
              return false;
            if (!objFile.writeWholeString(id))
              return false;
            return objFile.write(typeId);
          }

    boolean restore(PersistentFile &objFile)
          {
            if (persistentVersion > version) {
               cout << "AlarmUniqueId Versions not compatible" << endl;
               return false;
            }
            if (!objFile.readWholeString(selector))
              return false;
            if (!objFile.readWholeString(id))
              return false;
            return objFile.read(typeId);
          }


    // only static variable from persistence is currently persistentVersion
    static boolean restoreStaticVars(PersistentFile& objectFile);

    // only static variable from persistence is currently persistentVersion
    static boolean saveStaticVars(PersistentFile& objectFile);


private:
   AString selector;
   AString id;
   // type id's are currently
   // "i" - identified (these are for alarms that number of duplicates = 0
   // "n" - non-identified (these are for alarms that number of duplicates may be > 0 )
   // "d" - place holder alarms - not real alarms
   // "u" - unknown type - used for building AlarmUniqueIds for matching
   char typeId;
   static const int version;
   static boolean versionAdded;
   static int persistentVersion;
   static const AString classid;

};


#endif //_ALARMUID_DEF
