#ifndef EVENTSET_H
#define EVENTSET_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: eventset.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/eventset.h, tka_timer, abe.l6
// VERSION: 1.1 97/06/13 11:27:37
//
// INCLUDE AS:  #include <eventset.h>
//
// PURPOSE:  Definition of EventSet
//
// DESCRIPTION:
//    A Key Sorted bag of TimeAdapterEventInfo struct * (contain
//    IAUniqueId and Alarm
//    (sorted by the alarmtime of the alarm - when the alarm occurred
//
// CLASSES DECLARED:
//    AlarmSet
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// New   970411 WES  f664   Persistent Alarms
//
//******************************************************************************

// Openclass Includes
#include <time.h>
#include <iostream.h>
#include <iksbag.h>
#include <iksset.h>
#include <iapl/astring.h>
#include <alarm.h>
#include <istdops.h>


struct TimeAdapterEventInfo {
  IAUniqueId eventId;
  Alarm * notifiedAlarm;
};


class TimeEventOps : public IStdMemOps

 {
  public:

  void assign (TimeAdapterEventInfo *& lhs,TimeAdapterEventInfo* const& rhs) const { lhs=rhs; }

  IBoolean equal (TimeAdapterEventInfo * const& lhs, TimeAdapterEventInfo* const& rhs)
              { return lhs==rhs; }

  IATime const& key(TimeAdapterEventInfo* const& timeEvent) const
           { return (timeEvent-> notifiedAlarm) -> getAlarmTime();  }

  class KeyOps
   {
     public:

     IBoolean equal (IATime const& lhs, IATime const& rhs) const
        { return lhs==rhs; }

     long compare (IATime const& lhs, IATime const& rhs) const
        { return lhs.compare(rhs); }

   } keyOps;

 };

typedef IGKeySortedBag < TimeAdapterEventInfo*, IATime, TimeEventOps> TimeEventSet;

#endif
