#ifndef IATIME_H
#define IATIME_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: iatime.h
**
** Description: This class gets the current time and provides specific
** information requested.
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/iatime.h, tka_timer, abe.l6
// VERSION: 1.13.1.2 97/06/13 11:19:08
//
// INCLUDE AS:  #include <iatime.h>
//
// PURPOSE:  Definition of iatime
//
// DESCRIPTION:  This class contains time information and is used as an
//               Openclass collection class key
//
// CLASSES DECLARED:
//    IATime - class for time information
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $A1   960918 WES f304   Cleanup
// $A2   961017 WES f441   Support for intervals of months and years
// $A3   970227 WES f668   Add end bounded identified interval alarm
// $A4   970304 WES f658   Additional mapping for alarms
// $A5 = 970311 WES  f664   Persistent alarms
//******************************************************************************


// Openclass Includes
#include <iapl/common.h>
#include <time.h>
#include <iapl/astring.h>
#include <iostream.h>
#include <timeexcp.h>
#include <persobj.h>
#include <persfile.h>

typedef Bool boolean;
typedef AString DateType;
typedef AString TimeType;
typedef AString TimeStamp;
typedef int Interval;
typedef AString IntervalUnits;

static char* dateDelimeter="-";

#define IATIME_VERSION                  1                                 //@A5

class IATime: public PersistentObject  {
   public:

   friend ostream& operator<<(ostream& os, const IATime& a) {cout<< a.getDateTime();return os;}

// Default ctor
      IATime();
// Ctors
      IATime(const DateType& newdate, const TimeType& newtime)
                               throw(DateFormatError,TimeFormatError);

      IATime(time_t newTime) throw(TimeFormatError);
// Accessors

      int weekday() const { return thisTime.tm_wday; }                    //@A1
      int second() const  { return thisTime.tm_sec; }                     //@A1
      int minute() const { return thisTime.tm_min; }                      //@A1
      int hour() const { return thisTime.tm_hour; }                       //@A1
// minutes past midnight
      int dayminute() const
         { return (thisTime.tm_hour)*60+(thisTime.tm_min); }              //@A1
      int daysecond() const
         { return ((thisTime.tm_hour)*60+(thisTime.tm_min))*60+           //@A3
                    thisTime.tm_sec; }                                    //@A3
// Julian day
      int yearday() const { return thisTime.tm_yday+1; }                  //@A1
// since 1900
      int year() const { return thisTime.tm_year; }                       //@A1
      int month() const { return thisTime.tm_mon; }                       //@A1
// day in month
      int day() const { return thisTime.tm_mday; }                        //@A1

      int isdst() const { return thisTime.tm_isdst; }                     //@A3

      long compare(const IATime& rhs) const                               //@A4
           { return aTime-rhs.aTime; }                                    //@A4                                                                //@A4

      time_t gettime_t() const { return aTime; }                          //@A1
// To Strings
      DateType getDate() const;
      TimeType getTime() const;
      TimeStamp getDateTime() const;
// Setters
      void setDate(const DateType& newdate) throw(DateFormatError);
      void setTime(const TimeType& newtime) throw(TimeFormatError);
      void setDateTime(const DateType& newdate, const TimeType& newtime)
                               throw(DateFormatError,TimeFormatError);

      void settime_t(time_t) throw(TimeFormatError);                      //@A5

// Update IATime with current time

      void now();

// operations
      boolean operator==(const IATime& newTime) const                     //@A1
            { return aTime==newTime.aTime; }                              //@A1
      boolean operator!=(const IATime& newTime) const                     //@A1
            { return aTime!=newTime.aTime; }                              //@A1
      boolean operator<(const IATime& newTime) const                      //@A1
            { return aTime<newTime.aTime; }                               //@A1
      boolean operator<=(const IATime& newTime) const                     //@A1
            { return aTime<=newTime.aTime; }                              //@A1
      boolean operator>(const IATime& newTime) const                      //@A1
            { return aTime>newTime.aTime; }                               //@A1
      boolean operator>=(const IATime& newTime) const                     //@A1
            { return aTime>=newTime.aTime; }                              //@A1
      boolean between(const IATime& time1, const IATime& time2) const     //@A1
        {return ((operator>(time1)) && (operator<(time2))); }

      IATime addInterval(Interval ainterval,const IntervalUnits& units) const
             throw(IntervalValueError,IntervalUnitsError);

// difference in days
      int diffDate(const IATime& newtime) const;

// difference in years                                                    //@A3
      int diffYears(const IATime& newTime) const                          //@A3
         {
           boolean leapDiff=(newTime.day()==29 && day()==28 &&            //@A3
                   newTime.month()==1 && month()==2);                     //@A3
           return ((newTime.yearday() > yearday()  && !leapDiff) ||       //@A3              //@A3
            ((newTime.yearday() == yearday() || leapDiff) &&              //@A3
             newTime.daysecond() > daysecond() ) ) ?                        //@A3
            year()-newTime.year()-1:                                      //@A3
            year()-newTime.year();                                        //@A3
         }                                                                //@A3

// difference in months                                                   //@A3
      int diffMonths(const IATime& newTime) const                         //@A3
         {                                                                //@A3
           boolean dayEqual=(newTime.day() > day() &&                     //@A3
                   day()==daysInMonth(year(),month()));                   //@A3
           return ((newTime.day() > day() && !dayEqual) ||                //@A3
              ( ( newTime.day() == day() || dayEqual) &&                  //@A3
                newTime.daysecond() > daysecond() ) ) ?                   //@A3
            (year()-newTime.year())*12+month()-newTime.month()-1 :        //@A3
            (year()-newTime.year())*12+month()-newTime.month();           //@A3
         }                                                                //@A3

// difference in seconds
      double diffTime(const IATime& newtime) const
        { return difftime(aTime,newtime.aTime); }                         //@A1

      static boolean checkUnits(const IntervalUnits& units)
                         throw(IntervalUnitsError);

// converts to a value of seconds
      long convertInterval(Interval aInterval,                            //@A2
                           const IntervalUnits& units) const              //@A2
                           throw(IntervalUnitsError);

// get time 0 for defaults
      static IATime getZeroTime();

   static boolean isLeapYear(int year);                                   //@A2

   static int daysInMonth(int year, int month);                           //@A2

   AString className() const { return "IATime"; }                         //@A5

   AString classId() const {return classid; }                             //@A5

   IATime * createCopy() const { return new IATime(*this); }                    //@A5

   boolean save(PersistentFile &objFile) const
          {                                                               //@A5
            return objFile.write(aTime);                                  //@A5
          }                                                               //@A5

    boolean restore(PersistentFile &objFile)                              //@A5
          {                                                               //@A5
            if (persistentVersion > version) {
               cout << "IATime Versions not compatible" << endl;
               return false;
            }
            if (!objFile.read(aTime)) return false;                       //@A5
            thisTime=*localtime(&aTime);                                  //@A5
            return true;                                                  //@A5
          }                                                               //@A5

    static boolean restoreStaticVars(PersistentFile& objectFile);         //@A5

    static boolean saveStaticVars(PersistentFile& objectFile);            //@A5

private:
   time_t aTime;                                                          //@A1
   struct tm thisTime;                                                    //@A1
   static const int version;                                              //@A5
   static boolean versionAdded;                                           //@A5
   static int persistentVersion;                                          //@A5
   static const AString classid;                                          //@A5

   // helper function

   static int daysInYear (int year);
   boolean checkDate(const DateType& newDate) const throw(DateFormatError);
   boolean checkTime(const TimeType& newTime) const throw(TimeFormatError);

};

#endif
