#ifndef _PERSFILE_H
#define _PERSFILE_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: persfile.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/persfile.h, tka_timer, abe.l6
// VERSION: 1.1 97/06/13 11:30:52
//
// INCLUDE AS:  #include <persfile.h>
//
// PURPOSE:  Persistent definition
//
// DESCRIPTION:
//    Class used to save and restore persistent objects
//    Uses PersistentObject class in implementation of init
//
// CLASSES DECLARED:
//    PersistentFile
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// NEW   970331 WES  f664   Persistent Alarms
//******************************************************************************

// Includes
#include <string.h>
#include <iapl/astring.h>
#include <stdio.h>

#define NO_PERSISTENT_FILE_MODE 0
#define READ_PERSISTENT_FILE_MODE 1
#define WRITE_PERSISTENT_FILE_MODE 2
#define APPEND_PERSISTENT_FILE_MODE 3

class PersistentObject;
typedef Bool boolean;

class PersistentFile {
   public:

      PersistentFile(AString fn):filename(fn),
                                   objectFile(NULL),
                                   mode(NO_PERSISTENT_FILE_MODE),
                                   buffer(new char[bufferLength])
      {}

      PersistentFile():filename(""),
                       objectFile(NULL),
                       mode(NO_PERSISTENT_FILE_MODE),
                       buffer(new char[bufferLength])
      {}

      ~PersistentFile() { delete[] buffer; }

      boolean eraseContents()
        {
          if (!init(WRITE_PERSISTENT_FILE_MODE)) return false;
          return close();
        }

      void setFileName(AString fn) { filename=fn; }

      AString fileName() const { return filename; }

      // can only read or write between one invocation of init and close
      boolean init(int mode);  // open file and process static data

      boolean saveObject(PersistentObject * object);

      // creates new object - responsibility of caller to delete
      PersistentObject * restoreNewObject();

      PersistentObject * restoreNewObject(const AString& classid);

      boolean close();

      boolean processStaticVars();

      boolean atEOF();

// the following methods can only be called after init() and before close()
// has been called

  // write and read strings with white space or greater than 1024 chars
     boolean writeWholeString( const AString& string);

     boolean readWholeString(AString& string);

   // write and read string tokens - no blanks in string -not checked
   // also less than 1024 in length
   // no empty strings
     boolean write(const AString& string);

    boolean read(AString& string);

    boolean write(const long& number);

    boolean read(long& number);

    boolean write(const int& number);

    boolean read(int& number);

    boolean write(const char& aChar);

    boolean read(char& aChar);

    boolean readEnd();

    boolean writeEnd();

private:

   AString filename;
   FILE* objectFile;
   int mode;
   char * buffer;                                                                                 //@A
   static const int bufferLength;
};

#endif //_PERSFILE_DEF
