/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: persobj.cpp
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/persobj.cpp, tka_timer, abe.l6
// VERSION: 1.1 97/06/13 11:29:24
//
// PURPOSE:  PersistentObject class implementation
//
// DESCRIPTION:
//    Abstract class defining objects used for persistence
//
// CLASSES DECLARED:
//    PersistentObject- Abstract class
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// New   970401 WES  f664   Alarm persistence
//******************************************************************************

#include <iapl/common.h>

static char _partid_[] = "@(#) 1.1 src/adapters/timer/persobj.cpp, tka_timer, abe.l6";

#include <persobj.h>
#include <persfile.h>
#include <iostream.h>

int PersistentObject::staticVarClassNumber=0;
int PersistentObject::objectClassNumber=0;
const int PersistentObject::version=1;

staticVarItem * PersistentObject::staticVarList[MAX_CLASS_NUM];
ObjectItem * PersistentObject::persistentClassList[MAX_CLASS_NUM];

// 0 indicates persistent Version hasn't been restored yet
int PersistentObject::persistentVersion=0;

// no other static variables need to be saved?
boolean PersistentObject::versionAdded=
                     addStaticVars("PersistentObject",
                     &restoreStaticVars,
                     &saveStaticVars);


boolean PersistentObject::addStaticVars(const AString& name, const void* restoreFunctionPtr,
                                    const void* saveFunctionPtr)
   {
     staticVarItem * item=new staticVarItem;
     (item->className)=name;
     item->restoreStaticFunction=(boolean(*)(PersistentFile&))restoreFunctionPtr;
     item->saveStaticFunction=(boolean(*)(PersistentFile&))saveFunctionPtr;
     staticVarList[staticVarClassNumber]=item;
     staticVarClassNumber++;
     return true;
   }


PersistentObject * PersistentObject::addObject(const AString& name,
                                    PersistentObject * newObject)
   {
     ObjectItem * item=new ObjectItem;
     item->className=name;
     item->object=newObject;
     item->priority=99;
     persistentClassList[objectClassNumber]=item;
     objectClassNumber++;
     return newObject;
   }

int classPriorityCompare(const void * arg1, const void * arg2) {
 if ((*(ObjectItem **)arg1)->priority < (*(ObjectItem **)arg2)->priority)
                                                      return -1;
 else if ((*(ObjectItem **)arg1)->priority == (*(ObjectItem **)arg2)->priority)
                                                      return 0;
 else return 1;
}

PersistentObject* PersistentObject::addObject(const AString& name,
                                    PersistentObject * newObject,
                                    int listPriority)
   {
     // sorted list
     ObjectItem * item=new ObjectItem;
     item->className=name;
     item->object=newObject;
     item->priority=listPriority;
     // insert item according to priority
     int i;
     for (i=objectClassNumber;i > 0 && listPriority <
               persistentClassList[i-1]->priority;i--) {
         persistentClassList[i]=persistentClassList[i-1];
     }
     persistentClassList[i]=item;
     objectClassNumber++;
     return newObject;
   }

      // number of objects with higher priority
boolean PersistentObject::setObjectPriority(const AString& name,
                                                    int priority)
   {
     // find object, set priority, and move if necessary
     return true;
   }

    // only static variable from persistence is currently persistentVersion
boolean PersistentObject::restoreStaticVars(PersistentFile& objectFile)
    {
      if (!objectFile.read(persistentVersion)) return false;
      return objectFile.readEnd();
    }

    // only static variable from persistence is currently persistentVersion
boolean PersistentObject::saveStaticVars(PersistentFile& objectFile)
    {
      if (!objectFile.write(version)) return false;
      return objectFile.writeEnd();
    }

boolean PersistentObject::saveAllStatics(PersistentFile& objectFile)
 {
   if (!objectFile.write("BeginStatics")) return false;
   if (!objectFile.writeEnd()) return false;
   for (int i=0;i<staticVarClassNumber;i++)
     {
       if (!objectFile.write(staticVarList[i]->className)) return false;
       // call function saved in list for classname
       if (!(*(staticVarList[i]->saveStaticFunction))(objectFile)) return false;
     }
   if (!objectFile.write("EndStatics")) return false;
   return objectFile.writeEnd();
 }

boolean PersistentObject::restoreAllStatics(PersistentFile& objectFile)
 {
   // if className doesn't exist is an error - need to keep at least a
   // stub with dummy save and restoreStatic functions
   AString keyword;
   if (!objectFile.read(keyword)) return false;
   if (keyword!="BeginStatics") return false;
   if (!objectFile.readEnd()) return false;
   if (!objectFile.read(keyword)) return false;
   int i;
   while (keyword!="EndStatics")
     {
       for (i=0;
            (i< staticVarClassNumber && keyword!=staticVarList[i]->className);
            i++)
         {}
       if (keyword!=staticVarList[i]->className)
         {
          cout << "restoreAllStatics ClassName: " << keyword << "not found." << endl;
          return false;
         }
       else if
           // call function saved in list for classname
           (!(*(staticVarList[i]->restoreStaticFunction))(objectFile)) return false;
       if (!objectFile.read(keyword)) return false;
     }
  return objectFile.readEnd();
 }
