#ifndef _PERSOBJ_H
#define _PERSOBJ_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: persobj.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/persobj.h, tka_timer, abe.l6
// VERSION: 1.1 97/06/13 11:28:37
//
// INCLUDE AS:  #include <persobj.h>
//
// PURPOSE:  PersistentObject class definition
//
// DESCRIPTION:
//    Abstract class defining objects used for persistence
//
// CLASSES DECLARED:
//    PersistenObject- Abstract class
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// New   970331 WES  f664   Persistent Alarms
//******************************************************************************

// Includes
#include <iapl/astring.h>
#include <string.h>
typedef Bool boolean;

#define MAX_CLASS_NUM                  50

class PersistentFile;
class PersistentObject;

struct staticVarItem {                                                         //@A4
  AString className;
  boolean (*restoreStaticFunction)(PersistentFile&);
  boolean (*saveStaticFunction)(PersistentFile&);
};

struct ObjectItem {
  AString className;
  PersistentObject * object;
  int priority;
};

class PersistentObject {
   public:

    virtual ~PersistentObject() {}

    // only static variable from persistence is currently persistentVersion
    static boolean restoreStaticVars(PersistentFile& objectFile);

    // only static variable from persistence is currently persistentVersion
    static boolean saveStaticVars(PersistentFile& objectFile);

    virtual boolean save(PersistentFile& objectFile) const=0;

    virtual boolean restore(PersistentFile& objectFile)=0;

    virtual AString className() const { return "PersistentObject"; }

    virtual AString classId() const=0;

    virtual PersistentObject* createCopy() const=0;

    static boolean saveAllStatics(PersistentFile& objectFile);

    static boolean restoreAllStatics(PersistentFile& objectFile);

    static PersistentObject * getDefaultObject(AString aClass)                 //@A4
       {
         for (int i=0;i < objectClassNumber;i++)
             if (persistentClassList[i]->className==aClass)
                return persistentClassList[i]->object;
         return NULL;
      }


protected:

      PersistentObject() {}

      static  boolean addStaticVars(const AString& name,const void* saveFunctionPtr,
                                    const void * restoreFunctionPtr);

      static PersistentObject * addObject(const AString& name,
                                          PersistentObject * object);

      // Priority is 1-99  1 is highest priority
      static PersistentObject * addObject(const AString& name,
                                          PersistentObject * object,
                                          int priority);

      // number of objects with higher priority
      static boolean setObjectPriority(const AString& name, int priority);

private:
   static const int version;
   static int staticVarClassNumber;
   static int objectClassNumber;
   static staticVarItem* staticVarList[MAX_CLASS_NUM];
   static ObjectItem* persistentClassList[MAX_CLASS_NUM];
   static boolean versionAdded;
   static int persistentVersion;
};

#endif //_PERSOBJ_H
