/************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: Time_adapter.cpp
**
** Description:  Implementation of Timer Adapter
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/timeadap.cpp, tka_timer, abe.l6
// VERSION: 1.5.2.6 97/06/23 15:40:39
//
// PURPOSE:  Implementation of timer adapter
//
// CLASSES IMPLEMENTED:
//    Time_Adapter
//
//******************************************************************************
//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $B1 = 960709 WES d190   Fix endBoundedIntervalAlarms registerProcedure
//                         signature
//
// $B2 = 960710 WES d202   Fix wrong calls on endBoundedIntervalAlarms effectors
//
// $B3 = 960710 WES d203   Fix error in dayOfWeek sensor- was using integer to
//                         build what should have been string term
//
// $B4 = 960711 WES d204   Fix error in handling of free terms in sensors
//                         Fix determineTimeInterval isLogicalVariable (need
//                          to specify integer as type, not default)
//                         Fix compares of string terms in all sensors with
//                          free terms
// $B5 = 960826 WES d280   Cleanup
// $A2 = 960909 WES f304   Put support in for route-id in alarms,
//                         react to atom, factset, adapter changes,
//                         take out check termlist calls, add Identified one-time
// $A3 = 961009 WES f406   React to IAEventHeader and IATriggerEvent changes
//
// $A4 = 961031 WES f441   Support for months, years, also selector with blanks
//
// $A5 = 961115 WES f480   Support for reset, restart, quiesce
//
// $A6 = 961206 WES f512   React to IAAtom, IAFactSet throws
//
// $A7 = 961211 WES f534   Register listAlarmTimes
//
// $A8 = 970224 WES f661   New effector for random start from given start time
//
// $A9 = 970225 WES f658   Additional mapping for alarms
//
// $AA = 970227 WES f668   Support end bounded identified interval alarms
//
// $B1 = 970307 WES f674   end bounded identified interval alarm random start
//
// $B2 = 970411 WES f664   persistent alarms
//
// $B3 = 970509 WES        reset effector support
//******************************************************************************

#include <iapl/common.h>
static char _partid_[] = "@(#) 1.5.2.6 src/adapters/timer/timeadap.cpp, tka_timer, abe.l6";

// Includes
#include <iatk/iaerrors.h>
#include <timeadap.h>
#include <itrace.hpp>
#include <string.h>                                                       //@B4
#include <stdlib.h>                                                       //@A2
#include <persfile.h>                                                     //@B2
#include <eventset.h>                                                     //@B2



static int userNum = 0;
static char unitsIndex[7][8]={"onetime","years","months","days","hours","minutes","seconds"}; //@B2
static int maxCriteriaValue[6][7]= { { -1, -1, -1, -1, -1, -1, -1},       //@B2
             { 0, 1, 12, 12*365, 12*365*24, 12*365*24*60, 12*365*24*60*60}, //years //@B2
             { 0, 0, 1, 365, 365*24, 365*24*60, 365*24*60*60},  //months  //@B2
             { 0, 0, 0, 1, 24, 24*60, 24*60*60},                //days    //@B2
             { 0, 0, 0, 0, 1, 60, 60*60},                       // hours  //@B2
             { 0, 0, 0, 0, 0, 1, 60} };                          // minutes

static AString tempScope="timeUser";

char Time_Adapter::sensorIdentifiers[30][80]={
                     "",                                             // 0
                     "timeAfter",                                    // 1
                     "currentTimeAfter",                             // 2
                     "timeBefore",                                   // 3
                     "currentTimeBefore",                            // 4
                     "timeBetween",                                  // 5
                     "currentTimeBetween",                           // 6
                     "timeEqual",                                    // 7
                     "currentTimeEqual",                             // 8
                     "timeNotEqual",                                 // 9
                     "currentTimeNotEqual",                          // 10
                     "isCurrentDay",                                 // 11
                     "isCurrentWeekday",                             // 12
                     "isWeekend",                                    // 13
                     "isCurrentWeekend",                             // 14
                     "dateAndTime",                                  // 15
                     "date",                                         // 16
                     "time",                                         // 17
                     "dayOfWeek",                                    // 18
                     "addTimeInterval",                              // 19
                     "determineTimeInterval",                        // 20
                     "","","","","", "","","","" };    // 21 -29


char Time_Adapter::effectorIdentifiers[50][80]={
                     "",                                             // 0
                     "turnOffAlarm",                                 // 1
                     "setIntervalAlarm",                             // 2
                     "setBoundedIntervalAlarm",                      // 3
                     "setStartBoundedIntervalAlarm",                 // 4
                     "setDailyAlarm",                                // 5
                     "setSpecificDailyAlarm",                        // 6
                     "setAlarmAt",                                   // 7
                     "setEndBoundedIntervalAlarm",                   // 8
                     "setIntervalAlarmRandomStart",                  // 9
                     "setRandomStartBoundedIdentifiedIntervalAlarm", // 10
                     "setRandomBoundedIdentifiedIntervalAlarm",      // 11
                     "","","","","","","","",                        // 12-19
                     "","","","","","","","","","",                  // 20-29
                     "reset",                                        // 30
                     "setOverdueAlarmPresentationCriteria",          // 31
                     "setDuplicateHandling",                         // 32
                     "checkPoint",                                   // 33
                     "setPersistentCategories",                          // 34
                     "","","","","",                              // 35-39
                     "testSetTime",                                  // 40
                     "testAny",                                      // 41
                     "","","","","","",                           // 42-47
                     "listAlarmTimes",                               // 48
                     "listAlarms" };                                  // 49



// The global ::newAdapter function...

IAAdapter * newAdapter(const char *domain, void *parms)
{
   if (strcmp(domain, "Time")) {
      cerr << "TADAPTER: I was called to create a " << domain
           << " adapter and I don't know how." << endl;
      return NULL;
   }

   return new Time_Adapter((const char *) parms);
}

// constructor

Time_Adapter::Time_Adapter(const char * inputParms):
    enabledForNotify(false),
    configDone(false),                                                    //@B3
    persistenceOn(false),                                                 //@B2
    persistenceError(false),                                              //@B2
    configMutex(),                                                        //@B3
    duplicateHandling(IA_TIME_IGNORE_PREPERSISTENT_DUPLICATES),           //@B2
    eventSetMutex(),                                                      //@B2
    eventSet(),                                                           //@B2
    currentEventStamp(new PersistentFile()),                              //@B2
    alternateEventStamp(new PersistentFile()),                            //@B2
    currentEventStampFileName(),                                          //@B2
    lastPersistentTimeStamp(0),                                           //@B2
    lastPersistentTimeExceptions(),                                       //@B2
    AlarmSetMutex(),
    AlarmChangeEvent(AlarmSetMutex),
    Alarms(),
    mappedAlarms(),                                                       //@A9
    currentSavedAlarms(new PersistentFile()),                             //@B2
    alternateSavedAlarms(new PersistentFile()),                           //@B2
    currentSavedAlarmsFileName(),                                         //@B2
    transactionFile(),                                                    //@B2
    noOverdueCriteria(false),                                             //@B2

    timerThread(),                                                        //@B2
    inputParm(inputParms),                                                //@B2
    disabledSelectors(),                                                  //@B2
    persistentCategories(),                                               //@B2
    persistentCategoriesSet(false)
{
  // default is 0
  // one-time, years, months, days
  // one-time
  overdueCriteria[0]= new Time_Adapter_overdue_criteria;
  overdueCriteria[0]->amount=7;
  overdueCriteria[0]->units="days";
  // year
  overdueCriteria[1]= new Time_Adapter_overdue_criteria;
  overdueCriteria[1]->amount=7;
  overdueCriteria[1]->units="days";
  // month
  overdueCriteria[2]= new Time_Adapter_overdue_criteria;
  overdueCriteria[2]->amount=7;
  overdueCriteria[2]->units="days";
  // days
  overdueCriteria[3]= new Time_Adapter_overdue_criteria;
  overdueCriteria[3]->amount=4;
  overdueCriteria[3]->units="hours";
  // hours, minutes
  for (int i=4;i<=6;i++) {
     overdueCriteria[i]=new Time_Adapter_overdue_criteria;
     overdueCriteria[i]->amount=0;
     overdueCriteria[i]->units="seconds";
  }
  AString parms(inputParms);                                              //@B2
}

void Time_Adapter::setPersistentFileName(const IAAtom& atom) {
    AString filepath=atom.getStringTerm(1);                                 //@B3
    transactionFile.setFileName(filepath+"."+IA_TIME_RECONCILE_FILE_EXTENSION); //@B2
    currentEventStamp->setFileName(filepath+"."+IA_TIME_EVENTSTAMP1_FILE_EXTENSION); //@B2
    alternateEventStamp->setFileName(filepath+"."+IA_TIME_EVENTSTAMP2_FILE_EXTENSION); //@B2
    currentEventStampFileName.setFileName(filepath+"."+IA_TIME_EVENTSTAMPNAME_FILE_EXTENSION); //@B2
    currentSavedAlarms->setFileName(filepath+"."+IA_TIME_SAVEDALARMS1_FILE_EXTENSION); //@B2
    alternateSavedAlarms->setFileName(filepath+"."+IA_TIME_SAVEDALARMS2_FILE_EXTENSION); //@B2
    currentSavedAlarmsFileName.setFileName(filepath+"."+IA_TIME_SAVEDALARMSNAME_FILE_EXTENSION); //@B2
    persistenceOn=true;
  }

int Time_Adapter::findUnitsIndex(const AString& units)                    //@B2
 {
  int index;                                                              //@B2
  for (index=0;index < 7 && units!=(AString)unitsIndex[index];            //@B2
       index++) {}                                                        //@B2
  return index;                                                           //@B2
 }

// Do register procedures here
IABool Time_Adapter::identify()                                           //@A1
   {
    try                                                                   //@A6
     {
   // register Effectors

      registerProcedure(
            "listAlarms",
             IAEffectorProc,
             buildSignature("Symbol"),
             IA_LISTALARMS);
      registerProcedure(                                                  //@A7
            "listAlarmTimes",                                             //@A7
             IAEffectorProc,                                              //@A7
             buildSignature("Symbol"),                                    //@A7
             IA_LISTALARMTIMES);                                          //@A7
      registerProcedure(
            "setIntervalAlarm",
             IAEffectorProc,
             // interval value, interval units, id, category
             buildSignature("Integer String String String"),
             IA_SETINTERVALALARM);
      registerProcedure(
            "turnOffAlarm",
             IAEffectorProc,
             buildSignature("String"),
             IA_TURNOFFALARM);
      registerProcedure(
            "setBoundedIntervalAlarm",
             IAEffectorProc,
             // interval value, interval units, start date, start time,
             // end date, end time, id, category
             buildSignature("Integer String String String String String String String"),
             IA_SETBOUNDEDINTERVALALARM);
      registerProcedure(
            "setStartBoundedIntervalAlarm",
             IAEffectorProc,
             // interval value, interval units, start date, start time,
             // id, category
             buildSignature("Integer String String String String String"),
             IA_SETSTARTBOUNDEDINTERVALALARM);
      registerProcedure(
            "setDailyAlarm",
             IAEffectorProc,
             // start time, id, category
             buildSignature("String String String"),
             IA_SETDAILYALARM);
      registerProcedure(
            "setSpecificDailyAlarm",
             IAEffectorProc,
             // start time, days of week list, id, category
             buildSignature("String String String String"),
             IA_SETSPECIFICDAILYALARM);
      registerProcedure(
            "setAlarmAt",
             IAEffectorProc,
             // start time, start date, id, category
             buildSignature("String String String String"),
             IA_SETALARMAT);
      registerProcedure(
            "setEndBoundedIntervalAlarm",
             IAEffectorProc,
             // interval value, interval units, end date, end time,
             // id, category
             buildSignature("Integer String String String String String"),
             IA_SETENDBOUNDEDINTERVALALARM);
      registerProcedure(                                                  //@A2
            "setIntervalAlarmRandomStart",                                //@A2
             IAEffectorProc,                                              //@A2
             // interval value, interval units, randomization interval
             // id, category
             buildSignature("Integer String Integer String String"),      //@A2
             IA_SETINTERVALALARMRANDOMSTART);                             //@A2

      registerProcedure(                                                  //@A7
            "setRandomStartBoundedIntervalAlarm",                         //@B1
             IAEffectorProc,                                              //@A7
             // interval value, interval units, start date, start time
             // randomization interval, id, category
             buildSignature("Integer String String String Integer String String"), //@A7
             IA_SETRANDOMSTARTBOUNDEDINTERVALALARM);                      //@B1

      registerProcedure(                                                  //@B1
            "setRandomBoundedIntervalAlarm",                              //@B1
             IAEffectorProc,                                              //@B1
             buildSignature(
             // interval value, interval units, start date, start time
             // end date, end time, randomization interval, id, category
             "Integer String String String String String Integer String String"), //@B1
             IA_SETRANDOMBOUNDEDINTERVALALARM);                           //@B1

      registerProcedure(                                                  //@A2
            "reset",                                                      //@B3
             IAEffectorProc,                                              //@A2
             buildSignature("String"),                                    //@A2
             IA_RESET);                                                   //@A2

      registerProcedure(                                                  //@B3
            "setPersistentCategories",                                    //@B3
             IAEffectorProc,                                              //@B3
             buildSignature("String"),                                    //@B3
             IA_SETPERSISTENTCATEGORIES);                                 //@B3

      registerProcedure(                                                  //@A2
            "testSetTime",                                                //@A2
             IAEffectorProc,                                              //@A2
             buildSignature("Symbol Integer String"),                     //@A2
             IA_TESTSETTIME);                                             //@A2

      registerProcedure(                                                  //@A5
            "testAny",                                                    //@A5
             IAEffectorProc,                                              //@A5
             buildSignature("Symbol String"),                             //@A5
             IA_TESTANY);                                                 //@A5

      registerProcedure(                                                  //@B3
            "setPersistentFileName",                                      //@B3
             IAEffectorProc,                                              //@B3
             buildSignature("String"),                                    //@B3
             IA_SETPERSISTENTFILENAME);                                   //@B3

      registerProcedure(                                                  //@B2
            "setOverdueAlarmPresentationCriteria",                        //@B2
             IAEffectorProc,                                              //@B2
             buildSignature("String Integer String"),                     //@B2
             IA_SETOVERDUEALARMPRESENTATIONCRITERIA);                     //@B2

      registerProcedure(                                                  //@B2
            "setDuplicateHandling",                                       //@B2
             IAEffectorProc,                                              //@B2
             buildSignature("Symbol"),                                    //@B2
             IA_SETDUPLICATEHANDLING);                                    //@B2

      registerProcedure(                                                  //@B2
            "checkPoint",                                                 //@B2
             IAEffectorProc,                                              //@B2
             buildSignature(""),                                          //@B2
             IA_TIMERCHECKPOINT);                                         //@B2

   // register Sensors

      registerProcedure(
             "timeAfter",
             IASensorProc,
             buildSignature("String String String String"),
             IA_TIMEAFTER);
      registerProcedure(
             "currentTimeAfter",
             IASensorProc,
             buildSignature("String String"),
             IA_CURRENTIMEAFTER);
      registerProcedure(
             "timeBefore",
             IASensorProc,
             buildSignature("String String String String"),
             IA_TIMEBEFORE);
      registerProcedure(
             "currentTimeBefore",
             IASensorProc,
             buildSignature("String String"),
             IA_CURRENTIMEBEFORE);
      registerProcedure(
             "timeBetween",
             IASensorProc,
             buildSignature("String String String String String String"),
             IA_TIMEBETWEEN);
      registerProcedure(
             "currentTimeBetween",
             IASensorProc,
             buildSignature("String String String String"),
             IA_CURRENTIMEBETWEEN);
      registerProcedure(
             "timeEqual",
             IASensorProc,
             buildSignature("String String String String"),
             IA_TIMEEQUAL);
      registerProcedure(
             "currentTimeEqual",
             IASensorProc,
             buildSignature("String String"),
             IA_CURRENTIMEEQUAL);
      registerProcedure(
             "timeNotEqual",
             IASensorProc,
             buildSignature("String String String String"),
             IA_TIMENOTEQUAL);
      registerProcedure(
             "currentTimeNotEqual",
             IASensorProc,
             buildSignature("String String"),
             IA_CURRENTIMENOTEQUAL);
      registerProcedure(
             "isCurrentDay",
             IASensorProc,
             buildSignature("String"),
             IA_ISCURRENTDAY);
      registerProcedure(
             "isCurrentWeekday",
             IASensorProc,
             buildSignature("Symbol"),
             IA_ISCURRENTWEEKDAY);
      registerProcedure(
             "isWeekend",
             IASensorProc,
             buildSignature("String"),
             IA_ISWEEKEND);
      registerProcedure(
             "isCurrentWeekend",
             IASensorProc,
             buildSignature("Symbol"),
             IA_ISCURRENTWEEKEND);
      registerProcedure(
             "dateAndTime",
             IASensorProc,
             buildSignature("StringLVar StringLVar"),
             IA_DATEANDTIME);
      registerProcedure(
             "currentDate",
             IASensorProc,
             buildSignature("StringLVar"),
             IA_DATE);
      registerProcedure(
             "currentTime",
             IASensorProc,
             buildSignature("StringLVar"),
             IA_TIME);
      registerProcedure(
             "dayOfWeek",
             IASensorProc,
             buildSignature("StringLVar"),
             IA_DAYOFWEEK);
      registerProcedure(
             "addTimeInterval",
             IASensorProc,
             buildSignature("String String Integer String StringLVar StringLVar"),
             IA_ADDTIMEINTERVAL);
      registerProcedure(
             "determineTimeInterval",
             IASensorProc,
             buildSignature("String String String String IntegerLVar"),
             IA_DETERMINETIMEINTERVAL);
     }
    catch (IAError &e)                                                    //@A6
     {
      cout << "Identify error: " << e.errormsg() << endl;                 //@A6
      return false;                                                       //@A6
     }
    return true;                                                          //@A1
   }

// Allows notify to take place and starts up thread
IABool Time_Adapter::start()                                              //@A1
  {
   IFUNCTRACE_DEVELOP();
   startMutex.lock();
   if (isEnabled())
      {
        cout << "Adapter already started" << endl;
        return false;                                                  //@A1
      }
    enable();
   // start thread now

   //  alt. way of starting thread with processTimer as friend, pointer to
   //  adapter
   //  timerThread.start(processTimer,(void *)this);

   timerThread.start(AMemberFn<Time_Adapter>(*this,processTimer),NULL);
   startMutex.unlock();
   return true;
 }

// Stops thread - still can add to alarms
IABool Time_Adapter::stop()                                               //@A1
    {
      IFUNCTRACE_DEVELOP();
      disable();
      AlarmSetMutex.lock();
      try {
         if (persistenceOn) saveAlarms();
      }
      catch (PersistenceError &e) {
         IAFactSet errorFacts;
         // need fact with effector name
         IAAtom metaAtom;
         metaAtom.setPredicate("Stop");
         errorFacts.add(metaAtom);
         generateAlert("Persistent_Alarm_Error",IAFatal,&errorFacts,NULL);
      }
      AlarmChangeEvent.signal();
      AlarmSetMutex.unlock();
      return true;                                                        //@A1
    }

boolean Time_Adapter::updateAlarmsToPresent()
 {
  // get AlarmSet lock to make sure start/stop timing coordinated

  // Probably the timer thread won't be started
  AlarmSetMutex.lock();
  // Go through set of Alarms and increment past current time to avoid
  // events for alarms that occurred during dormant time

  Alarm * currentAlarm;
  // Could possibly have all expired alarms at which point isEmpty()
  // becomes true
  // process all expired alarms - if not persistent
  while ( !(Alarms.isEmpty()) &&
                 Alarms.firstElement()->isExpired())
    {
      // remove first element
      currentAlarm=Alarms.firstElement();
      Alarms.removeFirst();
      // If recurring, increment alarm and add to alarm set
      if (currentAlarm->isRecurring())
        {
         // no need to change mapped Alarms
         currentAlarm->incrementAlarm();
         Alarms.add(currentAlarm);
        }
      else
        {
         // remove from mapped Alarms before deleting
         mappedAlarms.removeElementWithKey(
                         currentAlarm->getAlarmUniqueId());
         delete currentAlarm;
        }
    }
  AlarmSetMutex.unlock();
  return true;
 }

// may only be called from start to simplify synchronization
boolean Time_Adapter::restoreAlarms() {                                   //@B2
      boolean newAlarms=false;
      boolean success=true;
      boolean filesExist=updateTimeStampInfo();

      // AlarmSetMutex also used for currentSavedAlarmsFileName,         //@B2
      // and currentSavedAlarms and alternateSavedAlarms
      AlarmSetMutex.lock();
      if (!currentSavedAlarmsFileName.init(READ_PERSISTENT_FILE_MODE)) {  //@B2
         // actually this is normal condition if no persistent alarms     //@B2
         currentSavedAlarmsFileName.close();                              //@B2
      }                                                                   //@B2
      else {                                                              //@B2
         AString filepath;                                                //@B2
         if (!currentSavedAlarmsFileName.read(filepath)) {                //@B2
              currentSavedAlarmsFileName.close();
              ITHROW(PersistenceError((AString)"Error reading file name in: "+                         //@B2
                  currentSavedAlarmsFileName.fileName()));                //@B2
         }
         currentSavedAlarmsFileName.close();                              //@B2
         // set currentSavedAlarms to currently valid file                //@B2
         PersistentFile * temp;                                           //@B2

// continue error checking here

         if (filepath==alternateSavedAlarms->fileName()) {                //@B2
             temp=currentSavedAlarms;                                     //@B2
             currentSavedAlarms=alternateSavedAlarms;                     //@B2
             alternateSavedAlarms=temp;                                   //@B2
         }
         else if (!filepath==currentSavedAlarms->fileName()) {            //@B2

         }
         Alarm * newAlarm;                                                //@B2
         if (!currentSavedAlarms->init(READ_PERSISTENT_FILE_MODE)) {       //@B2
         // actually this is normal condition if no persistent alarms     //@B2
           cout << "Unable to open file \""                                 //@B2
                 +currentSavedAlarms->fileName()+ "\"" << endl;            //@B2
           currentSavedAlarms->close();                                      //@B2
            AlarmSetMutex.unlock();                                          //@B2
            return false;                                                    //@B2
         }                                                                   //@B2
  //    cout << "restore: staticVars" << endl;
         currentSavedAlarms->processStaticVars();                         //@B2
  //    cout << "restore: build Alarms" << endl;
         while (!currentSavedAlarms->atEOF()) {                             //@B2
            newAlarm=(Alarm *)currentSavedAlarms->restoreNewObject();     //@B2
            if (newAlarm)                                                 //@B2
               {                                                          //@B2
                Alarms.add(newAlarm);                                     //@B2
                mappedAlarms.add(newAlarm);                               //@B2
               }                                                          //@B2
         }                                                                //@B2
         currentSavedAlarms->close();                                     //@B2
         // switch current alarm file                                     //@B2
         temp=currentSavedAlarms;                                         //@B2
         currentSavedAlarms=alternateSavedAlarms;                         //@B2
         alternateSavedAlarms=temp;                                       //@B2
      }                                                                   //@B2
 //     cout << "restore reconcile" << endl;
      if (reconcile() && newAlarms==false) newAlarms=true;                //@B2                                             //@B2

      // now update overdue alarms not to be presented

      Alarm * currentAlarm;
      AlarmIdSet::Cursor mCursor(mappedAlarms);                                   //@B2

       // update alarm will return false if it needs to be removed from
       // mappedAlarms - it is expired
       // true if not
      // key setting to first until we found one to keep
//      cout << "restore: update 1" << endl;
//      listAlarms();
//      cout << "Alarms  mapped" << endl;
//      listMappedAlarms();
      for (mCursor.setToFirst();mCursor.isValid() &&
           !updatePersistentAlarm(mappedAlarms.elementAt(mCursor));
           mCursor.setToFirst()) {
             delete mappedAlarms.elementAt(mCursor);
             mappedAlarms.removeAt(mCursor);
           }
      //mCursor is setToFirst
//      cout << "restore: update 2" << endl;
      if (mCursor.isValid())
         for (mCursor.setToNext();mCursor.isValid();mCursor.setToNext()) {
       // if false must also remove from mappedAlarms, already removed
       // from Alarms
         if (!updatePersistentAlarm(mappedAlarms.elementAt(mCursor))) {
              delete mappedAlarms.elementAt(mCursor);
              mappedAlarms.removeAtAndResetCursor(mCursor);
         }
       }
   //    cout << "Alarms not mapped" << endl;
   //   listAlarms();
   //   cout << "Alarms  mapped" << endl;
   //   listMappedAlarms();
      // loop through alternateAlarms and add to alarms                   //@B2
      // how to tell start alarm this is post restore????                 //@B2
      // startAlarm will bring it up to date

//      cout << "restore: save" << endl;
      if (newAlarms && !saveAlarms()) success=false;
      AlarmSetMutex.unlock();
      return success;

}

// alarmSetMutexHeld
boolean Time_Adapter::saveAlarms()                                        //@B2
 {                                                                        //@B2
   // use currentSavedAlarms                                              //@B2
   if (!currentSavedAlarms->init(WRITE_PERSISTENT_FILE_MODE)) {            //@B2
      // actually this is normal condition if no persistent alarms        //@B2
       currentSavedAlarms->close();                                        //@B2
       return false;                                                      //@B2
   }
   if (!currentSavedAlarms->processStaticVars()) {
       currentSavedAlarms->close();
       ITHROW(PersistenceError((AString)"Error processing static variables in: "+                         //@B2
                  currentSavedAlarms->fileName()));                       //@B2
      }
   AlarmIdSet::Cursor mCursor(mappedAlarms);                              //@A4
   forCursor(mCursor) {
      if (!mappedAlarms.elementAt(mCursor)->isDummy() &&
          mappedAlarms.elementAt(mCursor)->isPersistent() &&
          !currentSavedAlarms->saveObject(mappedAlarms.elementAt(mCursor))) {
         currentSavedAlarms->close();
         ITHROW(PersistenceError((AString)"Error saving alarms in: "+                         //@B2
                  currentSavedAlarms->fileName()));                       //@B2
      }
   }
   currentSavedAlarms->close();
   if (!currentSavedAlarmsFileName.init(WRITE_PERSISTENT_FILE_MODE)) {    //@B2
       currentSavedAlarmsFileName.close();                                //@B2
       ITHROW(PersistenceError((AString)"Error opening: "+                         //@B2
                  currentSavedAlarmsFileName.fileName()));                //@B2
   }
   // write out to filename file
   if (!currentSavedAlarmsFileName.write(currentSavedAlarms->fileName())) { //@B2
       currentSavedAlarmsFileName.close();                                //@B2
       ITHROW(PersistenceError((AString)"Error saving alarms file name: "+                         //@B2
                  currentSavedAlarmsFileName.fileName()));                //@B2
   }
   currentSavedAlarmsFileName.close();                                    //@B2
   // switch files
   PersistentFile * temp=currentSavedAlarms;                              //@B2
   currentSavedAlarms=alternateSavedAlarms;                               //@B2
   alternateSavedAlarms=temp;                                             //@B2
   transactionFile.eraseContents();                                         //@B2
   return true;
}

// Read in time stamp file - only called at Start
boolean Time_Adapter::updateTimeStampInfo()
{
 eventSetMutex.lock();
 if (!currentEventStampFileName.init(READ_PERSISTENT_FILE_MODE)) {        //@B2
    // actually this is normal condition if no persistent alarms     //@B2
    // now what do I do?                                                  //@B2
    currentEventStampFileName.close();                                    //@B2
    // time stamp some number before this code is finished
    lastPersistentTimeStamp=860000000;                                         //@B2
    eventSetMutex.unlock();
    // normal condition
    return false;
 }                                                                        //@B2
 AString filepath;                                                        //@B2
 if (!currentEventStampFileName.read(filepath)) {                           //@B2
       currentEventStampFileName.close();
       eventSetMutex.unlock();
       ITHROW(PersistenceError((AString)"File: "+                         //@B2
                  currentEventStampFileName.fileName()));                 //@B2
 }
 currentEventStampFileName.close();                                       //@B2
 // set currentSavedAlarms to currently valid file                        //@B2
 PersistentFile * temp;                                                   //@B2
 if (filepath==alternateEventStamp->fileName()) {                         //@B2
     temp=currentEventStamp;                                              //@B2
     currentEventStamp=alternateEventStamp;                               //@B2
     alternateEventStamp=temp;                                            //@B2
  }
  else if (!filepath==currentSavedAlarms->fileName()) {                   //@B2
       eventSetMutex.unlock();
       ITHROW(PersistenceError(
          (AString)"File path for event stamp file corrupted: "+                         //@B2
                  filepath));                                              //@B2
  }
  if (!currentEventStamp->init(READ_PERSISTENT_FILE_MODE)) {              //@B2
        currentEventStamp->close();                                       //@B2
        eventSetMutex.unlock();
        ITHROW(PersistenceError(
          (AString)"Unable to read file: "+                               //@B2
                   currentEventStamp->fileName()));
   }                                                                      //@B2
  Time_Adapter_Missed_Event_Completes * last_time_exception;
  Alarm* newAlarm=NULL;
  AString keyword="";
  while (!currentEventStamp->atEOF() && keyword!="timestamp") {                                   //@B2
      currentEventStamp->read(keyword);
      if (keyword!="timestamp")
         newAlarm=(Alarm *)currentSavedAlarms->restoreNewObject(keyword); //@B2
      if (newAlarm) {                                                     //@B2
          last_time_exception=new Time_Adapter_Missed_Event_Completes;    //@B2
          last_time_exception->alarmTime=newAlarm->getAlarmTime().        //@B2
                                                 gettime_t();             //@B2
          last_time_exception->alarmUid=newAlarm->getAlarmUniqueId();     //@B2
          lastPersistentTimeExceptions.addAtEnd(last_time_exception);    //@B2
          delete newAlarm;
          newAlarm=NULL;
         }                                                                //@B2
  }

  if (currentEventStamp->atEOF() ||
      !currentEventStamp->read(lastPersistentTimeStamp)) {
     currentEventStamp->close();
     eventSetMutex.unlock();
        ITHROW(PersistenceError(
          (AString)"Error reading timestamp in: "+                               //@B2
                   currentEventStamp->fileName()));
  }
  currentEventStamp->close();
  temp=currentEventStamp;                                             //@B2
  currentEventStamp=alternateEventStamp;                              //@B2
  alternateEventStamp=temp;                                           //@B2
  eventSetMutex.unlock();
  return true;
}

//AlarmSetMutex held
boolean Time_Adapter::reconcile() {
// For set either increment duplicate count or add alarm
  // no transactions
  if (!transactionFile.init(READ_PERSISTENT_FILE_MODE)) {
        transactionFile.close();                                            //@B2
        return false;                                                     //@B2
  }
  boolean alarmSetChanged=false;
  long eventNumber;
  AString keyword="";
  Alarm * alarm;
  AlarmUniqueId alarmuid;
  boolean temp=false;
  boolean temp2=false;
  AString selector,level;
  while (!transactionFile.atEOF()) {
     transactionFile.read(keyword);
     if (keyword=="set") {
        if (alarm=(Alarm *)transactionFile.restoreNewObject()) {
           startPersistentAlarm(alarm,false);
           }
     }
     else if (keyword=="turnoff") {
        if (alarmuid.restore(transactionFile))
            findAndRemoveAlarm(alarmuid,temp,temp2);                      //@B2
     }
     else if (keyword=="reset") {
       transactionFile.read(eventNumber);                                 //@B3
       if (!transactionFile.readWholeString(selector)) {                  //@B3
          // do I want to throw an error here?
          transactionFile.readEnd();                                      //@B3
          return alarmSetChanged;                                         //@B3
       }
       if (transactionFile.readWholeString(level))                        //@B3         //@B2
        findAndRemoveAlarmsForSelector(selector,temp,eventNumber,level); //@B3
       transactionFile.readEnd();                                         //@B2
     }
     else transactionFile.readEnd();
  }
  transactionFile.close();
  return alarmSetChanged;
//don't clear out reconcile file - only done when saved
}

// AlarmSetMutex held when called
boolean Time_Adapter::updateTransactions(Alarm * thisAlarm) {
// this is for a set
// append mode
 // if before config, ignore
  if (!configDone) return true;                                           //@B3
  if (!transactionFile.init(APPEND_PERSISTENT_FILE_MODE) ||
      !transactionFile.write("set") ||                                    //@B2
      !transactionFile.saveObject(thisAlarm))                                                 //@B2
    {
     if (!persistenceError) {
        persistenceError=true;
        ITHROW(PersistenceError("TurnOff"));
     }
     transactionFile.close();                                            //@B2
     return false;                                                     //@B2
  }
  persistenceError=false;
  return transactionFile.close();
}

// AlarmSetMutex held when called
boolean Time_Adapter::updateTransactions(const AlarmUniqueId& alarmUid) {
// this is for a turnoff
// append mode
 // if before config, ignore
  if (!configDone) return true;                                           //@B3
  if (!transactionFile.init(APPEND_PERSISTENT_FILE_MODE) ||
     !transactionFile.write("turnoff") ||                                       //@B2
     !alarmUid.save(transactionFile) ||                                //@B2
     !transactionFile.writeEnd()) {                                       //@B2
     if (!persistenceError) {
        persistenceError=true;
        ITHROW(PersistenceError("TurnOff"));
     }
     transactionFile.close();                                            //@B2
     return false;                                                     //@B2
  }
  persistenceError=false;
  return transactionFile.close();
}

// AlarmSetMutex held when called
boolean Time_Adapter::updateTransactions(const AString& selector,
                       long eventNumber, const AString& level)
{
 // if before config, ignore
  if (!configDone) return true;                                           //@B3
// this is for a reset
// append mode
  if (!transactionFile.init(APPEND_PERSISTENT_FILE_MODE)  ||
      !transactionFile.write("reset") ||                                                        //@B2
      !transactionFile.write(eventNumber) ||                              //@B3                                         //@B2
      !transactionFile.writeWholeString(selector) ||                                                        //@B2
      !transactionFile.writeWholeString(level) ||                                                        //@B2
      !transactionFile.writeEnd())  {                                                       //@B2
     if (!persistenceError) {
        persistenceError=true;
        IAFactSet errorFacts;
        // need fact with effector name
        IAAtom atom;
        atom.setPredicate("Reset");
        atom.addTerm(selector);
        generateAlert("Persistent_Alarm_Error",IAFatal,&errorFacts,NULL);
     }
     transactionFile.close();                                            //@B2
     return false;                                                     //@B2
  }
  persistenceError=false;
  return transactionFile.close();
}

// AlarmSetMutex is held when this is called

boolean Time_Adapter::updatePersistentAlarm(Alarm * alarm) {
  // we are done reconciling so these are before any events
  alarm->setEventNumber(0);
  if (!alarm->isExpired()) return true;
  IATime alarmTime=alarm->getAlarmTime();
  IATime lastTime(lastPersistentTimeStamp);
  IATime now;
  boolean dummy=false;
  AlarmUniqueId auid=alarm->getAlarmUniqueId();
  AString intervalUnits=alarm->getIntervalUnits();
  int uIndex=findUnitsIndex(intervalUnits);
  if (uIndex==7) {
     cout << "update alarm: unknown interval units" << endl;
     // throw error here
     return true;
  }
  eventSetMutex.lock();
  AlarmSet::Cursor aCursor(Alarms);
  int i;
  for (i=0;i< lastPersistentTimeExceptions.size() &&
   ((Alarm *)(lastPersistentTimeExceptions.elementAt(i)))->getAlarmUniqueId()!=auid;i++) {}
  if (i < lastPersistentTimeExceptions.size()) {
     lastTime.settime_t(((Alarm *)(lastPersistentTimeExceptions.elementAt(i)))->
                                            getAlarmTime().gettime_t());
     // alarm notified but not event complete so back up one
      if (intervalUnits=="onetime" )
         lastTime=lastTime.addInterval(-1,"seconds");
      else  lastTime=lastTime.addInterval(-1,intervalUnits);
  }
  eventSetMutex.unlock();
  if (intervalUnits=="onetime") {
      if (alarmTime <= lastTime ||           // already presented
          alarmTime.addInterval(overdueCriteria[uIndex]->amount,
                   overdueCriteria[uIndex]->units) < now) {  // outside criteria
                                                          // to be presented
          // we need to remove this alarm
          if (!alarm->duplicatesAllowed() || !alarm->decrementDuplicateCount()) {
              if (Alarms.findElementForId(aCursor,
                                 alarm->getAlarmUniqueId(),
                                 alarm->getAlarmTime())) {
                 Alarms.removeAt(aCursor);
                 return false;
              }
              else {
                ITHROW(PersistenceError(
                     "Update Alarm: Can't find one time Alarm in set of alarms"));
             }
          }
       }
  }
  else {
    // is interval alarm and will be updated or removed
    // so need to remove from Alarms to do either
    if (Alarms.findElementForId(aCursor,
           alarm->getAlarmUniqueId(),
           alarm->getAlarmTime())) {
       Alarms.removeAt(aCursor);
    }
    else {
       ITHROW(PersistenceError(
                    "Update Alarm: Can't find interval Alarm in set of alarms"));
    }
    // increment alarm to update to when agent stopped
    // this should work the same if lastPersistentTimeStamp=0
    if (alarm->isRecurring(lastTime)) {
        alarm->update(lastTime);
    }
    else {
       return false;
    }
    // now increment until within presentation criteria
    // first subtract criteria interval from present time
    lastTime=now.addInterval(0-(overdueCriteria[uIndex]->amount),
                  overdueCriteria[uIndex]->units);
    // then increment until that time
    if ( alarm->isRecurring(lastTime)) alarm->update(lastTime);
    else  {
       return false;
    }
    Alarms.add(alarm);
  }
  // now all updated add it back
  return true;
}

boolean Time_Adapter::startPersistentAlarm(Alarm* a, boolean prePersistent) //@B2
   {
    IFUNCTRACE_DEVELOP();
    // we need to start a thread if there are no alarms now and we are enabled
    boolean addAlarm=false;
    AlarmIdSet::Cursor mCursor(mappedAlarms);

    // is this alarm in set already?
    AlarmUniqueId newAuid=a->getAlarmUniqueId();                          //@A9
    if (mappedAlarms.locateElementWithKey(newAuid,mCursor))                                //@A9
       {

         // if duplicates allowed increment count of alarm already in set
         if ((mappedAlarms.elementAt(mCursor))->duplicatesAllowed())
            {
             mappedAlarms.elementAt(mCursor)->incrementDuplicateCount();
             // new Event Number
             mappedAlarms.elementAt(mCursor)->setEventNumber(a->getEventNumber());
             addAlarm=true;
            }
         else if  (prePersistent &&
             duplicateHandling!=IA_TIME_IGNORE_PREPERSISTENT_DUPLICATES)
            {
             delete a;
            // throw exception - could convert fact set to string at this point or add alarmDate function
             ITHROW(DuplicateAlarmError("Duplicate alarm not allowed for this type of Alarm"));
            }
         // a is not added; delete it
         delete a;
       }
     // If its not expired or is recurring, add it
     else if (!prePersistent || (!a->isExpired() || a->isRecurring() ))
        {
         // add the alarm
         if (prePersistent && a->isExpired() && a->isRecurring()) a->incrementAlarm();     //@A5
         Alarms.add(a);
         mappedAlarms.add(a);
         addAlarm=true;
        }
     // we didn't duplicate or add it
     else
       {
        delete a;
       }
    return addAlarm;
   }

// false means it is no longer in Alarms, true means it is still to be presented
boolean Time_Adapter::updateDeferredAlarm(Alarm * alarm) {
  if (!alarm->isExpired()) return true;
  IATime alarmTime=alarm->getAlarmTime();
  IATime now,lastTime;
  AlarmUniqueId auid=alarm->getAlarmUniqueId();
  AString intervalUnits=alarm->getIntervalUnits();
  AlarmSet::Cursor aCursor(Alarms);
  int uIndex=findUnitsIndex(intervalUnits);
  if (uIndex==7) {
     cout << "update alarm: unknown interval units" << endl;
     // throw error here
     return true;
  }
  if (intervalUnits=="onetime") {
      if (alarmTime.addInterval(overdueCriteria[uIndex]->amount,
                   overdueCriteria[uIndex]->units) < now) {  // outside criteria
                                                          // to be presented
          // we need to remove this alarm
          if (!alarm->duplicatesAllowed() || !alarm->decrementDuplicateCount()) {
              if (Alarms.findElementForId(aCursor,
                                 alarm->getAlarmUniqueId(),
                                 alarm->getAlarmTime())) {
                 Alarms.removeAt(aCursor);
                 return false;
              }
              else {
                ITHROW(PersistenceError(
                     "Update Alarm: Can't find one time Alarm in set of alarms"));
             }
          }
       }
  }
  else {
    // now increment until within presentation criteria
    // first subtract criteria interval from present time
    // is interval alarm and will be updated or removed
    // so need to remove from Alarms to do either
    if (Alarms.findElementForId(aCursor,
        alarm->getAlarmUniqueId(),
        alarm->getAlarmTime())) {
        Alarms.removeAt(aCursor);
    }
    else {
          ITHROW(PersistenceError(
                   "Update Alarm: Can't find interval Alarm in set of alarms"));
    }
    lastTime=now.addInterval(0-(overdueCriteria[uIndex]->amount),
                  overdueCriteria[uIndex]->units);
    // then increment until that time
    if ( alarm->isRecurring(lastTime)) {
        alarm->update(lastTime);
    }
    else  {
       return false;
    }
    Alarms.add(alarm);
  }
  // now all updated add it back
  return true;
}

void Time_Adapter::eventComplete(const IAEventHeader &hdr)                //@B2
 {                                                                        //@B2
   if (strcmp(hdr.getDomain(),"AGENT")== 0 &&
       strcmp(hdr.getType(),"CONFIG")== 0)
     {
       // no alarms set on AGENT:CONFIG are valid
       AlarmSetMutex.lock();
       mappedAlarms.removeAll();
       Alarms.RemoveAll();
       AlarmSetMutex.unlock();
       try {
           if (persistenceOn) restoreAlarms();
       }
       catch (PersistenceError &e) {
           cout << "caught error" << endl;
           cout << e.name() << " : " << e.text() << endl;
           if (AlarmSetMutex.amIowner()) AlarmSetMutex.unlock();               //@B2
           if (eventSetMutex.amIowner()) eventSetMutex.unlock();               //@B2
        }
       configDone=true;
     }
   if ((AString)hdr.getDomain()!="Time") return;                          //@B2
   boolean fileOps=persistenceOn;                                         //@B2
   eventSetMutex.lock();                                                  //@B2
   if (fileOps &&                                                         //@B2
     !currentEventStamp->init(WRITE_PERSISTENT_FILE_MODE)) {              //@B2
      // actually this is normal condition if no persistent alarms        //@B2
       cout << "Unable to open file \""                                   //@B2
                 +currentEventStamp->fileName()+ "\"" << endl;            //@B2
       currentEventStamp->close();                                        //@B2
       fileOps=false;                                                     //@B2
   }                                                                      //@B2
   IAUniqueId uid=hdr.getId();                                            //@B2
   TimeAdapterEventInfo * eventInfo=NULL;                                 //@B2
   Alarm * completedAlarm=NULL;                                           //@B2
   TimeEventSet::Cursor eCursor(eventSet);                                //@B2
   TimeEventSet::Cursor removeCursor(eventSet);                           //@B2
   // initialize proper EventStampFile                                    //@B2
   // find eventInfo for uid but while looking write out missed alarms    //@B2
   for (eCursor.setToFirst();eCursor.isValid() && !eventInfo;             //@B2
                                          eCursor.setToNext()) {          //@B2
      if (eventSet.elementAt(eCursor)->eventId==uid) {                    //@B2
         eventInfo=eventSet.elementAt(eCursor);                           //@B2
         removeCursor.copy(eCursor);                                      //@B2
         completedAlarm=eventInfo->notifiedAlarm;                         //@B2       //@B2
         if (fileOps)  {                                                  //@B2
            currentEventStamp->write("timestamp");                        //@B2
            currentEventStamp->write(completedAlarm->                     //@B2
                                    getAlarmTime().gettime_t());          //@B2
            currentEventStamp->writeEnd();                                //@B2
         }                                                                //@B2
      }                                                                   //@B2
      else {                                                              //@B2
         if (fileOps)  {                                                  //@B2
            currentEventStamp->saveObject(eventSet.elementAt(eCursor)->  //@B2
                               notifiedAlarm);                           //@B2
         }                                                                //@B2
      } // else eventSet                                                  //@B2
   }  // for                                                              //@B2
   if (fileOps) {                                                         //@B2
      currentEventStamp->close();                                         //@B2
      if (!currentEventStampFileName.init(WRITE_PERSISTENT_FILE_MODE)) {  //@B2
        // actually this is normal condition if no persistent alarms      //@B2
         cout << "Unable to open file \""                                 //@B2
                  +currentEventStampFileName.fileName()+ "\"" << endl;   //@B2
         currentEventStampFileName.close();                              //@B2          //@B2
      }                                                                   //@B2
   // write out to filename file                                          //@B2
     currentEventStampFileName.write(currentEventStamp->fileName());      //@B2
     currentEventStampFileName.close();                                   //@B2
   }                                                                      //@B2
   // switch files if persistenceOn even if file op problem               //@B2
   if (persistenceOn) {                                                   //@B2
      PersistentFile * temp=currentEventStamp;                            //@B2
      currentEventStamp=alternateEventStamp;                              //@B2
      alternateEventStamp=temp;                                           //@B2
   }                                                                      //@B2
   if (completedAlarm) delete completedAlarm;                             //@B2
   if (eventInfo) delete eventInfo;                                       //@B2                    //@B2
   if (removeCursor.isValid()) eventSet.removeAt(removeCursor);           //@B2
   eventSetMutex.unlock();                                                //@B2
 }

IABool Time_Adapter::quiesce(const char *theSelector)                     //@A5
    {                                                                     //@A5
      IFUNCTRACE_DEVELOP();                                               //@A5
      disabledSelectors.addAtEnd(strdup(theSelector));                    //@A5
      return true;                                                        //@A5
    }                                                                     //@A5

IABool Time_Adapter::reset(const char *theSelector)                       //@A5
    {                                                                     //@A5
      IFUNCTRACE_DEVELOP();                                               //@A5
      AString selector(theSelector);                                      //@A5
      resetSelector(selector,0,"all");                                  //@A5@B3
      return true;                                                        //@A5
    }                                                                     //@A5

IABool Time_Adapter::restart(const char *theSelector)                     //@A5
    {                                                                     //@A5
      IFUNCTRACE_DEVELOP();                                               //@A5
      char * elemPtr;                                                     //@A5
      int i;                                                              //@A5
      for (i=0; i < disabledSelectors.size() &&                           //@A5
         (strcmp(theSelector,(char*)disabledSelectors.elementAt(i))!=0);  //@A5
         i++) {}                                                          //@A5
      // if we found a match update the alarm times                         @A5
      // remove the disabledSelector from the list                          @A5
      // else just ignore - want to allow restarts without first quiesces   @A5
      if (i < disabledSelectors.size())    // we found a match            //@A5
        {                                                                 //@A5
         elemPtr=(char*)disabledSelectors.elementAt(i);                   //@A5
// not sure we really want to do this, alarms were already updated at
// set time
         if (elemPtr)                                                     //@A5
            {                                                             //@A5
             disabledSelectors.removeMatching(elemPtr);                   //@A5
             free(elemPtr);                                               //@A5
            }                                                             //@A5
        }                                                                 //@A5
      // also unmark marked alarms
      AlarmSetMutex.lock();
      AString selector(theSelector);                                      //@A5
      Alarm * dummy=new AlarmOneTime();                                   //@A9
      AlarmIdSet::Cursor mCursor(mappedAlarms);                           //@A9
      dummy->setSelector(selector);                                       //@A9
      // find dummy's location in set
      mappedAlarms.locateOrAddElementWithKey(dummy,mCursor);              //@A9
      // loop through and unmark all the elements for this selector       //@A9
      for (;mappedAlarms.setToNext(mCursor) &&                              //@A9
             mappedAlarms.elementAt(mCursor)->getSelector()==selector;)   //@A9
       {
         mappedAlarms.elementAt(mCursor)->setDeferredOff();                  //@B3
         if (!updateDeferredAlarm(mappedAlarms.elementAt(mCursor))) {
              delete mappedAlarms.elementAt(mCursor);
              mappedAlarms.removeAtAndResetCursor(mCursor);
         }
      }
      // need to signal now
      AlarmChangeEvent.signal();
      AlarmSetMutex.unlock();
      return true;                                                        //@A5
    }                                                                     //@A5

// quiesce and remove all alarms
void Time_Adapter::shutdown()                                             //@A5
    {
     IFUNCTRACE_DEVELOP();
      disable();
      AlarmSetMutex.lock();
      mappedAlarms.removeAll();
      Alarms.RemoveAll();
      AlarmChangeEvent.signal();
      if (currentSavedAlarms) delete currentSavedAlarms;
      if (alternateSavedAlarms) delete alternateSavedAlarms;
      AlarmSetMutex.unlock();
      int i;
      for (i=0;i<=6;i++) if (overdueCriteria[i]) delete overdueCriteria[i];
      eventSetMutex.lock();
      for (i=0;i < lastPersistentTimeExceptions.size();i++)
          if (lastPersistentTimeExceptions.elementAt(i))
               delete lastPersistentTimeExceptions.elementAt(i);
      if (currentEventStamp) delete currentEventStamp;
      if (alternateEventStamp) delete alternateEventStamp;
      eventSetMutex.unlock();
    }

IABool Time_Adapter::isEnabled()
    {
      IFUNCTRACE_DEVELOP();
      return enabledForNotify;
    }

void Time_Adapter::enable()
    {
      IFUNCTRACE_DEVELOP();
      enabledForNotify=true;
    }

void Time_Adapter::disable()
    {
     IFUNCTRACE_DEVELOP();
      enabledForNotify=false;
    }

// for testing
#ifdef IA_TIMERTEST
void Time_Adapter::notify(IATriggerEvent &event) throw(IAError)
     {
        // throws an error on a notify of a quiesced selector - used
        // for testing quiesce/restart with effector
        AString selector=event.getHeader().getSelector();
         int i;
         for (i=0; i < disabledSelectors.size() &&          //@A5
                 (strcmp((char*)selector,     //@A5
                         (char*)disabledSelectors.elementAt(i))!=0);     //@A5
                      i++) {}                                            //@A5
                      // is there a match?                                     @A5
         if (i < disabledSelectors.size()) // a match         //@A5
                         throw IAError(999,"quiesced");

        IAAdapter::notify(event);
     }
#endif

void Time_Adapter::listAlarms(const IAAtom& atom)  // Not const because need to acquire lock
   {
    IFUNCTRACE_DEVELOP();
    cout << "Alarms currently set:" << endl;
       AlarmSetMutex.lock();
       cout << Alarms;
       AlarmSetMutex.unlock();
   }


void Time_Adapter::listAlarms()  // Not const because need to acquire lock
   {
    IFUNCTRACE_DEVELOP();
    cout << "Alarms currently set:" << endl;
    if (!AlarmSetMutex.amIowner()) {
       AlarmSetMutex.lock();
       cout << Alarms;
       AlarmSetMutex.unlock();
    }
    else cout << Alarms;
   }

void Time_Adapter::listMappedAlarms()  // Not const because need to acquire lock
   {
    IFUNCTRACE_DEVELOP();
    cout << "mapped Alarms currently set:" << endl;
    AlarmIdSet::Cursor mCursor(mappedAlarms);
    if (!AlarmSetMutex.amIowner()) {
       AlarmSetMutex.lock();
       forCursor(mCursor)
       cout << *mappedAlarms.elementAt(mCursor) << endl;
       AlarmSetMutex.unlock();
    }
    else {
       forCursor(mCursor)
       cout << *mappedAlarms.elementAt(mCursor) << endl;
    }
   }

void Time_Adapter::listAlarmTimes(const AString& selector)                //@A7
   {                                                                      //@A4
    IFUNCTRACE_DEVELOP();                                                 //@A4
    boolean unlock=false;
    cout << "Alarms currently set for selector " << selector << " : "<< endl; //@A7
    if (!AlarmSetMutex.trylock()) {
       AlarmSetMutex.lock();                                                 //@A4
       unlock=true;
    }
    AlarmSet::Cursor aCursor(Alarms);                                     //@A4
    if (!Alarms.setToFirst(aCursor)) {                                    //@B2
       if (unlock) AlarmSetMutex.unlock();                                            //@B2
       return;                                                            //@B2
    }
    for (;!(Alarms.isLast(aCursor));Alarms.setToNext(aCursor))            //@A4
     {                                                                    //@A4
      if (selector == Alarms.elementAt(aCursor)->getSelector())           //@A7
        cout << Alarms.elementAt(aCursor)->getAlarmId() << " " <<         //@A7
          Alarms.elementAt(aCursor)->getEventTimeStamp() << endl;         //@A4
     }                                                                    //@A4
    // do last one                                                        //@A4
    if (selector == Alarms.elementAt(aCursor)->getSelector())             //@A7
       cout << Alarms.elementAt(aCursor)->getAlarmId() << " " <<          //@A7
            Alarms.elementAt(aCursor)->getEventTimeStamp() << endl;       //@A4
    if (unlock) AlarmSetMutex.unlock();                                   //@A4
   }                                                                      //@A4

void Time_Adapter::performAction(IAProcToken token,
                   const IAEventHeader &hdr, const char *bindingStr)
{
  IFUNCTRACE_DEVELOP();
  try
   {
    IAAtom atom;                                                          //@A2
    if (!(strcmp(bindingStr,"")==0 || bindingStr==NULL))
      {                                                                   //@A2
        atom.setKIF(bindingStr);                                          //@A6
      }                                                                   //@A2
    else                                                                  //@A2
       // atom error
     {
      cout << "Unrecoverable error: empty binding string on Time_Adapter " //@A6      //@A1
           << "performAction. Semantic Token= " << (long) token << endl;   //@A6                         //@A1
      return;
     }
    AString selector=hdr.getSelector();                                     //@A2
#ifdef TEST_ID
     if (selector=="") // do test stuff
       {
#ifdef NEW_NUM
        selector=tempScope+(AString)(userNum++);
#else
        selector=tempScope+(AString)(userNum);
#endif
       }
#endif
     switch(token)
     {
        case IA_LISTALARMS:
          listAlarms(atom);
          break;
        case IA_LISTALARMTIMES:                                           //@A7
          listAlarmTimes(selector);                                       //@A7
          break;                                                          //@A7
        case IA_SETINTERVALALARM:
          setIntervalAlarm(atom,selector,hdr);                            //@B3
          break;
        case IA_TURNOFFALARM:
          turnOffAlarm(AlarmUniqueId(selector,'u',atom.getStringTerm(1)));              //@A9
          break;
        case IA_SETBOUNDEDINTERVALALARM:
          setBoundedIntervalAlarm(atom,selector,hdr);                     //@B3
          break;
        case IA_SETSTARTBOUNDEDINTERVALALARM:
          setStartBoundedIntervalAlarm(atom,selector,hdr);                //@B3
          break;
        case IA_SETDAILYALARM:
          setDailyAlarm(atom,selector,hdr);                               //@B3
          break;
        case IA_SETSPECIFICDAILYALARM:
          setSpecificDailyAlarm(atom,selector,hdr);                       //@B3
          break;
        case IA_SETALARMAT:
          setAlarmAt(atom,selector,hdr);                                  //@B3
          break;
        case IA_SETENDBOUNDEDINTERVALALARM:
          setEndBoundedIntervalAlarm(atom,selector,hdr);                  //@B2@A2
          break;
        case IA_SETINTERVALALARMRANDOMSTART:                              //@A2
          setIntervalAlarmRandomStart(atom,selector,hdr);                 //@B3
          break;                                                          //@A2
        case IA_SETRANDOMSTARTBOUNDEDINTERVALALARM:                       //@B1
          setRandomStartBoundedIntervalAlarm(atom,selector,hdr);          //@B3
          break;                                                          //@A2
        case IA_SETRANDOMBOUNDEDINTERVALALARM:                            //@B1
          setRandomBoundedIntervalAlarm(atom,selector,hdr);               //@B3
          break;                                                          //@B1
        case IA_RESET:                                                    //@A2
          resetSelector(selector,hdr.eventNumber(),
                                   atom.getStringTerm(1));                //@B3@A5
          break;                                                          //@A2
        case IA_TESTSETTIME:                                              //@A2
          testSetTime(atom);                                              //@A2
          break;                                                          //@A2
        case IA_TESTANY:                                                  //@A5
          testAny(atom,selector);                                         //@A5
          break;                                                          //@A5
        case IA_TIMERCHECKPOINT:                                          //@B2
          checkPoint(atom,selector);                                      //@B2
          break;                                                          //@B2
        case IA_SETOVERDUEALARMPRESENTATIONCRITERIA:                      //@B2
            if (strcmp(hdr.getDomain(),"AGENT")!= 0 ||                    //@B3
                strcmp(hdr.getType(),"CONFIG")!=0)                       //@B3
                return;
          setOverdueAlarmPresentationCriteria(atom,selector);             //@B2
          break;                                                          //@B2
        case IA_SETDUPLICATEHANDLING:                                     //@B2
            if (strcmp(hdr.getDomain(),"AGENT")!= 0 ||                    //@B3
                strcmp(hdr.getType(),"CONFIG")!=0)                       //@B3
                return;
          setDuplicateHandling(atom,selector);                            //@B2
          break;                                                          //@B2
        case IA_SETPERSISTENTFILENAME:                                    //@B3
            if (strcmp(hdr.getDomain(),"AGENT")!= 0 ||                    //@B3
                strcmp(hdr.getType(),"CONFIG")!=0)                       //@B3
                return;
          setPersistentFileName(atom);
          break;
        case IA_SETPERSISTENTCATEGORIES:                                      //@B3
            if (strcmp(hdr.getDomain(),"AGENT")!= 0 ||                    //@B3
                strcmp(hdr.getType(),"CONFIG")!=0)                       //@B3
                return;
          setPersistentCategories(atom.getStringTerm(1));                     //@B3
          break;                                                          //@B3
     }
   }
   catch(IAError & I)                                                     //@A6
   {                                                                      //@A6
    cout << I.errormsg() << endl;                                         //@A6
   }                                                                      //@A6
   // timeadapter error
   catch (PersistenceError &I )
   {
     if (AlarmSetMutex.trylock()) AlarmSetMutex.unlock();               //@B2
     IAFactSet errorFacts;
     // need fact with effector name
     IAAtom atom,metaAtom;
     atom.setKIF(bindingStr);
     atom.setPredicate(effectorIdentifiers[(long)token]);
     metaAtom.setPredicate("Effector");
     metaAtom.addTerm(atom.getKif());
     errorFacts.add(metaAtom);
     generateAlert("Persistent_Alarm_Error",IAFatal,&errorFacts,&hdr);
   }
   catch(IException & I)
   {
    if (AlarmSetMutex.trylock()) AlarmSetMutex.unlock();               //@B2
    cout << "Unrecoverable Error: " << I.name() <<  ": " << I.text() << endl;
    cout << "Semantic Token= " << (long) token                            //@A1
         << "   Atom= " << (AString)bindingStr << endl;
   }
}

// sensors

IAFactSet Time_Adapter::answerQuery(IAProcToken token,
                                    const IAEventHeader &hdr,
                                    const char *bindingStr)
   {
    IFUNCTRACE_DEVELOP();
    IAFactSet FS;
    try
     {
       IAAtom atom;                                                       //@A2
       if (!(strcmp(bindingStr,"")==0 || bindingStr==NULL))
         {                                                                //@A2
          atom.setKIF(bindingStr);                                        //@A6
         }                                                                //@A2
       else                                                              //@A2
          // atom error
        {
          cout << "Unrecoverable error: empty binding string on Time_Adapter " //@A6      //@A1
           << "performAction. Semantic Token= " << (long) token << endl;   //@A6                         //@A1
         }
       switch(token)
       {
          case IA_TIMEAFTER:
            return timeAfter(atom);
            break;
          case IA_CURRENTIMEAFTER:
            return currentTimeAfter(atom);
            break;
          case IA_TIMEBEFORE:
            return timeBefore(atom);
            break;
          case IA_CURRENTIMEBEFORE:
            return currentTimeBefore(atom);
            break;
          case IA_TIMEBETWEEN:
            return timeBetween(atom);
            break;
          case IA_CURRENTIMEBETWEEN:
            return currentTimeBetween(atom);
            break;
          case IA_TIMEEQUAL:
            return timeEqual(atom);
            break;
          case IA_CURRENTIMEEQUAL:
            return currentTimeEqual(atom);
            break;
          case IA_TIMENOTEQUAL:
            return timeNotEqual(atom);
            break;
          case IA_CURRENTIMENOTEQUAL:
            return currentTimeNotEqual(atom);
            break;
          case IA_ISCURRENTDAY:
            return isCurrentDay(atom);
            break;
          case IA_ISCURRENTWEEKDAY:
            return isCurrentWeekday(atom);
            break;
          case IA_ISWEEKEND:
            return isWeekend(atom);
            break;
          case IA_ISCURRENTWEEKEND:
            return isCurrentWeekend(atom);
            break;
          case IA_DATEANDTIME:
            return dateAndTime(atom);
            break;
          case IA_DATE:
            return currentDate(atom);
            break;
          case IA_TIME:
            return currentTime(atom);
            break;
          case IA_DAYOFWEEK:
            return dayOfWeek(atom);
            break;
          case IA_ADDTIMEINTERVAL:
            return addTimeInterval(atom);
            break;
          case IA_DETERMINETIMEINTERVAL:
            return determineTimeInterval(atom);
            break;
       }
     }
   catch(IAError & I)                                                     //@A6
   {                                                                      //@A6
    cout << I.errormsg() << endl;                                         //@A6
   }                                                                      //@A6
   // timeadapter error
    catch(IException & I)
     {
       cout << "Unrecoverable Error: " << I.name() <<  ": " << I.text() << endl;
       cout << "Semantic Token= " << (long) token                         //@A1
            << "   Atom= " << (AString)bindingStr << endl;
     }
    return FS;
   }


IAFactSet Time_Adapter::timeAfter(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,4,"String");                                   @A2
       IATime t1(atom.getStringTerm(1),atom.getStringTerm(2)),            //@A6
              t2(atom.getStringTerm(3),atom.getStringTerm(4));            //@A6
       if (t1>t2)
            FS.add(atom);                                                 //@A6
       return FS;
    }

IAFactSet Time_Adapter::currentTimeAfter(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,2,"String");                                   @A2
       IATime t1,t2(atom.getStringTerm(1),atom.getStringTerm(2));         //@A6
       if (t1 > t2) FS.add(atom);                                         //@A6
       return FS;
    }

IAFactSet Time_Adapter::timeBefore(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,4,"String");                                   @A2
       IATime t1(atom.getStringTerm(1),atom.getStringTerm(2)),            //@A6
              t2(atom.getStringTerm(3),atom.getStringTerm(4));            //@A6
       if (t1 < t2) FS.add(atom);                                         //@A6
       return FS;
    }

IAFactSet Time_Adapter::currentTimeBefore(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,2,"String");                                   @A2
       IATime t1,t2(atom.getStringTerm(1),atom.getStringTerm(2));         //@A6
       if (t1 < t2) FS.add(atom);                                         //@A6
       return FS;
    }

IAFactSet Time_Adapter::timeBetween(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,6,"String");                                   @A2
       IATime t1(atom.getStringTerm(1),atom.getStringTerm(2)),            //@A6
              t2(atom.getStringTerm(3),atom.getStringTerm(4)),            //@A6
              t3(atom.getStringTerm(5),atom.getStringTerm(6));            //@A6
       if (t1<t2 && t2<t3) FS.add(atom);                                  //@A6
       return FS;
    }

IAFactSet Time_Adapter::currentTimeBetween(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       //checkTermList(atom,4,"String");                                    @A2
       IATime t1(atom.getStringTerm(1),atom.getStringTerm(2));            //@A6
       IATime t2;
       IATime t3(atom.getStringTerm(3),atom.getStringTerm(4));            //@A6
       if (t1<t2 && t2<t3) FS.add(atom);                                  //@A6
       return FS;
    }

IAFactSet Time_Adapter::timeEqual(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,4,"String");                                   @A2
       IATime t1(atom.getStringTerm(1),atom.getStringTerm(2)),            //@A6
              t2(atom.getStringTerm(3),atom.getStringTerm(4));            //@A6
       if (t1==t2) FS.add(atom);                                          //@A6
       return FS;
    }

IAFactSet Time_Adapter::currentTimeEqual(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,2,"String");                                   @A2
       IATime t1,t2(atom.getStringTerm(1),atom.getStringTerm(2));         //@A6
       if (t1==t2) FS.add(atom);                                          //@A6
       return FS;
    }

IAFactSet Time_Adapter::timeNotEqual(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,4,"String");                                   @A2
       IATime t1(atom.getStringTerm(1),atom.getStringTerm(2)),            //@A6
              t2(atom.getStringTerm(3),atom.getStringTerm(4));            //@A6
       if (t1!=t2) FS.add(atom);                                          //@A6
       return FS;
    }

IAFactSet Time_Adapter::currentTimeNotEqual(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,2,"String");                                   @A2
       IATime t1,t2(atom.getStringTerm(1),atom.getStringTerm(2));         //@A6
       if (t1!=t2) FS.add(atom);                                          //@A6
       return FS;
    }

IAFactSet Time_Adapter::isCurrentDay(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,1,"String");                                   @A2
       IATime t;
       if (atom.getStringTerm(1)==(AString)t.weekday()) FS.add(atom);    //@A6
       return FS;
    }

IAFactSet Time_Adapter::isWeekday(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,1,"String");                                   @A2
       AString Weekday(atom.getStringTerm(1));                            //@A6
       if (!(Weekday.indexOfAnyOf("12345")==-1)) FS.add(atom);            //@A6
       return FS;
    }

IAFactSet Time_Adapter::isCurrentWeekday(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,1,"Symbol");                                   @A2
       IATime t;
       if (t.weekday()> 0 && t.weekday()< 6) FS.add(atom);                //@A6
       return FS;
    }

IAFactSet Time_Adapter::isWeekend(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       // checkTermList(atom,1,"String");                                   @A2
       AString Weekday(atom.getStringTerm(1));                            //@A6
       if (!(Weekday.indexOfAnyOf("06")==-1)) FS.add(atom);               //@A6
       return FS;
    }

IAFactSet Time_Adapter::isCurrentWeekend(const IAAtom& atom)
    {
       IFUNCTRACE_DEVELOP();
       IAFactSet FS;
       //checkTermList(atom,1,"Symbol");                                    @A2
       IATime t;
       if (t.weekday()== 0 || t.weekday()== 6) FS.add(atom);              //@A6
       return FS;
    }

// Non-Boolean Sensors

IAFactSet Time_Adapter::dateAndTime(const IAAtom& atom)
 {
  IFUNCTRACE_DEVELOP();
  IAAtom timeAndDateFact;                                                 //@A2
  // checkTermList(atom,2,"Lvar");                                          @A2
  IATime t;
  timeAndDateFact.setPredicate(atom.getPredicate());                   //@B4@A6
  IAFactSet FS;
  // create correct fact                                                  //@B4
  timeAndDateFact.addTerm(t.getDate());                                   //@B4
  timeAndDateFact.addTerm(t.getTime());                                   //@B4
  // compare correct fact and input atom                                  //@B4
  if ((atom.isLogicalVariable(1) ||                                    //@A6
    strcmp(atom.getStringTerm(1),                                     //@B4@A2
    timeAndDateFact.getStringTerm(1))==0) &&                           //@A6
    (atom.isLogicalVariable(2) ||                                      //@A6
    strcmp(atom.getStringTerm(2),                                      //@A6
    timeAndDateFact.getStringTerm(2))==0))                            //@B4@A6
        FS.add(timeAndDateFact);                                       //@A6
  return FS;
 }

IAFactSet Time_Adapter::currentDate(const IAAtom& atom)
 {
  IFUNCTRACE_DEVELOP();
  IAAtom dateFact;                                                       //@B4@A2
  // checkTermList(atom,1,"Lvar");                                          @A2
  IATime t;
  dateFact.setPredicate(atom.getPredicate());                            //@B4@A6
  IAFactSet FS;
  // create correct fact                                                  //@B4
  dateFact.addTerm(t.getDate());                                          //@B4
  // compare correct fact and input atom                                  //@B4
  if (atom.isLogicalVariable(1) ||                                        //@A6
     strcmp(atom.getStringTerm(1),dateFact.getStringTerm(1))==0)         //@B4@A6
          FS.add(dateFact);                                               //@A6
  return FS;
 }

IAFactSet Time_Adapter::currentTime(const IAAtom& atom)
 {
  IFUNCTRACE_DEVELOP();
  IAAtom timeFact;                                                       //@B4@A3
  // checkTermList(atom,1,"Lvar");                                        //@A2
  IATime t;
  timeFact.setPredicate(atom.getPredicate());                            //@B4@A6
  IAFactSet FS;
  // create correct fact                                                  //@B4
  timeFact.addTerm(t.getTime());                                          //@B4
  // compare correct fact and input atom                                  //@B4
  if (atom.isLogicalVariable(1) ||                                        //@A6
     strcmp(atom.getStringTerm(1),timeFact.getStringTerm(1))==0)         //@B4@A6
          FS.add(timeFact);                                               //@A6
  return FS;
 }

IAFactSet Time_Adapter::dayOfWeek(const IAAtom& atom)
 {
  IFUNCTRACE_DEVELOP();
  IAAtom dayFact;                                                        //@B4@A2
  // checkTermList(atom,1,"Lvar");                                          @A2
  IATime t;
  dayFact.setPredicate(atom.getPredicate());                             //@B4@A6
  IAFactSet FS;
  // create correct fact                                                  //@B4
  dayFact.addTerm((AString)t.weekday());                               //@B3@B4
  // compare correct fact and input atom                                  //@B4
  if (atom.isLogicalVariable(1) ||                                        //@A6
     strcmp(atom.getStringTerm(1),dayFact.getStringTerm(1))==0)          //@B4@A6
        FS.add(dayFact);                                                  //@A6
  return FS;
 }

IAFactSet Time_Adapter::addTimeInterval(const IAAtom& atom)
 {
  IFUNCTRACE_DEVELOP();
  IAAtom addIntervalFact;                                                //@B4@A2
  // checkTermList(atom,6,"String String Integer String Lvar Lvar");        @A2
  IATime t(atom.getStringTerm(1),atom.getStringTerm(2));                  //@A6
  addIntervalFact.setPredicate(atom.getPredicate());                     //@B4@A6
  IAFactSet FS;
  t=t.addInterval(atom.getIntegerTerm(3),atom.getStringTerm(4));          //@A6
  // create correct fact                                                  //@B4
  addIntervalFact.addTerm(atom.getStringTerm(1));                        //@B4@A6
  addIntervalFact.addTerm(atom.getStringTerm(2));                        //@B4@A6
  addIntervalFact.addTerm(atom.getIntegerTerm(3));                       //@B4@A6
  addIntervalFact.addTerm(atom.getStringTerm(4));                        //@B4@A6
  addIntervalFact.addTerm(t.getDate());                                   //@B4
  addIntervalFact.addTerm(t.getTime());                                   //@B4
  // compare correct fact and input atom                                  //@B4
  if ((atom.isLogicalVariable(5) ||                                       //@A6
    strcmp(atom.getStringTerm(5),                                        //@B4@A6
    addIntervalFact.getStringTerm(5))==0) &&                              //@A6
    (atom.isLogicalVariable(6) ||                                         //@A6
    strcmp(atom.getStringTerm(6),                                        //@B4@A6
    addIntervalFact.getStringTerm(6))==0))                               //@B4@A6
               FS.add(addIntervalFact);                                   //@A6
  return FS;
 }

IAFactSet Time_Adapter::determineTimeInterval(const IAAtom& atom)
 {
  IFUNCTRACE_DEVELOP();
  IAAtom timeIntervalFact;                                                //@A2
  // checkTermList(atom,5,"String String String String Lvar");              @A2
  IATime t1(atom.getStringTerm(1),atom.getStringTerm(2));                  //@A6
  IATime t2(atom.getStringTerm(3),atom.getStringTerm(4));                  //@A6
  timeIntervalFact.setPredicate(atom.getPredicate());                    //@B4@A6
  IAFactSet FS;
  long interval=t1.diffTime(t2);
  // create correct fact                                                  //@B4
  timeIntervalFact.addTerm(atom.getStringTerm(1));                       //@B4@A6
  timeIntervalFact.addTerm(atom.getStringTerm(2));                       //@B4@A6
  timeIntervalFact.addTerm(atom.getStringTerm(3));                       //@B4@A6
  timeIntervalFact.addTerm(atom.getStringTerm(4));                       //@B4@A6
  timeIntervalFact.addTerm((int)interval);                                //@B4
  // compare correct fact and input atom                                  //@B4
  if (atom.isLogicalVariable(5) ||                                       //@B4@A6
     atom.getIntegerTerm(5)==timeIntervalFact.getIntegerTerm(5))         //@B4@A6
        FS.add(timeIntervalFact);                                         //@A6
  return FS;
 }

// Perform Actions - miscellaneous


void Time_Adapter::checkPoint(const IAAtom& atom,                         //@B2
                              const AString& selector)                    //@B2
{
  if (!persistenceOn) throw(IAError(99,(AString)"Unable to checkpoint time adapter"+
                      " when not in persistence mode"));
  AlarmSetMutex.lock();
  saveAlarms();
  AlarmSetMutex.unlock();
}

void Time_Adapter::setDuplicateHandling(const IAAtom& atom,               //@B2
                                        const AString& selector)          //@B2
{
 AString symbol=atom.getSymbolTerm(1);                                    //@B2
 if (symbol=="Ignore_Duplicates")                                         //@B2
    duplicateHandling=IA_TIME_IGNORE_ALL_DUPLICATES;                      //@B2
 else if (symbol=="Flag_Duplicates")                                      //@B2
    duplicateHandling=IA_TIME_DONT_IGNORE_DUPLICATES;                     //@B2
 else duplicateHandling=IA_TIME_IGNORE_PREPERSISTENT_DUPLICATES;          //@B2
}

void Time_Adapter::setOverdueAlarmPresentationCriteria(const IAAtom& atom, //@B2
                                              const AString& selector)    //@B2
{
  // sets variables only used at start
  // so null effect if not issued at AGENT:STARTING
  AString alarmUnits=atom.getStringTerm(1);
  int i,j;
  // onetime - minutes are valid for alarm units
  for (i=0;i<=5 && unitsIndex[i]!=alarmUnits;i++) {}
  if (i>5) throw(IAError(99,(AString)"Invalid alarm units \""+alarmUnits+"\""));
  int value=atom.getIntegerTerm(2);
  AString criteriaUnits=atom.getStringTerm(3);
  // years - seconds are valid for criteria
  for (j=1;j<=6 && unitsIndex[j]!=criteriaUnits;j++) {}
  if (j > 6) throw(IAError(99,(AString)"Invalid units for overdue criteria \""+
                   criteriaUnits+"\""));
  else if ( maxCriteriaValue[i][j] < value) throw(IAError( 9,(AString)"Invalid "+
                        "overdue criteria \""+(AString)value+" "+criteriaUnits +
                        " for alarm units \""+alarmUnits+ "\""));
  overdueCriteria[i]->amount=value;
  overdueCriteria[i]->units=criteriaUnits;
}

void Time_Adapter::setPersistentCategories(const AString& categoryList)
{
  // remove default value if set
  if (persistentCategoriesSet) {
       return;
  }
  persistentCategoriesSet=true;
  for (int i=1;i<=categoryList.numWords();i++)
    {
     persistentCategories.addAtEnd(strdup((const char *)categoryList.word(i)));
    }
}

boolean Time_Adapter::isPersistentCategory(const AString& category) const
{
  int i;
  for (i=0;i < persistentCategories.size() &&
          strcmp((char *)persistentCategories.elementAt(i),(const char *)category)!=0;i++) {}
  if (i==persistentCategories.size()) return false;
  else return true;
}

// Perform Actions - Set Alarms

void Time_Adapter::setIntervalAlarm(const IAAtom& atom,
                                    const AString& selector,                //@A2
                                    const IAEventHeader& hdr)               //@B3
     {
       IFUNCTRACE_DEVELOP();
       AlarmInterval * newAlarm=                                          //@B3
                  new AlarmInterval(atom.getStringTerm(3),
                                       atom.getIntegerTerm(1),               //@A9
                                       atom.getStringTerm(2),             //@A6
                                       selector);
       newAlarm->setEventNumber(hdr.eventNumber());                       //@B3
       // for now, category is init if set on AGENT:STARTING (pre start )    //@B3
       if (strlen(atom.getStringTerm(4)) > 0 )
          newAlarm->setAlarmCategory(atom.getStringTerm(4));
       else if (strcmp(hdr.getDomain(),"AGENT")== 0 &&                       //@B3
                strcmp(hdr.getType(),"STARTING")==0)                         //@B3
            newAlarm->setAlarmCategory("init");                              //@B3
       else newAlarm->setAlarmCategory("other");                             //@B3
       newAlarm->setPersistent(isPersistentCategory(newAlarm->getAlarmCategory())); //@B3
       startAlarm(newAlarm);                                              //@B3
     }

void Time_Adapter::setBoundedIntervalAlarm(const IAAtom& atom,
                                    const AString& selector,              //@A2
                                    const IAEventHeader& hdr)               //@B3
     {
       IFUNCTRACE_DEVELOP();
       AlarmInterval * newAlarm=                                          //@B3
              new AlarmBoundedInterval(atom.getStringTerm(7),
                             atom.getIntegerTerm(1),                      //@A9
                             atom.getStringTerm(2),                       //@A6
                             atom.getStringTerm(3),    // start date      //@A6
                             atom.getStringTerm(4),     // start time     //@A6
                             atom.getStringTerm(5),     // end date       //@A6
                             atom.getStringTerm(6),      // end time      //@A6
                             selector);
       newAlarm->setEventNumber(hdr.eventNumber());                       //@B3
       // for now, category is init if set on AGENT:STARTING (pre start )    //@B3
       if (strlen(atom.getStringTerm(8)) > 0)
          newAlarm->setAlarmCategory(atom.getStringTerm(8));
       else if (strcmp(hdr.getDomain(),"AGENT")== 0 &&                       //@B3
                strcmp(hdr.getType(),"STARTING")==0)                         //@B3
                     newAlarm->setAlarmCategory("init");                     //@B3
       else newAlarm->setAlarmCategory("other");                             //@B3
       newAlarm->setPersistent(isPersistentCategory(newAlarm->getAlarmCategory())); //@B3
       startAlarm(newAlarm);                                              //@B3
     }

void Time_Adapter::setStartBoundedIntervalAlarm(const IAAtom& atom,
                                    const AString& selector,              //@A2
                                    const IAEventHeader& hdr)               //@B3
     {
       IFUNCTRACE_DEVELOP();
       AlarmInterval * newAlarm=                                          //@B3
                     new AlarmInterval(atom.getStringTerm(5),
                          atom.getIntegerTerm(1),                         //@A9
                          atom.getStringTerm(2),                          //@A6
                          atom.getStringTerm(3),    // start date         //@A6
                          atom.getStringTerm(4),    // start time         //@A6
                          selector);
       newAlarm->setEventNumber(hdr.eventNumber());                       //@B3
       // for now, category is init if set on AGENT:STARTING (pre start )    //@B3
       if (strlen(atom.getStringTerm(6)) > 0 )
          newAlarm->setAlarmCategory(atom.getStringTerm(6));
       else if (strcmp(hdr.getDomain(),"AGENT")== 0 &&                       //@B3
                strcmp(hdr.getType(),"STARTING")==0)                         //@B3
                          newAlarm->setAlarmCategory("init");                //@B3
       else newAlarm->setAlarmCategory("other");                             //@B3
       newAlarm->setPersistent(isPersistentCategory(newAlarm->getAlarmCategory())); //@B3
       startAlarm(newAlarm);                                              //@B3
     }


void Time_Adapter::setEndBoundedIntervalAlarm(const IAAtom& atom,
                                    const AString& selector,                //@A2
                                    const IAEventHeader& hdr)               //@B3
     {
       IFUNCTRACE_DEVELOP();
       AlarmInterval * newAlarm=                                          //@B3
                     new AlarmBoundedInterval(atom.getStringTerm(5),
                          atom.getIntegerTerm(1),                         //@A6
                          atom.getStringTerm(2),                          //@A6
                          "0","0",
                          atom.getStringTerm(3),       // end date        //@A6
                          atom.getStringTerm(4),       // end time        //@A6
                          selector);
       newAlarm->setEventNumber(hdr.eventNumber());                       //@B3
       // for now, category is init if set on AGENT:STARTING (pre start )    //@B3
       if (strlen(atom.getStringTerm(6)) > 0)
          newAlarm->setAlarmCategory(atom.getStringTerm(6));
       else if (strcmp(hdr.getDomain(),"AGENT")== 0 &&                       //@B3
                strcmp(hdr.getType(),"STARTING")==0)                         //@B3
                          newAlarm->setAlarmCategory("init");                //@B3
       else newAlarm->setAlarmCategory("other");                             //@B3
       newAlarm->setPersistent(isPersistentCategory(newAlarm->getAlarmCategory())); //@B3
       startAlarm(newAlarm);                                              //@B3
     }

void Time_Adapter::setDailyAlarm(const IAAtom& atom,
                                    const AString& selector,              //@A2
                                    const IAEventHeader& hdr)               //@B3
     {
       IFUNCTRACE_DEVELOP();
       AlarmSpecificDaily * newAlarm=                                          //@B3
                  new AlarmSpecificDaily(atom.getStringTerm(2),
                                          atom.getStringTerm(1),            //@A6
                                         "0123456",selector);
       newAlarm->setEventNumber(hdr.eventNumber());                       //@B3
       // for now, category is init if set on AGENT:STARTING (pre start )    //@B3
       if (strlen(atom.getStringTerm(3)) > 0)
          newAlarm->setAlarmCategory(atom.getStringTerm(3));
       else if (strcmp(hdr.getDomain(),"AGENT")== 0 &&                       //@B3
                strcmp(hdr.getType(),"STARTING")==0)                         //@B3
                     newAlarm->setAlarmCategory("init");                     //@B3
       else newAlarm->setAlarmCategory("other");                             //@B3
       newAlarm->setPersistent(isPersistentCategory(newAlarm->getAlarmCategory())); //@B3
       startAlarm(newAlarm);                                              //@B3
     }

void Time_Adapter::setSpecificDailyAlarm(const IAAtom& atom,
                                    const AString& selector,                //@A2
                                    const IAEventHeader& hdr)               //@B3
     {
       IFUNCTRACE_DEVELOP();
       AlarmSpecificDaily * newAlarm=                                          //@B3
                       new AlarmSpecificDaily(atom.getStringTerm(3),
              atom.getStringTerm(1),atom.getStringTerm(2),                 //@A6
                          selector);
       newAlarm->setEventNumber(hdr.eventNumber());                       //@B3
       // for now, category is init if set on AGENT:STARTING (pre start )    //@B3
       if (strlen(atom.getStringTerm(4)) > 0)
          newAlarm->setAlarmCategory(atom.getStringTerm(4));
       else if (strcmp(hdr.getDomain(),"AGENT")== 0 &&                       //@B3
                strcmp(hdr.getType(),"STARTING")==0)                         //@B3
                          newAlarm->setAlarmCategory("init");                //@B3
       else newAlarm->setAlarmCategory("other");                             //@B3
       newAlarm->setPersistent(isPersistentCategory(newAlarm->getAlarmCategory())); //@B3
       startAlarm(newAlarm);                                              //@B3
     }

void Time_Adapter::setAlarmAt(const IAAtom& atom,
                                    const AString& selector,              //@A2
                                    const IAEventHeader& hdr)               //@B3
     {
       IFUNCTRACE_DEVELOP();
       AlarmOneTime * newAlarm=                                           //@B3
                  new AlarmOneTime(atom.getStringTerm(3),
                                   atom.getStringTerm(1),      // date    //@A6
                                   atom.getStringTerm(2),       // time   //@A6
                                   selector);
       newAlarm->setEventNumber(hdr.eventNumber());                       //@B3
       // for now, category is init if set on AGENT:STARTING (pre start )    //@B3
       if (strlen(atom.getStringTerm(4)) > 0)
          newAlarm->setAlarmCategory(atom.getStringTerm(4));
       else if (strcmp(hdr.getDomain(),"AGENT")== 0 &&                       //@B3
                strcmp(hdr.getType(),"STARTING")==0)                         //@B3
                          newAlarm->setAlarmCategory("init");                     //@B3
       else newAlarm->setAlarmCategory("other");                             //@B3
       newAlarm->setPersistent(isPersistentCategory(newAlarm->getAlarmCategory())); //@B3
       startAlarm(newAlarm);                                              //@B3
     }

void Time_Adapter::setIntervalAlarmRandomStart(const IAAtom& atom, //@A2
                     const AString& selector,                              //@A2
                     const IAEventHeader& hdr)                            //@B3
     {                                                                    //@A2
       IATime startTime;                                                  //@A2
       int randStartInterval=atom.getIntegerTerm(3);                      //@A6
       if (randStartInterval==0)                                          //@A2
          {                                                               //@A2
           randStartInterval=startTime.convertInterval                   //@A2@A4
                             (atom.getIntegerTerm(1),                     //@A6
                              atom.getStringTerm(2));                     //@A6
          }                                                               //@A2
       int start=rand();                                                  //@A2
       if (start<0) start=start;                                          //@A2
       start=1+start%randStartInterval;                                   //@A2
       startTime=startTime.addInterval(start,"seconds");                  //@A2
       AlarmInterval * newAlarm=                                //@B3
                  new AlarmInterval(atom.getStringTerm(4),      //@A6
                       atom.getIntegerTerm(1),                            //@A9
                       atom.getStringTerm(2),                             //@A6
                       startTime.getDate(),   // start date               //@A2
                       startTime.getTime(),   // start time               //@A2
                       selector);
       newAlarm->setEventNumber(hdr.eventNumber());                       //@B3
       // for now, category is init if set on AGENT:STARTING (pre start )    //@B3
       if (strlen(atom.getStringTerm(5)) > 0)
          newAlarm->setAlarmCategory(atom.getStringTerm(5));
       else if (strcmp(hdr.getDomain(),"AGENT")== 0 &&                       //@B3
                strcmp(hdr.getType(),"STARTING")==0)                         //@B3
                          newAlarm->setAlarmCategory("init");                //@B3
       else newAlarm->setAlarmCategory("other");                             //@B3
       newAlarm->setPersistent(isPersistentCategory(newAlarm->getAlarmCategory())); //@B3
       startAlarm(newAlarm);                                              //@B3
     }                                                                    //@A2

void Time_Adapter::setRandomStartBoundedIntervalAlarm(          //@B1
                     const IAAtom& atom,                                  //@A7
                     const AString& selector,                             //@A7
                     const IAEventHeader& hdr)                            //@B3
     {                                                                    //@A7
       IATime startTime(atom.getStringTerm(3),atom.getStringTerm(4));     //@A7
       int randStartInterval=atom.getIntegerTerm(5);                      //@A7
       if (randStartInterval==0)                                          //@A7
          {                                                               //@A7
           randStartInterval=startTime.convertInterval                    //@A7
                             (atom.getIntegerTerm(1),                     //@A7
                              atom.getStringTerm(2));                     //@A7
          }                                                               //@A7
       int start=rand();                                                  //@A7
       if (start<0) start=start;                                          //@A7
       start=1+start%randStartInterval;                                   //@A7
       startTime=startTime.addInterval(start,"seconds");                  //@A7
       AlarmInterval * newAlarm=                                //@B3
                  new AlarmInterval(atom.getStringTerm(6),      //@A7
                       atom.getIntegerTerm(1),                            //@A9
                       atom.getStringTerm(2),                             //@A7
                       startTime.getDate(),   // start date               //@A7
                       startTime.getTime(),   // start time               //@A7
                       selector);
       newAlarm->setEventNumber(hdr.eventNumber());                       //@B3
       // for now, category is init if set on AGENT:STARTING (pre start )    //@B3
       if (strlen(atom.getStringTerm(7)) > 0)
          newAlarm->setAlarmCategory(atom.getStringTerm(7));
       else if (strcmp(hdr.getDomain(),"AGENT")== 0 &&                       //@B3
                strcmp(hdr.getType(),"STARTING")==0)                         //@B3
                          newAlarm->setAlarmCategory("init");                //@B3
       else newAlarm->setAlarmCategory("other");                             //@B3
       newAlarm->setPersistent(isPersistentCategory(newAlarm->getAlarmCategory())); //@B3
       startAlarm(newAlarm);                                              //@B3
     }                                                                    //@A7


void Time_Adapter::setRandomBoundedIntervalAlarm(const IAAtom& atom, //@B1
                                    const AString& selector,              //@B1
                                    const IAEventHeader& hdr)             //@B3
     {                                                                    //@B1
       IATime startTime(atom.getStringTerm(3),atom.getStringTerm(4));     //@B1
       int randStartInterval=atom.getIntegerTerm(7);                      //@B1
       if (randStartInterval==0)                                          //@B1
          {                                                               //@B1
           randStartInterval=startTime.convertInterval                    //@B1
                             (atom.getIntegerTerm(1),                     //@B1
                              atom.getStringTerm(2));                     //@B1
          }                                                               //@B1
       int start=rand();                                                  //@B1
       if (start<0) start=start;                                          //@B1
       start=1+start%randStartInterval;                                   //@B1
       startTime=startTime.addInterval(start,"seconds");                  //@B1
       AlarmBoundedInterval * newAlarm=                                          //@B3
                  new AlarmBoundedInterval(atom.getStringTerm(8), //@B1
                       atom.getIntegerTerm(1),                            //@B1
                       atom.getStringTerm(2),                             //@B1
                       startTime.getDate(),   // start date               //@B1
                       startTime.getTime(),   // start time               //@B1
                       atom.getStringTerm(5),  // end date                //@B1
                       atom.getStringTerm(6),  // end time                //@B1
                       selector);
       newAlarm->setEventNumber(hdr.eventNumber());                       //@B3
       // for now, category is init if set on AGENT:STARTING (pre start )    //@B3
       if (strlen(atom.getStringTerm(9)) > 0)
          newAlarm->setAlarmCategory(atom.getStringTerm(9));
       else if (strcmp(hdr.getDomain(),"AGENT")== 0 &&                       //@B3
                strcmp(hdr.getType(),"STARTING")==0)                         //@B3
                          newAlarm->setAlarmCategory("init");                //@B3
       else newAlarm->setAlarmCategory("other");                             //@B3
       newAlarm->setPersistent(isPersistentCategory(newAlarm->getAlarmCategory())); //@B3
       startAlarm(newAlarm);                                              //@B3
     }                                                                    //@B1

// true if an alarm was removed
// alarmSetMutex is held
// eventNumber=0 means remove all alarms for selector regardless of eventNumber
boolean Time_Adapter::findAndRemoveAlarmsForSelector(                     //@B2
         const AString& selector, boolean& isFirst,                       //@B2
         long  eventNumber, const AString& level)                         //@B3

{
  boolean removed=false;
  AlarmIdSet::Cursor mCursor(mappedAlarms);                            //@A9
  AlarmSet::Cursor aCursor(Alarms);                                    //@A2
  AlarmIdSet::Cursor removeCursor(mappedAlarms);                       //@A9
  IATime alarmTime;                                                     //@A9
  AlarmUniqueId alarmuid;                                                 //@A9
  boolean all;
  if (level.keyWordIn("all",false,false) == -1) all=false;
  else all=true;
  int categoryNum=level.numWords();                                        //@B3
 // set up dummy alarm for selector                                      @A9
  Alarm * dummy=new AlarmOneTime();                                     //@A9
  Alarm * currentAlarm;
  dummy->setSelector(selector);                                         //@A9
  // find dummy's location in set
  mappedAlarms.locateOrAddElementWithKey(dummy,mCursor);                //@A9
  // loop through and remove all the elements for this selector         //@A9
  for (;mappedAlarms.setToNext(mCursor) &&                              //@A9
             mappedAlarms.elementAt(mCursor)->getSelector()==selector;)   //@A9
       {

        // if eventNumber is greater and category matches one in level and
        // we find alarm in Alarms, then remove both (mapped first)
         currentAlarm=mappedAlarms.elementAt(mCursor);
         if ((!eventNumber || eventNumber >                               //@B3
                 currentAlarm->getEventNumber()) &&                       //@B3
            ( all || level.keyWordIn(currentAlarm->getAlarmCategory(),       //@B3
                                     false,false) != -1) &&               //@B3
            Alarms.findElementForId(aCursor,
                        currentAlarm->getAlarmUniqueId(),
                        currentAlarm->getAlarmTime()))    {               //@A9
               if (Alarms.isFirst(aCursor)) isFirst=true;                 //@A9
               Alarms.removeAt(aCursor);                                  //@A9
               delete mappedAlarms.elementAt(mCursor);                    //@A9
               // should always be true because of dummy selector
               mappedAlarms.removeAtAndResetCursor(mCursor);              //@A9
               removed=true;
         }

       }                                                                  //@A2
  return removed;
}

void Time_Adapter::resetSelector(const AString& selector,                //@A2@A5
                                            long eventNumber,             //@B3
                                            const AString& level)         //@B3
  {                                                                       //@A2
  // add code to write this in transaction log
    IFUNCTRACE_DEVELOP();                                                 //@A2
    AlarmSetMutex.lock();                                                 //@A2
    boolean doSignal=false;                                               //@A2
    if (findAndRemoveAlarmsForSelector(selector,doSignal,eventNumber,     //@B2
                                       level)                             //@B3
                                    && persistenceOn)                     //@B2
                   updateTransactions(selector,eventNumber,level);        //@B2
    if (doSignal) AlarmChangeEvent.signal();                              //@A2
    AlarmSetMutex.unlock();                                               //@A2
  }                                                                       //@A2

void Time_Adapter::testAny(const IAAtom& atom, const AString& selector)   //@A5
  {                                                                       //@A5
   AString testType=atom.getSymbolTerm(1);                                //@A6
   AString parms=atom.getStringTerm(2);                                   //@A6
   if (testType=="reset") reset(parms);                                //@A5
   else if (testType=="quiesce") quiesce(parms);                       //@A5
   else if (testType=="restart") restart(parms);                       //@A5
  }                                                                       //@A5

void Time_Adapter::testSetTime(const IAAtom& atom)                        //@A2
     {                                                                    //@A2
       IFUNCTRACE_DEVELOP();                                              //@A2
       double time1,time2;
       AString test=atom.getSymbolTerm(1);                                //@A6
       int loops=atom.getIntegerTerm(2);                                  //@A6
       AString id(atom.getStringTerm(3));                                 //@A6
       Alarm * testAlarm;
       AString user("userid");
       AString selector;
       int i;
       int interval=60;
       if (test=="constructor")
        {
          time1=(double)clock();
          for (i=1;i<=loops;++i)
           {
            selector=user+(AString)i;
            startAlarm(new AlarmInterval(id,interval,                     //@B2
                              "seconds",selector));
            interval=interval+1;
            }
           time2=(double)clock();
         }
       else if (test=="setter")
        {
          time1=(double)clock();
          for (i=1;i<=loops;++i)
           {
            selector=user+(AString)i;
             testAlarm=new AlarmInterval(id,interval,"seconds",selector);            //@A2
             startAlarm(testAlarm);
             interval=interval+1;
            }
           time2=(double)clock();
         }
       else
         {
          cout << "Bad choice for symbol: " << test << endl;
          return;
         }
   //    listAlarms(atom);
       cout << "Clocks for " << i << " iterations =" << time2-time1 << endl;
     }                                                                     //@A2

// processTimer - routine run on thread to process timer occurrences

long Time_Adapter::processTimer(void * )
   {
      AlarmSetMutex.lock();
      Alarm * currentAlarm,*eventAlarm;
      AlarmSet::Cursor aCursor(Alarms);
      boolean recurringAlarm;
      // Used for eliminating duplicate notifies
      IATime currentTime;
      time_t nextTime;
      IAFactSet tempFactSet;
      int i;                                                              //@A5
      boolean alarmsActive;

// Main loop - do this as long as we have alarms and are enabled for notify
      while (isEnabled() && !Alarms.isEmpty())
       {
        alarmsActive=true;
// set up wait time - every thirty days if none active
        if (!Alarms.firstActiveAlarm(aCursor)) {
            nextTime=currentTime.addInterval(30,"days").gettime_t();
            alarmsActive=false;
        }
        else
            nextTime=Alarms.elementAt(aCursor)->getAlarmTime().gettime_t();

// timed condition variable wait for next alarm
// Is the adapter started?
// Did we change the first element or is the first element expired?
// need to check for isEmpty - in case last one removed before signal
      while ( isEnabled() &&  !Alarms.isEmpty() &&
             (!Alarms.firstActiveAlarm(aCursor) ||
              (!Alarms.elementAt(aCursor)->isExpired() &&
               nextTime == Alarms.elementAt(aCursor)->getAlarmTime().gettime_t())))
              {
                AlarmChangeEvent.timed_wait(nextTime);
                if (!alarmsActive && !Alarms.firstActiveAlarm(aCursor)) {
                      nextTime=((IATime)nextTime).addInterval(30,"days").gettime_t();
                }
              }  // end of timed condition variable wait
// either signal or alarm occurred at this point

// process all expired alarms - if signal, probably none will be expired
       while ( isEnabled() && !Alarms.isEmpty() &&
                Alarms.firstActiveAlarm(aCursor) &&
                Alarms.elementAt(aCursor)->isExpired())
          {
          // remove first element
             currentAlarm=Alarms.elementAt(aCursor);                      //@A9
             Alarms.removeAt(aCursor);                                    //@A9
             mappedAlarms.removeElementWithKey(                           //@A9
                             currentAlarm->getAlarmUniqueId());           //@A9

          // This next batch of code is to make sure we don't generate
          // events with identical event headers and fact sets from 2 different
          // alarms - the overhead to check for this is small probably
          // compared to a whole new inferencing event
          // Only do notify if duplicate events not possible or there is no
          // duplicate event waiting in the alarm set
             if (!currentAlarm->duplicateEventsPossible() ||
                 !eventMatch(currentAlarm))
               {

          // Process alarm event
          // release alarmset mutex before calling notify()
                AlarmSetMutex.unlock();
          // need to build fact set
                IATriggerEvent * Event=new IATriggerEvent();               //@A3
                try                                                        //@A6
                 {                                                         //@A6
                  Event->setType(currentAlarm->getEventType());           //@A6
                  Event->setFactSet(currentAlarm->getFactSet());             //@A3
                  Event->getHeader().setSelector(currentAlarm->getSelector()); //@A6
                  TimeAdapterEventInfo * newEventInfo=                      //@B2
                                new TimeAdapterEventInfo;                   //@B2
                  eventAlarm=currentAlarm->createCopy();                    //@B2
                  newEventInfo->notifiedAlarm=eventAlarm;                   //@B2
                  newEventInfo->eventId=Event->getHeader().getId();         //@B2                    //@B2
                  eventSetMutex.lock();                                     //@B2
                  eventSet.add(newEventInfo);                               //@B2
                  eventSetMutex.unlock();                                   //@B2
                  notify(*Event);                                          //@A6
                 }
                catch (IAError &e)                                        //@A6
                   {
                   // probably notify error - check for expected ones    //@A6
                   if (eventSetMutex.trylock()) eventSetMutex.unlock();  //@B2
                   if ( enabledForNotify)                                //@A5
                     {                                                   //@A5
                      for (i=0; i < disabledSelectors.size() &&          //@A5
                         (strcmp((char*)currentAlarm->getSelector(),     //@A5
                         (char*)disabledSelectors.elementAt(i))!=0);     //@A5
                         i++) {}                                          //@A5
                      // is there a match?                                     @A5
                      if (i == disabledSelectors.size()) // no match         //@A5
                          cout << e.errormsg() << endl;                      //@A5
                      // there is a match
                      else currentAlarm->setDeferredOn();                 //@B3
                      }
                   // if not enabled for Notify then notify error was expected @A6
                  }                                                       //@A5
                delete Event;
                // acquire mutex for event sequence and add event if not
                // recurring
                AlarmSetMutex.lock();
          // reacquire mutex and add recurring alarm if needed
               }
             if (currentAlarm->isDeferred())
                // put back in but don't increment
                {
                  Alarms.add(currentAlarm);
                  mappedAlarms.add(currentAlarm);                         //@A9
                }                                                         //@A9
             else if (currentAlarm->isRecurring())
                {
                  currentAlarm->incrementAlarm();
                  Alarms.add(currentAlarm);
                  mappedAlarms.add(currentAlarm);                         //@A9
                }                                                         //@A9

             // we are now done with alarm                                  @A9
             else {

                // now get AlarmSetMutex                                    @A9
                delete currentAlarm;                                      //@A9
                }
          } // processing all expired alarms
       }  // main loop - while isEnabled() && !Alarms.isEmpty()

  // release mutex and return
  AlarmSetMutex.unlock();
  return 0;
   }

// AlarmSetMutex is held when this is called and still held on return
boolean Time_Adapter::eventMatch(const Alarm * a) const
   {
    IFUNCTRACE_DEVELOP();
     AlarmSet::Cursor aCursor(Alarms);
     IATime thisAlarmTime=a->getAlarmTime();
     // Is there another alarm with the same time? - if not, no match
     if (!Alarms.locateElementWithKey(thisAlarmTime,aCursor)) return false;

     // Look for all alarms with same alarm time and see if event matches
     if (Alarms.elementAt(aCursor)->getSelector()==a->getSelector() &&      //@A2
          a->eventMatches(Alarms.elementAt(aCursor))) return true;
     while (Alarms.locateNextElementWithKey(thisAlarmTime,aCursor))
        {
         if (Alarms.elementAt(aCursor)->getSelector()==a->getSelector() && //@A2
              a->eventMatches(Alarms.elementAt(aCursor))) return true;
        }
     return false;
   }

boolean Time_Adapter::checkTermList(const IAAtom& atom,int numOfElem,
                                    AString signature) const
    {
    IFUNCTRACE_DEVELOP();
     if (atom.getNumberOfTerms()!=numOfElem)
      {
        AString badNumber;
        AString actualNumber((int)atom.getNumberOfTerms());
        AString expectedNumber(numOfElem);
        badNumber=badNumber+"Expected number of terms= "+
                  expectedNumber+" Actual number of terms= "+actualNumber;
        ITHROW(InvalidTermListLength((char*)badNumber));
      }
     AString termType;
     for ( int i=1;i<=numOfElem;i++)
       {
         if (i<=signature.numWords()) termType=signature.word(i);
         if (termType !="Lvar" &&  atom.getTermType(i)!=termType)
            {
             AString badTerm("term #");
             AString number(i);
             badTerm=badTerm+number+" Expected type= "+
                  termType+" Actual type= "+atom.getTermType(i);
             ITHROW(InvalidTermInTermList((char*)badTerm));
            }
       }
     return true;
    }

boolean Time_Adapter::startAlarm(Alarm* a)
   {
    IFUNCTRACE_DEVELOP();
    AlarmSetMutex.lock();

    // we need to start a thread if there are no alarms now and we are enabled
    boolean startThread=Alarms.isEmpty() && isEnabled();

    boolean addAlarm=true;
    AlarmSet::Cursor aCursor(Alarms);
    AlarmIdSet::Cursor mCursor(mappedAlarms);

    // is this alarm in set already?
    AlarmUniqueId newAuid=a->getAlarmUniqueId();                          //@A9
    if (mappedAlarms.locateElementWithKey(newAuid,mCursor))                                //@A9
       {

         // if duplicates allowed increment count of alarm already in set
         if ((mappedAlarms.elementAt(mCursor))->duplicatesAllowed())
            {
             mappedAlarms.elementAt(mCursor)->incrementDuplicateCount();
             // set new eventNumber
             mappedAlarms.elementAt(mCursor)->setEventNumber(a->getEventNumber());
             if (persistenceOn) updateTransactions(a);
            }
         else if (duplicateHandling!=IA_TIME_IGNORE_ALL_DUPLICATES)  //@B2
            {
            // throw exception - could convert fact set to string at this point or add alarmDate function
             AlarmSetMutex.unlock();
             delete a;
             ITHROW(DuplicateAlarmError("Duplicate alarm not allowed for this type of Alarm"));
            }
         // a is not added; delete it
         delete a;

       }
     // If its not expired or is recurring, add it
     else if  (!a->isExpired() || a->isRecurring() )
        {
         // add the alarm
         if (a->isExpired() && a->isRecurring()) a->incrementAlarm();     //@A5
         Alarms.add(a,aCursor);
         mappedAlarms.add(a);
         if (persistenceOn & a->isPersistent()) updateTransactions(a);     //@B2
         // start the timer Processing thread if necessary
         if (startThread)
            {
             timerThread.start(AMemberFn<Time_Adapter>(*this,processTimer),NULL);
            }
         // signal if the new alarm is first
         else if (Alarms.isFirst(aCursor))
            {
             //  signal condition variable
             AlarmChangeEvent.signal();
            }
        }
     // we didn't duplicate or add it
     else
       {
        delete a;
         AlarmSetMutex.unlock();
        ITHROW(StartAlarmError("Alarm already expired"));
       }
    AlarmSetMutex.unlock();
    return addAlarm;
   }


// true if any were removed
// AlarmSetMutex held
boolean Time_Adapter::findAndRemoveAlarm(const AlarmUniqueId& alarmuid,
       boolean& isFirst, boolean isPersistent) throw(IAError)            //@B2
{
    boolean removed=false;
    AlarmSet::Cursor aCursor(Alarms);                                    //@B2
    AlarmIdSet::Cursor mCursor(mappedAlarms);                           //@A9@B2
    IATime alarmKey;                                                     //@A9
// Now, go through new mapping and if duplicate is 0 for alarm, remove it  @A9
// from both sets                                                        //@A9
// Always update transaction log                                           @A9
    if (mappedAlarms.locateElementWithKey(alarmuid,mCursor))             //@A9                    //@A9
       {
         // If there wasn't a duplicate to decrement, remove the alarm

         if (!((mappedAlarms.elementAt(mCursor))->decrementDuplicateCount())) //@A9

         {
            alarmKey=mappedAlarms.elementAt(mCursor)->getAlarmTime();    //@A9
            Alarms.locateElementWithKey(alarmKey,aCursor);              //@A9
            for (;!(Alarms.elementAt(aCursor)->matches(alarmuid))         //@A9
                && !Alarms.isLast(aCursor) &&                             //@A9
               Alarms.elementAt(aCursor)->getAlarmTime()==alarmKey;       //@A9
               Alarms.setToNext(aCursor))                                 //@A9
            {}
            if (!(Alarms.elementAt(aCursor)->matches(alarmuid)))          //@A9
             {                                                             //@A9
             cout << "Error removing alarms for selector- no match in alarms " //@A9
                  << "from mapped alarms" << endl <<  " alarmuid= " <<     //@A9
                  alarmuid.getId() << " alarm uid= " <<                    //@A9
                  Alarms.elementAt(aCursor)->getAlarmId() << endl;         //@A9
             return false;                                                 //@A9
             }                                                             //@A9
            isPersistent=mappedAlarms.elementAt(mCursor)->isPersistent(); //@B3
            mappedAlarms.removeAt(mCursor);                               //@A9
            Alarms.RemoveAt(aCursor);
            removed=true;
         } // if !decrement
      // if count decremented we need to record this
      else removed=true;
      }  // if locate element with key
  return removed;
}

boolean Time_Adapter::turnOffAlarm(const AlarmUniqueId& alarmuid)         //@A9

   {

    IFUNCTRACE_DEVELOP();
    AlarmSetMutex.lock();
    boolean isFirst=false;                                                //@B2
    boolean isPersistent=false;
    if (findAndRemoveAlarm(alarmuid,isFirst,isPersistent)                 //@B2
            && persistenceOn && isPersistent)                             //@B3
        updateTransactions(alarmuid);                                     //@B2
    // this isFirst takes care of the case where it is also the last
    // because the signalled thread will check for an empty alarmset
    // cursor no longer valid when element removed - check for first first
    if (isFirst)  // first element
        AlarmChangeEvent.signal();
    AlarmSetMutex.unlock();
    return true;
   }

// Insulate against changes to signature
AString Time_Adapter::buildSignature(const AString& sigString)
      {return sigString;}

// Build event type used in trigger event                                 //@A1
AString Time_Adapter::buildEventType(const Alarm* a)                      //@A1
    {
       IFUNCTRACE_DEVELOP();
       return a->getEventType();                                          //@B5
     }


//#endif
