#ifndef _TIME_ADAPTER_H
#define _TIME_ADAPTER_H

/*************************************************************************
** (C) Copyright IBM Corp, 1996
**
** File: Time_adapter.h
**
** DISCLAIMER OF WARRANTIES:
** -------------------------
** The following [enclosed] code is sample code created by IBM
** Corporation.  This sample code is not part of any standard IBM product
** and is provided to you solely for the purpose of assisting you in the
** development of your applications.  The code is provided "AS IS",
** without warranty of any kind.  IBM shall not be liable for any damages
** arising out of your use of the sample code, even if they have been
** advised of the possibility of such damages.
*************************************************************************/
//******************************************************************************
//
// FILE:    src/adapters/timer/timeadap.h, tka_timer, abe.l6
// VERSION: 1.21.1.4 97/06/23 13:07:31
//
// INCLUDE AS:  #include <timeadap.h>
//
// PURPOSE:  Definition of Time_Adapter
//
// CLASSES DECLARED:
//    Time_Adapter - timer adapter
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $B1 = 960725 WES d219   Fix wrong "throws" in protected sensor and effector
//                         routines
// $E1 = 960909 JMG f308   React to changes to Adapter API
// $A1 = 960914 WES f310   Allow processing of selector, add Identified one time
// $A2 = 960928 JMG d352   Change IAAPI to IACALLBK, eliminate unncess IAAPIs
// $A3 = 961118 WES f441   Months and years support
// $A4 = 961118 WES f480   quiesce, reset, and restart
// $A5 = 961206 WES f512   React to IAAtom, IAFactSet throws
// $A6 = 961211 WES f534   Register listAlarmTimes
// $A7 = 970224 WES f661   New effector for random start from given start time
// $A8 = 970227 WES f668   Support end bounded identified interval alarms
// $A9 = 970225 WES f658   Additional mapping for alarms
// $B1 = 970307 WES f674   set bounded identified interval alarm random start
// $B2 = 970414 WES f664   Persistent alarms
// $B3 = 970509 WES        reset effector support
//
//******************************************************************************

// Openclass Includes
#include <iapl/common.h>
#include <stdlib.h>
#include <iapl/serializ.h>
#include <iapl/sequence.h>
#include <iapl/athreads.h>
#include <alarmbin.h>
#include <alarmot.h>
#include <alarmsda.h>
#include <alarmset.h>
#include <iatk/adapter.h>
#include <persfile.h>
#include <eventset.h>

// file extensions
#define IA_TIME_RECONCILE_FILE_EXTENSION            "rec"                 //@B2
#define IA_TIME_EVENTSTAMP1_FILE_EXTENSION          "ev1"                 //@B2
#define IA_TIME_EVENTSTAMP2_FILE_EXTENSION          "ev2"                 //@B2
#define IA_TIME_EVENTSTAMPNAME_FILE_EXTENSION       "evn"                 //@B2
#define IA_TIME_SAVEDALARMS1_FILE_EXTENSION         "al1"                 //@B2
#define IA_TIME_SAVEDALARMS2_FILE_EXTENSION         "al2"                 //@B2
#define IA_TIME_SAVEDALARMSNAME_FILE_EXTENSION      "aln"                 //@B2

// duplicate handling values
#define IA_TIME_IGNORE_ALL_DUPLICATES                   1                 //@B2
#define IA_TIME_DONT_IGNORE_DUPLICATES                  2                 //@B2
#define IA_TIME_IGNORE_PREPERSISTENT_DUPLICATES         3                 //@B2

// effector tokens

// for setting and turning off alarms
#define IA_TURNOFFALARM                                 1
#define IA_SETINTERVALALARM                             2
#define IA_SETBOUNDEDINTERVALALARM                      3
#define IA_SETSTARTBOUNDEDINTERVALALARM                 4
#define IA_SETDAILYALARM                                5
#define IA_SETSPECIFICDAILYALARM                        6
#define IA_SETALARMAT                                   7
#define IA_SETENDBOUNDEDINTERVALALARM                   8

// random start from now within given interval
#define IA_SETINTERVALALARMRANDOMSTART                  9                  //@A1
// set random start from given start time within given interval
#define IA_SETRANDOMSTARTBOUNDEDINTERVALALARM          10                 //@B1
// same as above but also has end time
#define IA_SETRANDOMBOUNDEDINTERVALALARM               11                 //@B1

// for testing
#define IA_TESTSETTIME                                 40                 //@A1
#define IA_TESTANY                                     41                 //@A4
#define IA_LISTALARMS                                   49
#define IA_LISTALARMTIMES                               48                //@A6

// for administration
#define IA_RESET                                       30                 //@B3
#define IA_SETOVERDUEALARMPRESENTATIONCRITERIA         31                 //@B2
#define IA_SETDUPLICATEHANDLING                        32                 //@B2
#define IA_TIMERCHECKPOINT                             33                 //@B2
#define IA_SETPERSISTENTCATEGORIES                     34                 //@B2
#define IA_SETPERSISTENTFILENAME                       35                 //@B3

// sensor tokens
#define IA_TIMEAFTER                                    1
#define IA_CURRENTIMEAFTER                              2
#define IA_TIMEBEFORE                                   3
#define IA_CURRENTIMEBEFORE                             4
#define IA_TIMEBETWEEN                                  5
#define IA_CURRENTIMEBETWEEN                            6
#define IA_TIMEEQUAL                                    7
#define IA_CURRENTIMEEQUAL                              8
#define IA_TIMENOTEQUAL                                 9
#define IA_CURRENTIMENOTEQUAL                          10
#define IA_ISCURRENTDAY                                11
#define IA_ISCURRENTWEEKDAY                            12
#define IA_ISWEEKEND                                   13
#define IA_ISCURRENTWEEKEND                            14
#define IA_DATEANDTIME                                 15
#define IA_DATE                                        16
#define IA_TIME                                        17
#define IA_DAYOFWEEK                                   18
#define IA_ADDTIMEINTERVAL                             19
#define IA_DETERMINETIMEINTERVAL                       20

class IAError;

struct Time_Adapter_overdue_criteria {
  int amount;
  AString units;
};

struct Time_Adapter_Missed_Event_Completes {
  time_t alarmTime;
  AlarmUniqueId alarmUid;
};

class Time_Adapter:  public IAAdapter {
   public:

//temp

      Time_Adapter(const char *);

      ~Time_Adapter() {shutdown();}       // cleanup                      //@A2

      void IACALLBK eventComplete(const IAEventHeader &hdr);             //@B2

      IABool IACALLBK identify();                                         //@A2

      // Allows notify to take place and starts up thread
      IABool IACALLBK start();                                            //@A2

      IABool IACALLBK stop();                                             //@A2

      void IACALLBK shutdown();            // for now shutdown just cleans  //@A2
                                         // up alarms                     //@A1

      IABool IACALLBK reset(const char * selector);  //  remove all alarms  //@A4

      IABool IACALLBK restart(const char * selector);                       //@A4

      IABool IACALLBK quiesce(const char * selector);                       //@A4

      IABool isEnabled();

      void enable();

      void disable();

      void IACALLBK performAction(IAProcToken token,const IAEventHeader &hdr,//@A2
                               const char *bindingStr);


      IABool IACALLBK testCondition(IAProcToken token,                    //@A2
                                 const IAEventHeader &hdr,
                                 const char *bindingStr)  { return 0; }

      IAFactSet IACALLBK answerQuery(IAProcToken token,                   //@A2
                                  const IAEventHeader &hdr,
                                  const char *bindingStr);

      void listAlarms(const IAAtom& atom);

      void listAlarms();                                                  //@B2

      void listMappedAlarms();                                            //@B2

#ifdef IA_TIMERTEST
      // for testing
      void notify(IATriggerEvent &event) throw(IAError);
#endif

protected:
// timer processing
long processTimer(void *);

boolean saveAlarms() throw(PersistenceError);                             //@B2

// includes saving in alternate alarmset
boolean restoreAlarms() throw(PersistenceError);

boolean updateAlarmsToPresent();                                          //@B2

boolean reconcile() throw(PersistenceError);                              //@B2

boolean updateTransactions(Alarm * alarm)                                 //@B2
              throw(PersistenceError);                                    //@B2

boolean updateTransactions(const AlarmUniqueId& alarmuid)                 //@B2
              throw(PersistenceError);                                    //@B2

boolean updateTransactions(const AString& selector,long eventNumber,     //@B2@B3
                           const AString& level);                         //@B3

boolean updateTimeStampInfo() throw(PersistenceError);                    //@B2

boolean updatePersistentAlarm(Alarm * alarm);                             //@B2

boolean updateDeferredAlarm(Alarm * alarm);                               //@B3

// Perform Actions - Set Alarms

void setIntervalAlarm(const IAAtom&,const AString&,                       //@B2
                      const IAEventHeader& hdr)                           //@B3
     throw(InvalidTermListLength,InvalidTermInTermList,
           IntervalUnitsError,IntervalValueError,IAError,                 //@A5
           PersistenceError,AlarmCategoryError,DuplicateAlarmError);         //@B2

void setBoundedIntervalAlarm(const IAAtom&,const AString&,                //@B2
                      const IAEventHeader& hdr)                           //@B3
     throw(InvalidTermListLength,InvalidTermInTermList,
           DateFormatError,TimeFormatError,
           IntervalUnitsError,IntervalValueError,
           StartAlarmError, IAError,DuplicateAlarmError,                  //@A5
           PersistenceError,AlarmCategoryError);                             //@B2

void setEndBoundedIntervalAlarm(const IAAtom&, const AString&,            //@B2
                      const IAEventHeader& hdr)                           //@B3
     throw(InvalidTermListLength,InvalidTermInTermList,
           DateFormatError,TimeFormatError,
           IntervalUnitsError,IntervalValueError,
           StartAlarmError,IAError,DuplicateAlarmError,                   //@A5
           PersistenceError,AlarmCategoryError);                             //@B2

void setStartBoundedIntervalAlarm(const IAAtom&, const AString&,          //@B2
                      const IAEventHeader& hdr)                           //@B3
     throw(InvalidTermListLength,InvalidTermInTermList,
           DateFormatError,TimeFormatError,DuplicateAlarmError,
           IntervalUnitsError,IntervalValueError,IAError,                 //@A5
           PersistenceError,AlarmCategoryError);                             //@B2
                                                                          //@B1
void setDailyAlarm(const IAAtom&, const AString&,                         //@B2
                      const IAEventHeader& hdr)                           //@B3
     throw(InvalidTermListLength,InvalidTermInTermList,TimeFormatError,
           IAError,PersistenceError,AlarmCategoryError,DuplicateAlarmError);  //@B2


void setSpecificDailyAlarm(const IAAtom&,const AString&,                  //@A1
                      const IAEventHeader& hdr)                           //@B3
     throw(InvalidTermListLength,InvalidTermInTermList,
           TimeFormatError,WeekdayValueError,IAError,                     //@A5
           PersistenceError,AlarmCategoryError,DuplicateAlarmError);         //@B2

void setAlarmAt(const IAAtom&, const AString&,                            //@A1
                      const IAEventHeader& hdr)                           //@B3
     throw (InvalidTermListLength,InvalidTermInTermList,
           DateFormatError,TimeFormatError,
           StartAlarmError,IAError,DuplicateAlarmError,                   //@A5
           PersistenceError,AlarmCategoryError);                             //@B2


void setIntervalAlarmRandomStart(const IAAtom&, const AString&,           //@A1
                      const IAEventHeader& hdr)                           //@B3
     throw(InvalidTermListLength,InvalidTermInTermList,                   //@B1
           IntervalUnitsError,IntervalValueError,                         //@B1
           DateFormatError,TimeFormatError,                               //@B1
           DuplicateAlarmError,IAError,                                   //@B1
           PersistenceError,AlarmCategoryError);                             //@B2

void setRandomStartBoundedIntervalAlarm(const IAAtom&,                    //@B1
                                           const AString&,                //@B1
                                           const IAEventHeader& hdr)      //@B3
     throw(InvalidTermListLength,InvalidTermInTermList,                   //@B1
           IntervalUnitsError,IntervalValueError,                         //@B1
           DateFormatError,TimeFormatError,                               //@B1
           DuplicateAlarmError,IAError,                                   //@B1
           PersistenceError,AlarmCategoryError);                             //@B2

void setRandomBoundedIntervalAlarm(const IAAtom&,                         //@B1
                      const AString&,                                     //@B1
                      const IAEventHeader& hdr)                           //@B3
     throw(InvalidTermListLength,InvalidTermInTermList,                   //@B1
           IntervalUnitsError,IntervalValueError,                         //@B1
           DateFormatError,TimeFormatError,                               //@B1
           DuplicateAlarmError,IAError,                                   //@B1
           PersistenceError,AlarmCategoryError);                             //@B2

// I think selector has no meaning - but just in case
void setOverdueAlarmPresentationCriteria(const IAAtom&, const AString&)   //@B2
     throw(InvalidTermListLength,InvalidTermInTermList,IAError,           //@B2
           PersistenceEffectorError,PersistenceError);                    //@B2

void setPersistentFileName(const IAAtom&) throw(IAError);                 //@B3

void setPersistentCategories(const AString& categoryList);                       //@B3

boolean isPersistentCategory(const AString& category) const;                       //@B3

// I think selector has no meaning - but just in case
void setDuplicateHandling(const IAAtom&, const AString&)                  //@B2
     throw(InvalidTermListLength,InvalidTermInTermList,IAError,           //@B2
           PersistenceEffectorError,PersistenceError);                    //@B2

void checkPoint(const IAAtom&, const AString&)                            //@B2
           throw(PersistenceEffectorError,PersistenceError);              //@B2

void resetSelector(const AString&,long eventNumber,                      //@B3
                              const AString& level) throw(IAError);      //@B3

// true if any were removed
boolean findAndRemoveAlarmsForSelector(const AString&, boolean&,          //@B2
                                       long eventNumber,const AString&)   //@B3
                             throw(IAError);                            //@B2@B3

void testSetTime(const IAAtom&) throw(PersistenceError,IAError);          //@B2

void testAny(const IAAtom&,const AString&) throw(IAError);                //@A5

void listAlarmTimes(const AString&) throw(IAError);                       //@A5


// Sensors

IAFactSet timeAfter(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

IAFactSet currentTimeAfter(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

IAFactSet timeBefore(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

IAFactSet currentTimeBefore(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

IAFactSet timeBetween(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

IAFactSet currentTimeBetween(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

IAFactSet timeEqual(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

IAFactSet currentTimeEqual(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

IAFactSet timeNotEqual(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

IAFactSet currentTimeNotEqual(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

IAFactSet isCurrentDay(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,IAError);         //@A5
                                                                          //@B1

IAFactSet isWeekday(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,IAError);         //@A5
                                                                          //@B1

IAFactSet isCurrentWeekday(const IAAtom&)
     throw (InvalidTermListLength,IAError);                               //@A5
                                                                          //@B1

IAFactSet isWeekend(const IAAtom&)
     throw (InvalidTermListLength,InvalidTermInTermList,IAError);         //@A5
                                                                          //@B1

IAFactSet isCurrentWeekend(const IAAtom&)
     throw (InvalidTermListLength,IAError);                               //@A5
                                                                          //@B1

// Non-Boolean Sensors

IAFactSet dateAndTime(const IAAtom& atom)
     throw (InvalidTermListLength,InvalidTermInTermList,IAError);         //@A5

IAFactSet currentDate(const IAAtom& atom)
     throw (InvalidTermListLength,InvalidTermInTermList,IAError);         //@A5

IAFactSet currentTime(const IAAtom& atom)
     throw (InvalidTermListLength,InvalidTermInTermList,IAError);         //@A5

IAFactSet dayOfWeek(const IAAtom& atom )
     throw (InvalidTermListLength,InvalidTermInTermList,IAError);         //@A5

IAFactSet addTimeInterval(const IAAtom& atom)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IntervalUnitsError,IAError);  //@A5

IAFactSet determineTimeInterval(const IAAtom& atom)
     throw (InvalidTermListLength,InvalidTermInTermList,
            DateFormatError,TimeFormatError,IAError);                     //@A5

// Internal

AString buildEventType(const Alarm*);                                     //@A1
boolean startAlarm(Alarm*)
       throw (DuplicateAlarmError,StartAlarmError,PersistenceError);
boolean startPersistentAlarm(Alarm*, boolean prePersistent)
       throw (DuplicateAlarmError,StartAlarmError);
boolean turnOffAlarm(const AlarmUniqueId& alarmuid)
        throw(PersistenceError);                                          //@B2

// true if any were removed
boolean findAndRemoveAlarm(const AlarmUniqueId&, boolean& isFirst,
                           boolean isPersistent)  throw(IAError);         //@B2

boolean checkTermList(const IAAtom&,int,AString signature) const
        throw (InvalidTermListLength,InvalidTermInTermList,IAError);
// Insulate against changes to signature
static AString buildSignature(const AString&);
void updateAlarmsForSelector(const char *);                               //@A4

// AlarmSetMutex is held when this is called and still held on return
boolean eventMatch(const Alarm*) const;

static int findUnitsIndex(const AString& units);                          //@B2


private:
   AMutex startMutex;
   volatile boolean enabledForNotify;
   boolean configDone;                                                    //@B3
   boolean persistenceOn;                                                 //@B2
   int duplicateHandling;                                                 //@B2
   boolean persistenceError;                                              //@B2
   AMutex configMutex;                                                    //@B3


   // the following are all associated with eventSetMutex                 //@B2
   AMutex eventSetMutex;                                                  //@B2
   TimeEventSet eventSet;   // set of events notified but not             //@B2
                            // having received event complete
   // the following files are alternating files with a timestamp of the
   // alarm time of the last event complete and any
   // events that have not completed that have earlier alarm times.
   // Along with the alternating files is a file with the name of the current
   // file
   PersistentFile * currentEventStamp;                                    //@B2
   PersistentFile * alternateEventStamp;                                  //@B2
   PersistentFile currentEventStampFileName;                              //@B2
   // The information contained in the event stamp files from the previous
   // invocation of the agent - this is read in once into the time adapter
   // when the persistent alarms are restored at start()
   time_t lastPersistentTimeStamp;                                        //@B2
   AVPSeq lastPersistentTimeExceptions;                                   //@B2


   // following are all associated with AlarmSetMutex                     //@B2
   AMutex AlarmSetMutex;
   ACondVar AlarmChangeEvent;
   // Two mappings of the set of all the alarms
   AlarmSet Alarms;
   AlarmIdSet mappedAlarms;                                               //@A9
   // Alternating files representing all the alarms and the
   // file with the name of the current alarm file
   PersistentFile * currentSavedAlarms;                                   //@B2
   PersistentFile * alternateSavedAlarms;                                 //@B2
   PersistentFile currentSavedAlarmsFileName;                             //@B2
   // File of transactions on the alarm set since the last save of the
   // set of alarms.
   // Transactions are only adding and removing alarms
   // Information about the last presentation of recurring alarms is
   // contained in the event stamp files.
   PersistentFile transactionFile;                                        //@B2

   AThread timerThread;
   AString inputParm;   // could be used in future

   boolean noOverdueCriteria;                                            //@B2
   // this array only changed before start, only read during start
   // so mutex needed
   Time_Adapter_overdue_criteria * overdueCriteria[7];                    //@B2

   AVPSeq disabledSelectors;                                              //@A4

  // optimized for small number of categories
   AVPSeq persistentCategories;                                               //@B3
   boolean persistentCategoriesSet;  // only allow this effector once         //@B3

   static char sensorIdentifiers[30][80];                                 //@B2
   static char effectorIdentifiers[50][80];                               //@B2
};


#endif //_TIME_ADAPTER
