#ifndef TIME_EXCEPTION_H
#define TIME_EXCEPTION_H

//******************************************************************************
//
// FILE:    src/adapters/timer/timeexcp.h, tka_timer, toolkit.b1
// VERSION: 1.7 96/12/19 15:52:27
//
// INCLUDE AS:  #include <timeexcp.h>
//
// PURPOSE:  timer adapter exception classes
//
// CLASSES DECLARED:
//    TimeFormatError - error in format of string representing a time
//    DateFormatError - error in format of string representing a date
//    IntervalUnitsError - string representing interval units not valid
//    IntervalValueError - error when interval value represented as a string
//    WeekdayValueError - invalid weekday ( not 0 - 6)
//    InvalidTermListLength - wrong number of terms in sensor or effector
//    DuplicateAlarmError - set alarm effector issued for identified alarm with
//                          alarmid matching an existing alarm
//    InvalidTermInTermList - wrong type of term in sensor of effector
//    StartAlarmError - error in starting alarm
//    PersistenceError - error in saving alarms or transactions
//    PersistenceEffectorError - error in effectors related to persistence
//
//
//******************************************************************************

//** Version History ***********************************************************
//
// FLG   DATE   WHO REASON DESCRIPTION
// ===   ====== === ====== ===========
// $A1   960909 WES  f304   Add support for route-id field in alarms
//
//******************************************************************************


// Exception classes that we derive from the built-in classes
#include <iexcept.hpp>
class TimeFormatError : public IException {
public:
  TimeFormatError(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "TimeFormatError";}
};

class DateFormatError : public IException {
public:
  DateFormatError(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "DateFormatError";}
};

class IntervalUnitsError : public IException {
public:
  IntervalUnitsError(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "IntervalUnitsError";}
};

class IntervalValueError : public IException {
public:
  IntervalValueError(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "IntervalValueError";}
};

class WeekdayValueError : public IException {
public:
  WeekdayValueError(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "WeekdayValueError";}
};

class InvalidTermListLength : public IException {
public:
  InvalidTermListLength(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "InvalidTermListLength";}
};

class DuplicateAlarmError : public IException {
public:
  DuplicateAlarmError(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "DuplicateAlarmError";}
};

class InvalidTermInTermList : public IException {
public:
  InvalidTermInTermList(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "InvalidTermInTermList";}
};

class PersistenceError : public IException {
public:
  PersistenceError(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "PersistenceError";}
};

class PersistenceEffectorError : public IException {
public:
  PersistenceEffectorError(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "PersistenceEffectorError";}
};

class BadScopeId : public IException {                                    //@A1
public:                                                                   //@A1
  BadScopeId(const char *text, unsigned long errorId = 0,                 //@A1
        Severity severity = IException::unrecoverable):                   //@A1
        IException(text,errorId,severity) {}                              //@A1
  virtual const char* name() const {return "BadScopeId";}                 //@A1
};

class StartAlarmError : public IException {
public:
  StartAlarmError(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "StartAlarmError";}
};

class AlarmCategoryError : public IException {
public:
  AlarmCategoryError(const char *text, unsigned long errorId = 0,
        Severity severity = IException::unrecoverable):
        IException(text,errorId,severity) {}
  virtual const char* name() const {return "AlarmCategoryError";}
};


#endif
