/* KMapNet class

 Constructing Intelligent Agents with Java
   (C) Joseph P. Bigus and Jennifer Bigus 1997
    

 */

import java.awt.*;
import java.util.*;
import java.lang.Math;
import java.io.* ;

public class KMapNet extends Object {
    String name ;

  // data parameters
    static DataSet ds;
    Vector data ;
    int recInx=0 ;  // current record index
    int numRecs=0 ; // number of records in data
    int fieldsPerRec;
    int numPasses=0 ;

  // network architecture parameters
    int numInputs ;
    int numRows ;
    int numCols ;
    int numOutputs ;
    int numUnits ;

  // network control parameters
    int mode;
    float learnRate ;
    float initLearnRate = (float)1.0 ;
    float finalLearnRate = (float)0.05 ;
    float sigma ;
    int maxNumPasses=20 ;  // default

  // network data
    int winner ;         // index of the winning unit
    float activations[];
    float weights[];
    int   distance[];   // used in neighborhood computation

    static TextArea textArea1 ;

  KMapNet(String Name) {
       name = Name ;
       data = new Vector() ;
  }
 
    public void show_array(String name, float[] arr) {
        textArea1.append("\n" + name  + "= ") ;
     for (int i=0 ; i < arr.length ; i++) {
        textArea1.append(arr[i] + "  ") ;
     }

    }

    public void display_network() {
//        show_array("weights",weights);
	    show_array("activations",activations) ;
	    textArea1.append("\nWinner = " + winner + "\n") ;
    }


    // move data from train/test set into network input units
    public void readInputs() {
     recInx = recInx % numRecs ; // keep index from 0 to n-1 records
     int inx=0 ;
     float[] tempRec = (float[])data.elementAt(recInx) ; // get record
     for (inx=0 ; inx < numInputs ; inx++ ) {
       activations[inx] = tempRec[inx] ;
     }
     recInx++ ;
     if (recInx ==0){
        numPasses++ ;           // completed another pass through the data
        adjustNeighborhood() ;
     }
    }

    // initialize a matrix of distances from each unit to all others
    public void computeDistances() {
        int i, j, xi, xj, yi, yj ;
        distance = new int[numOutputs* numOutputs] ;
        for(i=0 ; i < numOutputs ; i++) {
          xi = i % numCols ;
          yi = i / numRows ;
          for (j=0 ; j < numOutputs ; j++) {
            xj = j % numCols ;
            yj = j / numRows ;
            distance[i*numOutputs+j] = (int) Math.pow((double)((xi-xj)*(xi-xj)),2.0)+
                                       (int) Math.pow((double)((yi-yj)*(yi-yj)),2.0);
          }
        }
    }

    public void adjustNeighborhood() {
        double ratio = (double) (numPasses/ maxNumPasses) ;
        learnRate = initLearnRate * (float)Math.pow((double)finalLearnRate / (double)initLearnRate,ratio);
        sigma = (float)numCols * (float)Math.pow((0.20 / (double)numCols),ratio) ;

    }

    // do a single forward pass through the network
    // compute the Euclidean distance from input vector
    // to all output units
    public void computeOutputs() {
     int index, i, j ;
     int lastOut = numUnits - 1;
     int firstOut = numInputs;

     //first layer
     for(i = firstOut ; i <= lastOut ; i++) {
        index = (i - firstOut) * numInputs;
        activations[i] = (float)0.0 ;
       for (j = 0 ; j < numInputs ; j++) { // compute net inputs
         activations[i] += (activations[j] - weights[index+j]) *
                           (activations[j] - weights[index+j]);
       }
     }
   }

    // find the output unit with the smallest activation
    // and declare it the "winner"
    public void selectWinner() {

       winner = 0;
       float min = activations[numInputs];

       for(int i = 0 ; i < numOutputs ; i++) {
         if (activations[i+numInputs] < min) {
            min = activations[i+numInputs] ;
            winner = i ;
         }
       }
    }


    public void adjustWeights() {

    // apply the changes to the weights
     int i, j, inx, base ;
     int numOutputs = numRows * numCols ;
     double dist, range, sigma_squared ;

     sigma_squared = sigma * sigma ;

     for(i = 0 ; i < numOutputs ; i++) {
        dist = Math.exp((distance[winner*numOutputs+i] * -1.0) /
                   (2.0 * sigma_squared));

        base = i * numInputs ; // compute the base index
        range = learnRate * dist ;
        for (j=0 ; j < numInputs ; j++) {
            inx = base + j ;
            weights[inx] += range * (activations[j]- weights[inx]);
        }
      }
    }


    public void cluster() {

     readInputs() ;   // set input unit activations
     computeOutputs() ;  // do forward pass through network
     selectWinner() ;  // find the winning unit

     // only adjust if in training mode
     if (mode == 0) adjustWeights() ;   // apply changes to weights

    }

    public void reset() {
        int i ;
        for (i=0 ; i < weights.length ; i++) {
          weights[i] = (float)0.6 - ((float)0.2 *(float)Math.random());    // between 0.4 and 0.6
        }

    }

    // create a Kohonen network with the specified architecture
    public void createNetwork(int NumIn, int NumRows, int NumCols){

        // set the network architecture
        numInputs = NumIn;
        numRows = NumRows;
        numCols = NumCols;
        numOutputs = numRows * numCols ;
        numUnits = numInputs + numOutputs;

        // initialize control parameters
        learnRate = (float)0.1;
        mode = 0 ;     // 0 = train mode, 1 = run mode

        // create arrays
        activations = new float[numUnits];
        weights = new float[numInputs*numOutputs];

        // fill in the Distance matrix
        computeDistances() ;
  
        adjustNeighborhood() ; // set the initial learnRate
        reset() ;              // reset and initialize weight arrays


        return ;  // all data in the network
    }


	


}
