/*  RuleApplet class

 Constructing Intelligent Agents with Java
   (C) Joseph P.Bigus and Jennifer Bigus 1997

 Modified: 8/14/98 jpb convert to 1.1 Event model

 Copyright (c) 1998 John Wiley & Sons, Inc.  All rights reserved.  
 Reproduction or translation of this work beyond that permitted in 
 Section 117 of the 1976 United States Copyright Act without the 
 express written permission of the copyright owner is unlawful.  
 Requests for further information should be addressed to Permissions 
 Department, John Wiley & Sons, Inc. The purchaser may make back-up 
 copies for his/her own use only and not for distribution or resale.
 The Publisher assumes no responsibility for errors, omissions, or 
 damages, caused by the use of these programs or from the use of the 
 information contained herein.

*/

import java.awt.*;
import java.applet.*;
import java.util.* ;

public class RuleApplet extends Applet {

    // user selected a rule base
    void choice1_Clicked() {
       String rbName = choice1.getSelectedItem() ;
       if (rbName.equals("Vehicles")) currentRuleBase = vehicles ;
       if (rbName.equals("Bugs")) currentRuleBase = bugs ;
       if (rbName.equals("Plants")) currentRuleBase = plants ;

       currentRuleBase.reset() ;  // reset the rule base
       choice2.removeAll();  // jpb -- bug fix
       choice3.removeAll(); 
       Enumeration vars = currentRuleBase.variableList.elements() ;
       while (vars.hasMoreElements()) {
         choice2.addItem(((RuleVariable)vars.nextElement()).name) ;
       }
       currentRuleBase.displayVariables(textArea3) ;

	}

    // user selected a variable
    void choice2_Clicked(java.awt.event.ItemEvent event) {
       String varName = choice2.getSelectedItem() ;
       choice3.removeAll() ;

       RuleVariable rvar = (RuleVariable)currentRuleBase.variableList.get(varName);
       Enumeration labels = rvar.labels.elements();
       while (labels.hasMoreElements()) {
          choice3.addItem(((String)labels.nextElement())) ;
       }
	}

    // user selected a value for a variable
	void choice3_Clicked(java.awt.event.ItemEvent event) {
       String varName = choice2.getSelectedItem() ;
       String varValue = choice3.getSelectedItem() ;

       RuleVariable rvar = (RuleVariable)currentRuleBase.variableList.get(varName);
       rvar.setValue(varValue) ;
       textArea3.append("\n" + rvar.name + " set to " + varValue) ;

	}

    // user pressed Find button -- do an infernece cycle
	void button1_Clicked(java.awt.event.ActionEvent event) {
       String goal = textField1.getText() ;

       textArea2.append("\n --- Starting Inferencing Cycle --- \n");
       currentRuleBase.displayVariables(textArea2) ;
       if (radioButton1.getState() == true) currentRuleBase.forwardChain();
	   if (radioButton2.getState() == true) currentRuleBase.backwardChain(goal);
       currentRuleBase.displayVariables(textArea2) ;
       textArea2.append("\n --- Ending Inferencing Cycle --- \n");
	}

	// user pressed Demo button -- do inference with pre-set values
	void button2_Clicked(java.awt.event.ActionEvent event) {
       String rbName = choice1.getSelectedItem() ;
       if (rbName.equals("Vehicles")) {
         if (radioButton1.getState() == true) demoVehiclesFC(vehicles);
	     if (radioButton2.getState() == true) demoVehiclesBC(vehicles);
	   } else if (rbName.equals("Bugs")) {
         if (radioButton1.getState() == true) demoBugsFC(bugs);
	     if (radioButton2.getState() == true) demoBugsBC(bugs);
	   } else {
	     if (radioButton1.getState() == true) demoPlantsFC(plants);
	     if (radioButton2.getState() == true) demoPlantsBC(plants);
	   }

	}

    void button3_Clicked(java.awt.event.ActionEvent event) {

		//{{CONNECTION
		// Clear the text for TextArea
		textArea1.setText("");
		textArea2.setText("");
		textArea3.setText("");
		//}}

        currentRuleBase.reset() ;
        currentRuleBase.displayRules(textArea1);
        currentRuleBase.displayVariables(textArea3) ;
	}

// display dialog to get user value for a variable
static public String waitForAnswer(String prompt, String labels) {

  	// position dialog over parent dialog

    Frame	frame = new Frame("Ask User") ;
	  dlg = new RuleVarDialog(frame, true) ;
   	dlg.label1.setText("   " + prompt + " (" + labels + ")");
    dlg.setLocation( 200, 200) ;
    dlg.show() ;
    String ans = dlg.getText() ;
    return ans ;

}


	public void init() {
		super.init();


		//{{INIT_CONTROLS
		setLayout(null);
		addNotify();
		resize(624,527);
		button1 = new java.awt.Button("Find Goal");
		button1.setBounds(36,468,108,30);
		add(button1);
		button2 = new java.awt.Button("Run Demo");
		button2.setBounds(228,468,108,30);
		add(button2);
		button3 = new java.awt.Button("Reset");
		button3.setBounds(444,468,108,30);
		add(button3);
		textArea1 = new java.awt.TextArea();
		textArea1.setBounds(12,48,312,144);
		add(textArea1);
		textArea2 = new java.awt.TextArea();
		textArea2.setBounds(12,216,600,168);
		add(textArea2);
		label2 = new java.awt.Label("Trace Log");
		label2.setBounds(24,192,168,24);
		add(label2);
		label1 = new java.awt.Label("Rule Base");
		label1.setBounds(24,12,96,24);
		add(label1);
		choice1 = new java.awt.Choice();
		add(choice1);
		choice1.setBounds(132,12,192,24);
		Group1 = new CheckboxGroup();
		radioButton1 = new java.awt.Checkbox("Forward Chain", Group1, false);
		radioButton1.setBounds(36,396,156,21);
		add(radioButton1);
		choice3 = new java.awt.Choice();
		add(choice3);
		choice3.setBounds(480,36,135,24);
		label5 = new java.awt.Label("Value");
		label5.setBounds(480,12,95,24);
		add(label5);
		choice2 = new java.awt.Choice();
		add(choice2);
		choice2.setBounds(336,36,137,24);
		textArea3 = new java.awt.TextArea();
		textArea3.setBounds(336,72,276,122);
		add(textArea3);
		label4 = new java.awt.Label("Variable");
		label4.setBounds(336,12,109,24);
		add(label4);
		radioButton2 = new java.awt.Checkbox("Backward Chain", Group1, false);
		radioButton2.setBounds(36,420,156,24);
		add(radioButton2);
		textField1 = new java.awt.TextField();
		textField1.setBounds(324,420,142,27);
		add(textField1);
		label3 = new java.awt.Label("Goal");
		label3.setBounds(324,384,80,30);
		add(label3);
		//}}

	 	frame = new Frame("Ask User") ;
	 	frame.setSize(50,50) ;
	 	frame.setLocation(100,100) ;
	 	choice1.addItem("Vehicles") ;
	 	choice1.addItem("Bugs") ;
	 	choice1.addItem("Plants") ;

	    vehicles = new RuleBase("Vehicles Rule Base") ;
	    vehicles.setDisplay(textArea2) ;
	 	initVehiclesRuleBase(vehicles) ;
        currentRuleBase = vehicles ;

	 	bugs = new RuleBase("Bugs Rule Base") ;
	 	bugs.setDisplay(textArea2) ;
	 	initBugsRuleBase(bugs) ;

	 	plants = new RuleBase("Plants Rule Base") ;
	 	plants.setDisplay(textArea2) ;
	 	initPlantsRuleBase(plants) ;

        // initialize textAreas and list controls
        currentRuleBase.displayRules(textArea1) ;
        currentRuleBase.displayVariables(textArea3) ;
        radioButton1.setState(true) ;
        choice1_Clicked() ; // fill variable list
	
		//{{REGISTER_LISTENERS
		SymAction lSymAction = new SymAction();
		button1.addActionListener(lSymAction);
		button2.addActionListener(lSymAction);
		button3.addActionListener(lSymAction);
		SymItem lSymItem = new SymItem();
		choice1.addItemListener(lSymItem);
		choice2.addItemListener(lSymItem);
		choice3.addItemListener(lSymItem);
		//}}
	}
 
	//{{DECLARE_CONTROLS
	java.awt.Button button1;
	java.awt.Button button2;
	java.awt.Button button3;
	java.awt.TextArea textArea1;
	java.awt.TextArea textArea2;
	java.awt.Label label2;
	java.awt.Label label1;
	java.awt.Choice choice1;
	java.awt.Checkbox radioButton1;
	CheckboxGroup Group1;
	java.awt.Choice choice3;
	java.awt.Label label5;
	java.awt.Choice choice2;
	java.awt.TextArea textArea3;
	java.awt.Label label4;
	java.awt.Checkbox radioButton2;
	java.awt.TextField textField1;
	java.awt.Label label3;
	//}}

	static Frame frame ;
	static RuleVarDialog dlg ;
	static RuleBase bugs ;
	static RuleBase plants ;
    static RuleBase vehicles ;
    static RuleBase currentRuleBase ;


 // initialize the Bugs rule base
 public void initBugsRuleBase(RuleBase rb) {
  rb.goalClauseStack = new Stack() ; // goals and subgoals

  rb.variableList = new Hashtable() ;
  RuleVariable bugClass = new RuleVariable("bugClass") ;
  bugClass.setLabels("arachnid insect") ;
  bugClass.setPromptText("What class of bug is it?") ;
  rb.variableList.put(bugClass.name,bugClass) ;

  RuleVariable insectType = new RuleVariable("insectType") ;
  insectType.setLabels("beetle orthoptera") ; 
  insectType.setPromptText("What type of bug is it?") ;
  rb.variableList.put(insectType.name,insectType) ;

  RuleVariable species = new RuleVariable("species") ;
  species.setLabels("Spider Tick Ladybug Japanese_Beetle Cricket Praying_Mantis") ;
  species.setPromptText("What species of bug is it?") ;
  rb.variableList.put(species.name,species) ;

  RuleVariable color = new RuleVariable("color") ;
  color.setLabels("orange_and_black green_and_black black") ;
  color.setPromptText("What color is the bug?") ;
  rb.variableList.put(color.name,color) ;

  RuleVariable size = new RuleVariable("size") ;
  size.setLabels("small large") ;
  size.setPromptText("What size is the bug?") ;
  rb.variableList.put(size.name,size) ;

  RuleVariable leg_length = new RuleVariable("leg_length") ;
  leg_length.setLabels("short long") ;
  leg_length.setPromptText("How long are the bug's legs?") ;
  rb.variableList.put(leg_length.name,leg_length) ;

  RuleVariable antennae = new RuleVariable("antennae") ;
  antennae.setLabels("0 2") ;
  antennae.setPromptText("How many antennae does the bug have?") ;
  rb.variableList.put(antennae.name,antennae) ;

  RuleVariable shape = new RuleVariable("shape") ;
  shape.setLabels("round elongated") ;
  shape.setPromptText("What shape is the bug?") ;
  rb.variableList.put(shape.name,shape) ;

  RuleVariable legs = new RuleVariable("legs") ;
  legs.setLabels("6 8") ;
  legs.setPromptText("How many legs does the bug have?") ;
  rb.variableList.put(legs.name,legs) ;

  RuleVariable wings = new RuleVariable("wings") ;
  wings.setLabels("0 2") ;
  wings.setPromptText("How many wings does the bug have?") ;
  rb.variableList.put(wings.name,wings) ;

  // Note: at this point all variables values are NULL

 Condition cEquals = new Condition("=") ;
 Condition cNotEquals = new Condition("!=") ;

 // define rules
 rb.ruleList = new Vector() ;
 Rule Spider = new Rule(rb, "spider",
      new Clause(bugClass,cEquals, "arachnid")  ,
      new Clause(leg_length,cEquals, "long"),
      new Clause(species,cEquals, "Spider")) ;

 Rule Tick = new Rule(rb, "tick",
      new Clause(bugClass,cEquals, "arachnid")  ,
      new Clause(leg_length,cEquals, "short"),
      new Clause(species,cEquals, "Tick")) ;

 Rule Ladybug = new Rule(rb, "ladybug",
      new Clause(insectType,cEquals, "beetle")  ,
      new Clause(color,cEquals, "orange_and_black"),
      new Clause(species,cEquals, "Ladybug")) ;

 Rule JapaneseBeetle = new Rule(rb, "Japanese_Beetle",
      new Clause(insectType,cEquals, "beetle")  ,
      new Clause(color,cEquals, "green_and_black"),
      new Clause(species,cEquals, "Japanese_Beetle")) ;

 Rule Cricket = new Rule(rb, "cricket",
      new Clause(insectType,cEquals, "orthoptera")  ,
      new Clause(color,cEquals, "black"),
      new Clause(species,cEquals, "Cricket")) ;

 Rule PrayingMantis = new Rule(rb, "praying_mantis",
      new Clause(insectType,cEquals, "orthoptera")  ,
      new Clause(color,cEquals, "green"),
      new Clause(size, cEquals, "large"),
      new Clause(species,cEquals, "Praying_Mantis")) ;

 Rule ClassArachnid1 = new Rule(rb, "class_is_Arachnid1",
      new Clause(antennae,cEquals, "0")  ,
      new Clause(legs,cEquals, "8"),
      new Clause(bugClass,cEquals, "arachnid")) ;

 Rule ClassArachnid2 = new Rule(rb, "class_is_Arachnid2",
      new Clause(wings,cEquals, "0")  ,
      new Clause(bugClass,cEquals, "arachnid")) ;

 Rule ClassInsect1 = new Rule(rb, "class_is_Insect1",
      new Clause(antennae,cEquals, "2")  ,
      new Clause(legs,cEquals, "6"),
      new Clause(bugClass,cEquals, "insect")) ;

 Rule ClassInsect2 = new Rule(rb, "class_is_Insect2",
      new Clause(wings,cEquals, "2")  ,
      new Clause(bugClass,cEquals, "insect")) ;

 Rule TypeBeetle = new Rule(rb, "typeBeetle",
      new Clause(bugClass,cEquals, "insect")  ,
      new Clause(size,cEquals, "small"),
      new Clause(shape, cEquals, "round"),
      new Clause(insectType,cEquals, "beetle")) ;

 Rule TypeOrthoptera = new Rule(rb, "typeOrthoptera",
      new Clause(bugClass,cEquals, "insect")  ,
      new Clause(size,cNotEquals, "small"),
      new Clause(shape, cEquals, "elongated"),
      new Clause(insectType,cEquals, "orthoptera")) ;

}

// initialize the Plants rule base
 public void initPlantsRuleBase(RuleBase rb) {
  rb.goalClauseStack = new Stack() ; // goals and subgoals

  rb.variableList = new Hashtable() ;
  RuleVariable family = new RuleVariable("family") ;
  family.setLabels("cypress pine bald_cypress ") ;
  family.setPromptText("What family of plant is it?") ;
  rb.variableList.put(family.name,family) ;

  RuleVariable treeClass = new RuleVariable("treeClass") ;
  treeClass.setLabels("angiosperm gymnosperm") ;
  treeClass.setPromptText("What class of plant is it?") ;
  rb.variableList.put(treeClass.name, treeClass) ;

  RuleVariable plantType = new RuleVariable("plantType") ;
  plantType.setLabels("herb vine tree shrub") ;
  plantType.setPromptText("What type of plant is it?") ;
  rb.variableList.put(plantType.name,plantType) ;

  RuleVariable stem = new RuleVariable("stem") ;
  stem.setLabels("woody green") ;
  stem.setPromptText("What type of stem does the plant have?") ;
  rb.variableList.put(stem.name,stem) ;

  RuleVariable stemPosition = new RuleVariable("stemPosition") ;
  stemPosition.setLabels("upright creeping") ;
  stemPosition.setPromptText("What kind of stem does the plant have?") ;
  rb.variableList.put(stemPosition.name,stemPosition) ;

  RuleVariable one_main_trunk = new RuleVariable("one_main_trunk") ;
  one_main_trunk.setLabels("yes no") ;
  one_main_trunk.setPromptText("Does the plant have one main trunk?") ;
  rb.variableList.put(one_main_trunk.name,one_main_trunk) ;

  RuleVariable broad_and_flat_leaves = new RuleVariable("broad_and_flat_leaves") ;
  broad_and_flat_leaves.setLabels("yes no") ;
  broad_and_flat_leaves.setPromptText("Does the plant have broad and flat leaves?") ;
  rb.variableList.put(broad_and_flat_leaves.name, broad_and_flat_leaves) ;

  RuleVariable leaf_shape = new RuleVariable("leaf_shape") ;
  leaf_shape.setLabels("needlelike scalelike") ;
  leaf_shape.setPromptText("What kind of leaves does the plant have?") ;
  rb.variableList.put(leaf_shape.name,leaf_shape) ;

  RuleVariable needle_pattern = new RuleVariable("needle_pattern") ;
  needle_pattern.setLabels("random even") ;
  needle_pattern.setPromptText("How are the needles distributed on the plant?") ;
  rb.variableList.put(needle_pattern.name,needle_pattern) ;

  RuleVariable silver_bands = new RuleVariable("silver_bands") ;
  silver_bands.setLabels("yes no") ;
  silver_bands.setPromptText("Does the plant have silver bands?") ;
  rb.variableList.put(silver_bands.name,silver_bands) ;

  // Note: at this point all variables values are NULL

 Condition cEquals = new Condition("=") ;
 Condition cNotEquals = new Condition("!=") ;

 // define rules
 rb.ruleList = new Vector() ;
 Rule Cypress = new Rule(rb, "cypress",
      new Clause(treeClass,cEquals, "gymnosperm")  ,
      new Clause(leaf_shape,cEquals, "scalelike"),
      new Clause(family,cEquals, "cypress")) ;

 Rule Pine1 = new Rule(rb, "pine1",
      new Clause(treeClass,cEquals, "gymnosperm")  ,
      new Clause(leaf_shape,cEquals, "needlelike"),
      new Clause(needle_pattern,cEquals, "random"),
      new Clause(family,cEquals, "pine")) ;

 Rule Pine2 = new Rule(rb, "pine2",
      new Clause(treeClass,cEquals, "gymnosperm")  ,
      new Clause(leaf_shape,cEquals, "needlelike"),
      new Clause(needle_pattern,cEquals, "even"),
      new Clause(silver_bands,cEquals, "yes"),
      new Clause(family,cEquals, "pine")) ;

 Rule BaldCypress = new Rule(rb, "baldCypress",
      new Clause(treeClass,cEquals, "gymnosperm")  ,
      new Clause(leaf_shape,cEquals, "needlelike"),
      new Clause(needle_pattern,cEquals, "even"),
      new Clause(silver_bands,cEquals, "no"),
      new Clause(family,cEquals, "bald_cypress")) ;

 Rule Angiosperm = new Rule(rb, "angiosperm",
      new Clause(plantType,cEquals, "tree")  ,
      new Clause(broad_and_flat_leaves, cEquals, "yes"),
      new Clause(treeClass,cEquals, "angiosperm")) ;

 Rule Gymnosperm = new Rule(rb, "gymnosperm",
      new Clause(plantType,cEquals, "tree")  ,
      new Clause(broad_and_flat_leaves,cEquals, "no"),
      new Clause(treeClass,cEquals, "gymnosperm")) ;

 Rule Herb = new Rule(rb, "herb",
      new Clause(stem,cEquals, "green")  ,
      new Clause(plantType,cEquals, "herb")) ;

 Rule Vine = new Rule(rb, "vine",
      new Clause(stem,cEquals, "woody")  ,
      new Clause(stemPosition,cEquals, "creeping"),
      new Clause(plantType,cEquals, "vine")) ;

 Rule Tree = new Rule(rb, "tree",
      new Clause(stem,cEquals, "woody")  ,
      new Clause(stemPosition,cEquals, "upright"),
      new Clause(one_main_trunk,cEquals, "yes"),
      new Clause(plantType,cEquals, "tree")) ;

 Rule Shrub = new Rule(rb, "shrub",
      new Clause(stem,cEquals, "woody")  ,
      new Clause(stemPosition,cEquals, "upright"),
      new Clause(one_main_trunk,cEquals, "no"),
      new Clause(plantType,cEquals, "shrub")) ;

 }


// initialize the Vehicles rule base
 public void initVehiclesRuleBase(RuleBase rb) {
  rb.goalClauseStack = new Stack() ; // goals and subgoals

  rb.variableList = new Hashtable() ;
  RuleVariable vehicle = new RuleVariable("vehicle") ;
  vehicle.setLabels("Bicycle Tricycle MotorCycle Sports_Car Sedan MiniVan Sports_Utility_Vehicle") ;
  vehicle.setPromptText("What kind of vehicle is it?");
  rb.variableList.put(vehicle.name,vehicle) ;

  RuleVariable vehicleType = new RuleVariable("vehicleType") ;
  vehicleType.setLabels("cycle automobile") ;
  vehicleType.setPromptText("What type of vehicle is it?") ;
  rb.variableList.put(vehicleType.name, vehicleType) ;

  RuleVariable size = new RuleVariable("size") ;
  size.setLabels("small medium large") ;
  size.setPromptText("What size is the vehicle?") ;
  rb.variableList.put(size.name,size) ;

  RuleVariable motor = new RuleVariable("motor") ;
  motor.setLabels("yes no") ;
  motor.setPromptText("Does the vehicle have a motor?") ;
  rb.variableList.put(motor.name,motor) ;

  RuleVariable num_wheels = new RuleVariable("num_wheels") ;
  num_wheels.setLabels("2 3 4") ;
  num_wheels.setPromptText("How many wheels does it have?");
  rb.variableList.put(num_wheels.name,num_wheels) ;

  RuleVariable num_doors = new RuleVariable("num_doors") ;
  num_doors.setLabels("2 3 4") ;
  num_doors.setPromptText("How many doors does it have?") ;
  rb.variableList.put(num_doors.name,num_doors) ;

  // Note: at this point all variables values are NULL

 Condition cEquals = new Condition("=") ;
 Condition cNotEquals = new Condition("!=") ;
 Condition cLessThan = new Condition("<") ;

 // define rules
 rb.ruleList = new Vector() ;
 Rule Bicycle = new Rule(rb, "bicycle",
      new Clause(vehicleType,cEquals, "cycle")  ,
      new Clause(num_wheels,cEquals, "2"),
      new Clause(motor, cEquals, "no"),
      new Clause(vehicle, cEquals, "Bicycle")) ;

 Rule Tricycle = new Rule(rb, "tricycle",
      new Clause(vehicleType,cEquals, "cycle")  ,
      new Clause(num_wheels,cEquals, "3"),
      new Clause(motor, cEquals, "no"),
      new Clause(vehicle, cEquals, "Tricycle")) ;

 Rule Motorcycle = new Rule(rb, "motorcycle",
      new Clause(vehicleType,cEquals, "cycle")  ,
      new Clause(num_wheels,cEquals, "2"),
      new Clause(motor,cEquals, "yes"),
      new Clause(vehicle,cEquals, "Motorcycle")) ;

 Rule SportsCar = new Rule(rb, "sportsCar",
      new Clause(vehicleType,cEquals, "automobile")  ,
      new Clause(size,cEquals, "small"),
      new Clause(num_doors,cEquals, "2"),
      new Clause(vehicle,cEquals, "Sports_Car")) ;

 Rule Sedan = new Rule(rb, "sedan",
      new Clause(vehicleType,cEquals, "automobile")  ,
      new Clause(size,cEquals, "medium"),
      new Clause(num_doors,cEquals, "4"),
      new Clause(vehicle,cEquals, "Sedan")) ;

 Rule MiniVan = new Rule(rb, "miniVan",
      new Clause(vehicleType,cEquals, "automobile")  ,
      new Clause(size,cEquals, "medium"),
      new Clause(num_doors,cEquals, "3"),
      new Clause(vehicle,cEquals, "MiniVan")) ;

 Rule SUV = new Rule(rb, "SUV",
      new Clause(vehicleType,cEquals, "automobile")  ,
      new Clause(size,cEquals, "large"),
      new Clause(num_doors,cEquals, "4"),
      new Clause(vehicle,cEquals, "Sports_Utility_Vehicle")) ;

 Rule Cycle = new Rule(rb, "Cycle",
      new Clause(num_wheels,cLessThan, "4")  ,
      new Clause(vehicleType,cEquals, "cycle")) ;

 Rule Automobile = new Rule(rb, "Automobile",
      new Clause(num_wheels,cEquals, "4")  ,
      new Clause(motor,cEquals, "yes"),
      new Clause(vehicleType,cEquals, "automobile")) ;

 }

public void demoVehiclesFC(RuleBase rb) {

 textArea2.append("\n --- Starting Demo ForwardChain ---\n ") ;
  // should be a Mini-Van
 ((RuleVariable)rb.variableList.get("vehicle")).setValue(null) ;
 ((RuleVariable)rb.variableList.get("vehicleType")).setValue(null) ;
 ((RuleVariable)rb.variableList.get("size")).setValue("medium") ;
 ((RuleVariable)rb.variableList.get("num_wheels")).setValue("4") ;
 ((RuleVariable)rb.variableList.get("num_doors")).setValue("3") ;
 ((RuleVariable)rb.variableList.get("motor")).setValue("yes") ;
  rb.displayVariables(textArea2) ;
  rb.forwardChain() ;  // chain until quiescence...
  textArea2.append("\n --- Stopping Demo ForwardChain! ---\n") ;

  rb.displayVariables(textArea2);
 }


public void demoVehiclesBC(RuleBase rb) {
 textArea2.append("\n --- Starting Demo BackwardChain ---\n ") ;
  // should be a minivan
 ((RuleVariable)rb.variableList.get("vehicle")).setValue(null) ;
 ((RuleVariable)rb.variableList.get("vehicleType")).setValue(null) ;
 ((RuleVariable)rb.variableList.get("size")).setValue("medium") ;
 ((RuleVariable)rb.variableList.get("num_wheels")).setValue("4") ;
 ((RuleVariable)rb.variableList.get("num_doors")).setValue("3") ;
 ((RuleVariable)rb.variableList.get("motor")).setValue("yes") ;
  rb.displayVariables(textArea2) ;
  rb.backwardChain("vehicle") ;  // chain until quiescence...
  textArea2.append("\n --- Stopping Demo BackwardChain! ---\n ") ;
  rb.displayVariables(textArea2) ;
}




public void demoBugsBC(RuleBase rb) {
 textArea2.append("\n --- Starting Demo BackwardChain ---\n ") ;
  // should be a insect, ladybug
 ((RuleVariable)rb.variableList.get("wings")).setValue("2") ;
 ((RuleVariable)rb.variableList.get("legs")).setValue("6") ;
 ((RuleVariable)rb.variableList.get("shape")).setValue("round") ;
 ((RuleVariable)rb.variableList.get("antennae")).setValue("2") ;
 ((RuleVariable)rb.variableList.get("color")).setValue("orange_and_black") ;
 ((RuleVariable)rb.variableList.get("leg_length")).setValue("long") ;
 ((RuleVariable)rb.variableList.get("size")).setValue("small") ;
  rb.displayVariables(textArea2) ;
  rb.backwardChain("species") ;  // chain until quiescence...
  textArea2.append("\n --- Stopping Demo BackwardChain! ---\n") ;
  rb.displayVariables(textArea2) ;
}

public void demoBugsFC(RuleBase rb) {

 textArea2.append("\n --- Starting Demo ForwardChain ---\n") ;
  // should be a insect, ladybug
 ((RuleVariable)rb.variableList.get("bugClass")).setValue(null) ;
 ((RuleVariable)rb.variableList.get("insectType")).setValue(null) ;
 ((RuleVariable)rb.variableList.get("wings")).setValue("2") ;
 ((RuleVariable)rb.variableList.get("legs")).setValue("6") ;
 ((RuleVariable)rb.variableList.get("shape")).setValue("round") ;
 ((RuleVariable)rb.variableList.get("antennae")).setValue("2") ;
 ((RuleVariable)rb.variableList.get("color")).setValue("orange_and_black") ;
 ((RuleVariable)rb.variableList.get("leg_length")).setValue("long") ;
 ((RuleVariable)rb.variableList.get("size")).setValue("small") ;
  rb.displayVariables(textArea2) ;
  rb.forwardChain() ;  // chain until quiescence...
  textArea2.append("\n --- Stopping Demo ForwardChain! ---\n") ;

 // now display the results
 rb.displayVariables(textArea2);
 }

public void demoPlantsBC(RuleBase rb) {
 textArea2.append("\n --- Starting Demo BackwardChain ---\n ") ;
  // should be a pine tree
 ((RuleVariable)rb.variableList.get("stem")).setValue("woody") ;
 ((RuleVariable)rb.variableList.get("stemPosition")).setValue("upright") ;
 ((RuleVariable)rb.variableList.get("one_main_trunk")).setValue("yes") ;
 ((RuleVariable)rb.variableList.get("broad_and_flat_leaves")).setValue("no") ;
 ((RuleVariable)rb.variableList.get("leaf_shape")).setValue("needlelike") ;
 ((RuleVariable)rb.variableList.get("needle_pattern")).setValue("random") ;
  rb.displayVariables(textArea2) ;

  rb.backwardChain("family") ;  // chain until quiescence...
  textArea2.append("\n --- Stopping Demo BackwardChain! ---\n") ;
  rb.displayVariables(textArea2) ;
}

public void demoPlantsFC(RuleBase rb) {

 textArea2.append("\n --- Starting Demo ForwardChain --- \n ") ;
 // should be a pine tree
 ((RuleVariable)rb.variableList.get("stem")).setValue("woody") ;
 ((RuleVariable)rb.variableList.get("stemPosition")).setValue("upright") ;
 ((RuleVariable)rb.variableList.get("one_main_trunk")).setValue("yes") ;
 ((RuleVariable)rb.variableList.get("broad_and_flat_leaves")).setValue("no") ;
 ((RuleVariable)rb.variableList.get("leaf_shape")).setValue("needlelike") ;
 ((RuleVariable)rb.variableList.get("needle_pattern")).setValue("random") ;
  rb.displayVariables(textArea2) ;

  rb.forwardChain() ;  // chain until quiescence...
  textArea2.append("\n --- Stopping Demo ForwardChain! --- \n") ;

  rb.displayVariables(textArea2);
 }

//-------------------------------------------------------------
 // This file has been migrated from the 1.0 to 1.1 event model.
 // This method is not used with the new 1.1 event model. You can 
 // move any code you need to keep, then remove this method.
 //-------------------------------------------------------------
 // 
 // 
 // 
 // 	public boolean handleEvent(Event event) {
 // 		if (event.target == button1 && event.id == Event.ACTION_EVENT) {
 // 			button1_Clicked(event);
 // 			return true;
 // 		}if (event.target == button2 && event.id == Event.ACTION_EVENT) {
 // 			button2_Clicked(event);
 // 			return true;
 // 		}if (event.target == button3 && event.id == Event.ACTION_EVENT) {
 // 			button3_Clicked(event);
 // 			return true;
 // 		}
 // 		if (event.target == dlg && event.id == Event.ACTION_EVENT) {
 // 			return dlg.handleEvent(event);
 // 		}
 // 	    if (event.target == choice1 && event.id == Event.ACTION_EVENT) {
 // 			choice1_Clicked();
 // 			return true;
 // 	    }
 // 	    if (event.target == choice2 && event.id == Event.ACTION_EVENT) {
 // 			choice2_Clicked(event);
 // 			return true;
 // 	    }
 // 	        if (event.target == choice3 && event.id == Event.ACTION_EVENT) {
 // 			choice3_Clicked(event);
 // 			return true;
 // 	    }
 // 		return super.handleEvent(event);
 // 	}
 //-------------------------------------------------------------


	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == button1)
				button1_Clicked(event);
			else if (object == button2)
				button2_Clicked(event);
			else if (object == button3)
				button3_Clicked(event);
		}
	}

	class SymItem implements java.awt.event.ItemListener
	{
		public void itemStateChanged(java.awt.event.ItemEvent event)
		{
			Object object = event.getSource();
			if (object == choice1)
				choice1_Clicked();
			else if (object == choice2)
				choice2_Clicked(event);
			else if (object == choice3)
				choice3_Clicked(event);
		}
	}
}
