/* BestBuyerAgent class

   Constructing Intelligent Agents with Java
   (C) Joseph P. Bigus and Jennifer Bigus 1997

*/
// package sunw.demo.ciagent ;
 import java.util.*;

 public class BestBuyerAgent extends BuyerAgent implements Effector {

  RuleBase rb = new RuleBase("BestBuyer");
  RuleVariable offerDelta;
  RuleVariable spread ;
  RuleVariable firstOffer ;
  Offer offer ;   // the current offer

  public void initialize() {
    initBestBuyerRuleBase() ;
    rb.setDisplay(textArea) ;  // give rulebase a display area
  }

  public BestBuyerAgent() { name = "BestBuyer"; }

  public BestBuyerAgent(String Name) { super(Name); }


  // used by action rules in rule base to make or accept offers
  public long effector(Object obj, String eName, String args) {

    if (eName.equals("make-offer")) {
        // rule base decided to counter-offer
        long delta = (new Long(offerDelta.getValue())).longValue() ;
        current.lastOffer = offer.price - delta ;  //
        CIAgentMessage answer = new CIAgentMessage("make-offer", offer.item + " " + offer.id + " " + current.lastOffer, offer.item , msg.sender , offer.item , name) ;
        CIAgentEvent e = new CIAgentEvent(this, answer) ;
        notifyCIAgentEventListeners(e);   // respond through Facilitator
        return 0 ;
    }

    if (eName.equals("accept-offer")) {
        // rule base decided to accept the offer
        // just resend it to the seller
        CIAgentMessage answer = new CIAgentMessage("make-offer", msg.content , msg.replyWith , msg.sender , offer.item , name) ;
        CIAgentEvent e = new CIAgentEvent(this, answer) ;
        notifyCIAgentEventListeners(e);   // respond through Facilitator
        return 0 ;
    }
    return 1 ; // unknown effector method
  }


  // this method must be overridden to provide alternate
  // negotiation strategies
  void negotiate(Offer offer, CIAgentMessage msg) {

     // figure out if 1st time or second time or other
     // compute spread
     // let rule base figure out our response
     // if deltaoffer is 0 then we should accept

     rb.reset() ;  // allow all rules to fire, clear vars
     this.offer = offer;  // make visible to rule base effectors

     if (offer.price >= current.strikePrice) {

        long delta = offer.price - current.strikePrice;
        if (delta < 25) {
          spread.setValue("<25") ;
        } else if (delta < 50) {
          spread.setValue("25-50") ;
        } else if (delta >= 50) {
          spread.setValue(">50") ;
        }
     } else {
        spread.setValue("0") ; // meets our price, accept

     }
     if (current.prevOffer.id.equals("")) {
         firstOffer.setValue("yes") ;
       } else {
         firstOffer.setValue("no") ;
     }

     offerDelta.setValue(null) ;
     rb.forwardChain() ;  // inference

     if (offerDelta.getValue() == null) {
        trace(name + " rule base couldn't decide what to do.\n") ;
     }
  }


 // initialize the BestBuyer rule base
 public void initBestBuyerRuleBase() {
  rb.goalClauseStack = new Stack() ; // goals and subgoals

  rb.variableList = new Hashtable() ;

  offerDelta = new RuleVariable("offerDelta") ;
  offerDelta.setLabels("0 25 50 100") ;
  rb.variableList.put(offerDelta.name, offerDelta) ;

  firstOffer = new RuleVariable("firstOffer") ;
  firstOffer.setLabels("yes no") ;
  rb.variableList.put(firstOffer.name, firstOffer) ;

  spread = new RuleVariable("spread") ;
  spread.setLabels("<25 25-50 >50") ;
  rb.variableList.put(spread.name, spread) ;



  // Note: at this point all variables values are NULL

 Condition cEquals = new Condition("=") ;
 Condition cNotEquals = new Condition("!=") ;

 // define rules
 rb.ruleList = new Vector() ;
 Rule first = new Rule(rb, "first",
      new Clause(firstOffer,cEquals, "yes") ,
      new Clause(offerDelta,cEquals, "75")) ;  // counter

 Rule second1 = new Rule(rb, "second1",
      new Clause(firstOffer, cEquals, "no"),
      new Clause(spread, cEquals, ">50"),
      new Clause(offerDelta,cEquals, "50")) ; // counter

 Rule second2 = new Rule(rb, "second2",
      new Clause(firstOffer, cEquals, "no"),
      new Clause(spread, cEquals, "25-50"),
      new Clause(offerDelta,cEquals, "25")) ;  // counter

 Rule second3 = new Rule(rb, "second3",
      new Clause(firstOffer, cEquals, "no"),
      new Clause(spread, cEquals, "<25"),
      new Clause(offerDelta,cEquals, "0")) ;  // accept

 // action rule
 Rule accept = new Rule(rb, "accept",
      new Clause(offerDelta, cEquals, "0"),
      new EffectorClause("accept-offer", "0")) ;


 // action rule
 Rule counter = new Rule(rb, "counter",
      new Clause(offerDelta, cNotEquals, "0"),
      new EffectorClause("make-offer", null)) ; // use offerDelta

 // define this object as effector implementor
 rb.addEffector(this, "make-offer");
 rb.addEffector(this, "accept-offer");
 }

 };
