/* BestSellerAgent class

   Constructing Intelligent Agents with Java
   (C) Joseph P. Bigus and Jennifer Bigus 1997

*/
// package sunw.demo.ciagent ;
 import java.util.*;

 public class BestSellerAgent extends SellerAgent implements Effector {

  RuleBase rb = new RuleBase("BestSeller");
  RuleVariable offerDelta;
  RuleVariable spread ;
  RuleVariable firstOffer ;
  Offer offer ;   // the current offer

  public void initialize() {
    initBestSellerRuleBase() ;
    rb.setDisplay(textArea) ;  // give rulebase a display area
  }

  public BestSellerAgent() { name = "BestSeller"; }

  public BestSellerAgent(String Name) { super(Name); } ;

  // used by action rules in rule base to make or accept offers
  public long effector(Object obj, String eName, String args) {

    if (eName.equals("make-offer")) {
        // rule base decided to counter-offer
        long delta = (new Long(offerDelta.getValue())).longValue() ;
        if (delta == -100) {
           current.lastOffer = current.strikePrice ;
        } else {
           current.lastOffer = offer.price + delta ;  //
        }
        CIAgentMessage answer = new CIAgentMessage("make-offer", offer.item + " " + offer.id + " " + current.lastOffer, offer.item , msg.sender , offer.item , name) ;
        CIAgentEvent e = new CIAgentEvent(this, answer) ;
        notifyCIAgentEventListeners(e);   // respond through Facilitator
        return 0 ;
    }
   if (eName.equals("reject-offer")) {
        // rule base decided to reject the offer
         rejectOffer(offer) ;
         return 0 ;
    }

    if (eName.equals("accept-offer")) {
        // rule base decided to accept the offer
        CIAgentMessage answer = new CIAgentMessage("accept-offer", msg.content , msg.replyWith , msg.sender , offer.item , name) ;
        CIAgentEvent e = new CIAgentEvent(this, answer) ;
        notifyCIAgentEventListeners(e);   // respond through Facilitator
        return 0 ;
    }
    return 1 ; // unknown effector method
  }




 void negotiate(Offer offer, CIAgentMessage msg) {

    // figure out if 1st time or second time or other
    // compute spread
    // let rule base figure out our response
    // if deltaoffer is 0 then we should accept
     rb.reset() ;  // allow all rules to fire, clear vars
     this.offer = offer;  // make visible to rule base effectors


    if (offer.price < current.strikePrice) {
        spread.setValue("<0") ; // below asking price
    } else {                    // above asking price
       long delta = offer.price - current.strikePrice ;
       if (delta < 25) {
         spread.setValue("0-25") ;
       } else if (delta < 50) {
         spread.setValue("25-50") ;
       } else if (delta >= 50) {
         spread.setValue(">50") ;
       }
    }

    // if buyer echos the lastOffer, then he is agreeing
    if (offer.price == current.lastOffer) {
        spread.setValue(">50") ; // accept via rule action
    }


     if (current.iteration == 0) {
         firstOffer.setValue("yes") ;
     } else {
         firstOffer.setValue("no") ;
     }

     current.iteration++ ; // increment iteration count

     if (current.iteration > 10) {  // could be randomized
         rejectOffer(offer) ;
         return; // break off negotiations
      }

     rb.forwardChain() ;  // inference

     if (offerDelta.getValue() == null) {
        trace(name + " rule base couldn't decide what to do -- so reject it.\n") ;
        rejectOffer(offer) ;
      }


  }

 // initialize the BestSeler rule base
 public void initBestSellerRuleBase() {
  rb.goalClauseStack = new Stack() ; // goals and subgoals

  rb.variableList = new Hashtable() ;

  offerDelta = new RuleVariable("offerDelta") ;
  offerDelta.setLabels("-100 0 30 60 100") ;
  rb.variableList.put(offerDelta.name, offerDelta) ;

  firstOffer = new RuleVariable("firstOffer") ;
  firstOffer.setLabels("yes no") ;
  rb.variableList.put(firstOffer.name, firstOffer) ;

  spread = new RuleVariable("spread") ;
  spread.setLabels("<0 0-25 25-50 >50") ;
  rb.variableList.put(spread.name, spread) ;


  // Note: at this point all variables values are NULL

 Condition cEquals = new Condition("=") ;
 Condition cNotEquals = new Condition("!=") ;

 // define rules
 rb.ruleList = new Vector() ;
 Rule first = new Rule(rb, "first",
      new Clause(firstOffer,cEquals, "yes") ,
      new Clause(offerDelta,cEquals, "50")) ;  // counter

 Rule second1 = new Rule(rb, "second1",
      new Clause(firstOffer,cEquals, "no")  ,
      new Clause(spread, cEquals, "25-50"),
      new Clause(offerDelta,cEquals, "40")) ;  // counter

 Rule second2 = new Rule(rb, "second2",
      new Clause(firstOffer,cEquals, "no")  ,
      new Clause(spread, cEquals, "0-25"),
      new Clause(offerDelta,cEquals, "0")) ;  // accept

 Rule second3 = new Rule(rb, "second3",
      new Clause(firstOffer,cEquals, "no")  ,
      new Clause(spread, cEquals, ">50"),
      new Clause(offerDelta,cEquals, "0")) ; // accept


 Rule second4 = new Rule(rb, "second4",
      new Clause(firstOffer,cEquals, "no")  ,
      new Clause(spread, cEquals, "<0"),
      new Clause(offerDelta,cEquals, "-100")) ;  // reject


// action rule
 Rule accept = new Rule(rb, "accept",
      new Clause(offerDelta, cEquals, "0"),
      new EffectorClause("accept-offer", "0")) ;

// action rule
 Rule reject = new Rule(rb, "reject",
      new Clause(firstOffer, cEquals, "no"),
      new Clause(offerDelta, cEquals, "-100"),
      new EffectorClause("reject-offer", "0")) ;

 // action rule
 Rule counter = new Rule(rb, "counter",
      new Clause(offerDelta, cNotEquals, "0"),
      new EffectorClause("make-offer", null)) ; // use offerDelta

 // define this object as effector implementor
 rb.addEffector(this, "make-offer");
 rb.addEffector(this, "reject-offer");
 rb.addEffector(this, "accept-offer");
 }


};
