/* BuyerAgent class

   Constructing Intelligent Agents with Java
   (C) Joseph P. Bigus and Jennifer Bigus 1997

*/
// package sunw.demo.ciagent ;
 import java.util.*;

 public class BuyerAgent extends CIAgent {

  CIAgentMessage msg ;    // current message being processed
  BasicNegotiation current;

  Vector wishList = new Vector() ;

  BasicNegotiation pending = null ; // item waiting for seller

  Hashtable inventory = new Hashtable() ; // items we have purchased
  long totalSpent = 0 ;                   // total money spent on items

  Hashtable negotiations = new Hashtable() ; // transaction history
                                              // key is the item id

  public BuyerAgent() { name = "Buyer"; }

  public BuyerAgent(String Name) { super(Name); } ;


  public void initialize() {} ;

  public void process() {
    initialize() ;  // call any specific initialization code
    stopped = false ;
    // start a thread running and send an update message every interval secs
    trace(name + ": process() \n") ;
    msg = new CIAgentMessage("register:", name , null , null , null , null) ;
    CIAgentEvent e = new CIAgentEvent(this, msg) ;
    FacilitatorAgent.register(e) ;   // add this agent to the Facilitator???

    // add items to buy on wish list
    wishList.addElement(new BasicNegotiation("guitar", 100)) ;
    wishList.addElement(new BasicNegotiation("drums", 200)) ;
    wishList.addElement(new BasicNegotiation("guitar", 100)) ;
    runnit = new Thread(this) ;
    runnit.start() ;
 }

 public void stop() {
    trace(name + ": stopped \n") ;
    stopped = true ;
    runnit.stop() ;
}

 // method of Runnable (Thread) interface
 public void run() {
    while(stopped == false){
      try {
         Thread.sleep((long)5 * 1000) ;  // come alive every 5 seconds
         if (traceLevel > 0) trace(name + ":  active \n") ;
      }
      catch (InterruptedException e)
      {
        // interrupted
      }
      if ((wishList.size() > 0) && (pending == null)) {
        current = (BasicNegotiation)wishList.firstElement() ;
        pending = current ; // we have a pending negotiation
        wishList.removeElementAt(0);
        trace(name + " is looking to buy " + current.offer.item + "\n");
        msg = new CIAgentMessage("recommend-one", current.offer.item , null , "Seller" , current.offer.item , name) ;
        CIAgentEvent e = new CIAgentEvent(this, msg) ;
        notifyCIAgentEventListeners(e);   // signal interested observer
      }
      trace(name + " has purchased " + inventory.size() + " items for " + totalSpent + "\n");
  }

 }

 public  void ciaEventFired(CIAgentEvent e) {
    if (traceLevel > 0) {
      trace(name + ":  CIAgentEvent received by " + name +
            " from " + e.getSource() + " with args " + e.getArgObject()) ;
    }
    Object arg = e.getArgObject() ;
    msg = (CIAgentMessage)arg ;
    if (traceLevel > 0) msg.display() ; // show message contents
    processMessage(msg) ;
 }


  public void processMessage(CIAgentMessage msg) {

  // facilitator has found a seller for item
  if (msg.sender.equals("Facilitator")) {
    if (msg.performative.equals("tell" )) {
      // OK, ask seller what he wants for the item
      CIAgentMessage answer = new CIAgentMessage("ask", pending.offer.item, msg.replyWith , msg.content , pending.offer.item , name) ;
      CIAgentEvent e = new CIAgentEvent(this, answer) ;
      notifyCIAgentEventListeners(e);   // respond through Facilitator
      return ;
    }
  }

    // seller is denying he has any items to sell
    // this is a response to an "ask"
    if (msg.performative.equals("deny" )) {
        trace(name + ": Seller denied our 'ask' about " + msg.content + "\n") ;
        wishList.addElement(pending); // put back on wish list
        pending = null ;  // try again
        return;
    }


   Offer offer = new Offer(msg) ;  //
   trace(name + ": Offer from " + offer.sender + ": content is " +
         offer.item + " " + offer.id + " " + offer.price + "\n") ;

   if (negotiations.containsKey(offer.id)) {
      current = (BasicNegotiation)negotiations.get(offer.id) ;

   } else {
      current = pending ; // get pending negotiation
      pending = null ;  // no items pending
      negotiations.put(offer.id, current) ; // place on active list
  }
    current.newOffer(offer) ; // update the negotiation object

    // seller has agreed and sales transaction is complete
    // transfer item to buyer, seller accepts our last offer
    if (msg.performative.equals("accept-offer" )) {
      trace(name + ": OK -- sale of item " + offer.item + " with id " + offer.id +
             " at price of " + offer.price + " is complete \n") ;
      CIAgentMessage answer = new CIAgentMessage("tell", msg.content , msg.replyWith , msg.sender , offer.item , name) ;
      trace(name + ": " + offer.item + " purchased! \n") ;
      inventory.put(offer.id, current) ;
      negotiations.remove(offer.id) ;
      totalSpent += offer.price ;
      CIAgentEvent e = new CIAgentEvent(this, answer) ;
      notifyCIAgentEventListeners(e);   // respond through Facilitator
    }

    // seller is making a counter offer
    // we can either, make a counter offer, accept, or reject
    if (msg.performative.equals("make-offer" )) {
        negotiate(offer, msg) ;
    }


    // seller is rejecting our last offer - no counter offer
    if (msg.performative.equals("reject-offer" )) {
        trace(name + ": " + msg.sender + " rejected our last offer ") ;
        negotiations.remove(offer.id) ;  // remove from active list
        wishList.addElement(current); // put back on wish list
    }
  }

  // this method must be overridden to provide alternate
  // negotiation strategies
  void negotiate(Offer offer, CIAgentMessage msg) {

        if (offer.price < current.strikePrice) {
           // accept the offer -- by repeating it to the seller
           CIAgentMessage answer = new CIAgentMessage("make-offer", msg.content , msg.replyWith , msg.sender , offer.item , name) ;
           CIAgentEvent e = new CIAgentEvent(this, answer) ;
           notifyCIAgentEventListeners(e);   // respond through Facilitator
        } else {
          // make a counter offer
          current.lastOffer = offer.price - 25 ;  //
          CIAgentMessage answer = new CIAgentMessage("make-offer", offer.item + " " + offer.id + " " + current.lastOffer, offer.item , msg.sender , offer.item , name) ;
          CIAgentEvent e = new CIAgentEvent(this, answer) ;
          notifyCIAgentEventListeners(e);   // respond through Facilitator
        }
    }

};
