/* FacilitatorAgent class

   Constructing Intelligent Agents with Java
   (C) Joseph P. Bigus and Jennifer Bigus 1997

*/
// package sunw.demo.ciagent ;
 import java.util.*;

 public class FacilitatorAgent extends CIAgent {

  private static FacilitatorAgent instance = null ;  // Singleton

  // in Singleton design pattern, used to get single instance
  static public FacilitatorAgent Instance() {
        if (instance == null) {
            instance = new FacilitatorAgent("Facilitator");
        }
        return instance ;
  }

  // Note: can't be used as a Java Bean without public constructor
  protected FacilitatorAgent() { super() ; name = "Facilitator"; }

  protected FacilitatorAgent(String Name) { super(Name); } ;


  public java.awt.TextArea textArea ;
  public synchronized void trace(String msg) { textArea.append(msg) ; }

  Random random = new Random() ;  // used to select agents

  Hashtable allAgents = new Hashtable() ;    // agent name is key, agent object is value
  Hashtable communities = new Hashtable() ;  // domain name is key, vector of agent object is value
  CIAgentMessage msg ;    // current message being processed

  public void reset() {
   allAgents = new Hashtable() ;  // clear all agents
   communities = new Hashtable() ; // clear all communities
  }

  public void process() {
    stopped = false ;
    // start a thread running and send an update message every interval secs
    trace("Facilitator: process() \n") ;
    runnit = new Thread(this) ;
    runnit.start() ;
 }

 public void stop() {
    stopped = true;
    trace("Facilitator: stopped \n") ;
    runnit.stop() ;
}

 // method of Runnable (Thread) interface
 public void run() {
    while(stopped == false){
        try {
           Thread.sleep((long)10*1000) ;  // come alive every 10 seconds
           if (traceLevel > 0) trace("Facilitator: active \n") ;
        }
        catch (InterruptedException e)
        {
          // interrupted
        }
       // do any housekeeping here !!!
    }

 }

  public  void ciaEventFired(CIAgentEvent e) {
      if (traceLevel > 0) {
        trace("Facilitator:  CIAgentEvent received by " + name +
              " from " + e.getSource() + " with args " + e.getArgObject() + "\n") ;
      }
      Object arg = e.getArgObject() ;
      msg = (CIAgentMessage)arg ;
      if (traceLevel > 0) msg.display() ;
      route(msg) ;
  }

  public static synchronized void register(CIAgentEvent e) {
      if (instance == null) instance = new FacilitatorAgent() ;
      instance.allAgents.put(((CIAgent)e.getSource()).getName(), e.getSource()) ;
      ((CIAgent)e.getSource()).addCIAgentEventListener(instance) ;
      instance.addCIAgentEventListener((CIAgent)e.getSource()) ;
  }


  public synchronized void route(CIAgentMessage msg) {
    CIAgent sender = (CIAgent)allAgents.get(msg.sender) ;
  // agent wants to say they can handle questions concerning content
  if (msg.performative.equals("advertise")) {
      trace("Facilitator: adding " + msg.sender +
                         " to " + msg.content + " community \n") ;

     if (communities.containsKey(msg.content)) {
         Vector agents = (Vector)communities.get(msg.content) ;
         agents.addElement(sender) ;
     } else {
         Vector agents = new Vector() ;
         communities.put(msg.content, agents) ;
         agents.addElement(sender) ;
     }
     return ;
   }

  // agent wants to remove themselves from the community
  if (msg.performative.equals("unadvertise")) {
      trace("Facilitator: removing " + msg.sender +
                         " from " + msg.content + " community \n") ;

     if (communities.containsKey(msg.content)) {
         Vector agents = (Vector)communities.get(msg.content) ;
         agents.removeElement(sender) ;
         if (agents.size() == 0) communities.remove(msg.content) ;
     }
     return ;
   }

   // agent wants facilitator to recommend an agent to process content
   // randomly pick one from the list of advertising seller agents
   if (msg.performative.equals("recommend-one")) {
     String item = msg.content ;
     if (communities.containsKey(msg.content)) {
       Vector agents = (Vector)communities.get(msg.content) ;
       int num = agents.size() ;
       int index;
       if (num > 1) {
        double rand = random.nextDouble() ;
         index = (int)(rand * num) ;
        } else {
         index = 0 ;
       }
       CIAgent agent = (CIAgent)agents.elementAt(index) ;  // first agent?
       msg.performative = "tell" ;
       msg.content = agent.name ;
       msg.receiver = msg.sender ;
       msg.sender = name ; // show facilitator is the sender of this message
     // need to construct an anwser message and send it to the sender agent
       trace("Facilitator: Recommended " + agent.name +
                           " to " + msg.receiver +
                           " for " + item + "\n") ;
       sender.ciaEventFired(new CIAgentEvent(this,msg)) ;

     } else {
        trace("Facilitator: there are no agents advertising " +
                            msg.content + "\n") ;
     }
     return ;
   }
   trace("Facilitator: routing " + msg.performative +
         " message from " + msg.sender + " to " + msg.receiver + "\n") ;
   // route the message to the receiver agent
   CIAgent receiver = (CIAgent)allAgents.get(msg.receiver) ;
   if (receiver != null) {
      receiver.ciaEventFired(new CIAgentEvent(this,msg)) ;
   } else {
      // should tell sender that receiver could not be found?
      trace("Facilitator: receiver " +
             msg.receiver + " is unknown! \n") ;
   }

  }
};
