/* Marketplace Application Class

   Constructing Intelligent Agents with Java
   (C) Joseph P. Bigus and Jennifer Bigus 1997
   
   Modified: 8/14/98 jpb convert to 1.1 Event model
 */

import java.awt.*;

public class Marketplace extends Frame {

    // used by agents to display messages in market place window
    synchronized void trace(String msg) { textArea2.append(msg) ; }



	void About_Action(java.awt.event.ActionEvent event) {
		//{{CONNECTION
		// Action from About Create and show as modal
		(new AboutMarketplaceDialog(this, true)).show();
		//}}
	}

	void Exit_Action(java.awt.event.ActionEvent event) {
		//{{CONNECTION
		// Action from Exit Create and show as modal
		(new QuitDialog(this, true)).show();
		//}}
	}

    void Clear_Action(java.awt.event.ActionEvent event) {
        textArea1.setText("") ;
        textArea2.setText("") ;
    }


	void Start_Action(java.awt.event.ActionEvent event) {


	    textArea1.setText("") ;
	    textArea2.setText("") ;

	    int traceLevel=0 ;  // default is summary
	    if (viewDetails.getState() == true) {
	        traceLevel = 1 ;
	    }

        Object display = this ;    // or textArea2 ???
        facilitator = FacilitatorAgent.Instance() ; // singleton
        facilitator.reset() ; // clear it out
        facilitator.textArea = textArea1 ; // give view port for msgs
        facilitator.traceLevel = traceLevel ;
        facilitator.process() ;

	    // need to check Buyer and Seller menu items
	    // to see if we need to instantiate any agents
	    if (basicSeller.getState()==true) {
          basicSellerAgent = new SellerAgent() ;
          basicSellerAgent.setDisplay(display,traceLevel);
          basicSellerAgent.process() ;
	    }
	    if (intermedSeller.getState()==true) {
          intermedSellerAgent = new BetterSellerAgent() ;
          intermedSellerAgent.setDisplay(display,traceLevel);
          intermedSellerAgent.process() ;
	    }
	    if (advancedSeller.getState()==true) {
          advancedSellerAgent = new BestSellerAgent() ;
          advancedSellerAgent.setDisplay(display,traceLevel);
          advancedSellerAgent.process() ;
	    }
	    if (basicBuyer.getState()==true) {
          basicBuyerAgent = new BuyerAgent() ;
          basicBuyerAgent.setDisplay(display, traceLevel);
          basicBuyerAgent.process() ;
	    }
	    if (intermedBuyer.getState()==true) {
          intermedBuyerAgent = new BetterBuyerAgent() ;
          intermedBuyerAgent.setDisplay(display, traceLevel);
          intermedBuyerAgent.process() ;
	    }
	    if (advancedBuyer.getState()==true) {
	      advancedBuyerAgent = new BestBuyerAgent() ;
          advancedBuyerAgent.setDisplay(display, traceLevel);
          advancedBuyerAgent.process() ;
	    }
        textArea1.append("Starting Marketplace \n") ;

	}

	void Stop_Action(java.awt.event.ActionEvent event) {

	    if (facilitator != null) facilitator.stop() ;
	    if (basicSellerAgent != null) basicSellerAgent.stop() ;
	    if (intermedSellerAgent != null) intermedSellerAgent.stop() ;
	    if (advancedSellerAgent != null) advancedSellerAgent.stop() ;
	    if (basicBuyerAgent != null) basicBuyerAgent.stop() ;
	    if (intermedBuyerAgent != null) intermedBuyerAgent.stop() ;
	    if (advancedBuyerAgent != null) advancedBuyerAgent.stop() ;
	    textArea1.append("Ending Marketplace \n") ;
	}


	public Marketplace() {

		//{{INIT_CONTROLS
		setLayout(null);
		setVisible(false);
		setSize(537,449);
    setLocation(100,100);
		setBackground(new Color(16777215));
		openFileDialog1 = new java.awt.FileDialog(this);
		openFileDialog1.setMode(FileDialog.LOAD);
		openFileDialog1.setTitle("Open");
		//$$ openFileDialog1.move(48,384);
		textArea1 = new java.awt.TextArea();
		textArea1.setBounds(12,76,516,120);
		textArea1.setBackground(new Color(16777215));
		add(textArea1);
		textArea2 = new java.awt.TextArea();
		textArea2.setBounds(12,220,516,216);
		textArea2.setBackground(new Color(12632256));
		add(textArea2);
		label1 = new java.awt.Label("Facilitator");
		label1.setBounds(24,52,221,21);
		add(label1);
		label2 = new java.awt.Label("Marketplace");
		label2.setBounds(24,196,224,26);
		add(label2);
		setTitle("CIAgent Marketplace Application");
		setResizable(false);
		//}}

		//{{INIT_MENUS
		mainMenuBar = new java.awt.MenuBar();
		menu1 = new java.awt.Menu("File");
		miClear = new java.awt.MenuItem("Clear");
		menu1.add(miClear);
		menu1.addSeparator();
		miExit = new java.awt.MenuItem("Exit");
		menu1.add(miExit);
		mainMenuBar.add(menu1);
		menu5 = new java.awt.Menu("Actions");
		miStart = new java.awt.MenuItem("Start");
		menu5.add(miStart);
		miStop = new java.awt.MenuItem("Stop");
		menu5.add(miStop);
		mainMenuBar.add(menu5);
		menu4 = new java.awt.Menu("View");
		viewDetails = new java.awt.CheckboxMenuItem("Details");
		viewDetails.setState(false);
		menu4.add(viewDetails);
		viewSummary = new java.awt.CheckboxMenuItem("Summary");
		viewSummary.setState(true);
		menu4.add(viewSummary);
		mainMenuBar.add(menu4);
		menu6 = new java.awt.Menu("Buyers");
		basicBuyer = new java.awt.CheckboxMenuItem("Basic");
		basicBuyer.setState(true);
		menu6.add(basicBuyer);
		intermedBuyer = new java.awt.CheckboxMenuItem("Intermediate");
		intermedBuyer.setState(false);
		menu6.add(intermedBuyer);
		advancedBuyer = new java.awt.CheckboxMenuItem("Advanced");
		advancedBuyer.setState(false);
		menu6.add(advancedBuyer);
		mainMenuBar.add(menu6);
		menu7 = new java.awt.Menu("Seller");
		basicSeller = new java.awt.CheckboxMenuItem("Basic");
		basicSeller.setState(true);
		menu7.add(basicSeller);
		intermedSeller = new java.awt.CheckboxMenuItem("Intermediate");
		intermedSeller.setState(false);
		menu7.add(intermedSeller);
		advancedSeller = new java.awt.CheckboxMenuItem("Advanced");
		advancedSeller.setState(false);
		menu7.add(advancedSeller);
		mainMenuBar.add(menu7);
		menu3 = new java.awt.Menu("Help");
		mainMenuBar.setHelpMenu(menu3);
		miAbout = new java.awt.MenuItem("About");
		menu3.add(miAbout);
		mainMenuBar.add(menu3);
		setMenuBar(mainMenuBar);
		//$$ mainMenuBar.move(12,384);
		//}}
	
		//{{REGISTER_LISTENERS
		SymAction lSymAction = new SymAction();
		miAbout.addActionListener(lSymAction);
		miExit.addActionListener(lSymAction);
		miClear.addActionListener(lSymAction);
		miStart.addActionListener(lSymAction);
		miStop.addActionListener(lSymAction);
		this.addWindowListener(new windowEvents());
		//}}
	}

	public Marketplace(String title) {
	    this();
	    setTitle(title);
	}

 
 //-------------------------------------------------------------
 // This file has been migrated from the 1.0 to 1.1 event model.
 // This method is not used with the new 1.1 event model. You can 
 // move any code you need to keep, then remove this method.
 //-------------------------------------------------------------
 // 
 // 	public boolean handleEvent(Event event) {
 //     	if (event.id == Event.WINDOW_DESTROY) {
 //             hide();         // hide the Frame
 //             dispose();      // free the system resources
 //             System.exit(0); // close the application
 //             return true;
 //     	}
 // 		return super.handleEvent(event);
 // 	}
 //-------------------------------------------------------------
 //-------------------------------------------------------------
 // This file has been migrated from the 1.0 to 1.1 event model.
 // This method is not used with the new 1.1 event model. You can 
 // move any code you need to keep, then remove this method.
 //-------------------------------------------------------------
 // 
 // 	public boolean action(Event event, Object arg) {
 // 
 // 
 // 		if (event.target instanceof MenuItem) {
 // 			String label = ((MenuItem)event.target).getLabel();
 // 		 	if (label.equalsIgnoreCase("About")) {
 // 				About_Action(event);
 // 				return true;
 // 			} else
 // 				if (label.equalsIgnoreCase("Exit")) {
 // 					Exit_Action(event);
 // 					return true;
 //      		} else
 // 				if (label.equalsIgnoreCase("Clear")) {
 // 					Clear_Action(event);
 // 					return true;
 // 			 } else
 // 				if (label.equalsIgnoreCase("Details")) {
 // 					if (viewDetails.getState()== true) {
 // 					    viewSummary.setState(false) ;
 // 					} else {
 // 					    viewSummary.setState(true) ;
 // 					}
 // 					return true;
 // 			} else
 // 				if (label.equalsIgnoreCase("Summary")){
 // 					if (viewSummary.getState()== true) {
 // 					    viewDetails.setState(false) ;
 // 					} else {
 // 					    viewDetails.setState(true) ;
 // 					}
 // 					return true;
 // 		 	 } else
 // 	 			if (label.equalsIgnoreCase("Start")) {
 // 				     Start_Action(event);
 // 					return true;
 // 			 } else
 // 				if (label.equalsIgnoreCase("Stop")) {
 // 				     Stop_Action();
 // 					return true;
 // 			}
 // 
 // 		}
 // 		return super.action(event, arg);
 // 	}
 //-------------------------------------------------------------

	static public void main(String args[]) {
	    (new Marketplace()).show();
	}

	//{{DECLARE_CONTROLS
	java.awt.FileDialog openFileDialog1;
	java.awt.TextArea textArea1;
	java.awt.TextArea textArea2;
	java.awt.Label label1;
	java.awt.Label label2;
	//}}

	//{{DECLARE_MENUS
	java.awt.MenuBar mainMenuBar;
	java.awt.Menu menu1;
	java.awt.MenuItem miClear;
	java.awt.MenuItem miExit;
	java.awt.Menu menu5;
	java.awt.MenuItem miStart;
	java.awt.MenuItem miStop;
	java.awt.Menu menu4;
	java.awt.CheckboxMenuItem viewDetails;
	java.awt.CheckboxMenuItem viewSummary;
	java.awt.Menu menu6;
	java.awt.CheckboxMenuItem basicBuyer;
	java.awt.CheckboxMenuItem intermedBuyer;
	java.awt.CheckboxMenuItem advancedBuyer;
	java.awt.Menu menu7;
	java.awt.CheckboxMenuItem basicSeller;
	java.awt.CheckboxMenuItem intermedSeller;
	java.awt.CheckboxMenuItem advancedSeller;
	java.awt.Menu menu3;
	java.awt.MenuItem miAbout;
	//}}

    FacilitatorAgent facilitator ;
    BuyerAgent basicBuyerAgent ;
    BetterBuyerAgent intermedBuyerAgent ;
    BestBuyerAgent advancedBuyerAgent ;
    SellerAgent basicSellerAgent ;
    BetterSellerAgent intermedSellerAgent ;
    BestSellerAgent advancedSellerAgent ;



	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == miAbout)
				About_Action(event);
			else if (object == miExit)
				Exit_Action(event);
			else if (object == miClear)
				Clear_Action(event);
			else if (object == miStart)
				Start_Action(event);
			else if (object == miStop)
				Stop_Action(event);
		}
	}
	
	class windowEvents extends java.awt.event.WindowAdapter {
        public void windowClosing(java.awt.event.WindowEvent event) {
               System.exit(0) ;
        }     
    }     
}
