/* SellerAgent class

   Constructing Intelligent Agents with Java
   (C) Joseph P. Bigus and Jennifer Bigus 1997

*/
// package sunw.demo.ciagent ;
 import java.util.*;

 public class SellerAgent extends CIAgent {

  private long seed = 0 ;


  CIAgentMessage msg ;    // current message being processed
  BasicNegotiation current;
  long income = 0 ;   // total money earned
  Vector inventory = new Vector() ; // items we have for sale

  Hashtable negotiations = new Hashtable() ; // transaction history
                                              // key is the item id


  public SellerAgent() { name = "Seller"; }

  public SellerAgent(String Name) { super(Name); } ;


  public void process() {
    initialize() ;  // call any specific initialization code
    stopped = false ;
    // start a thread running and send an update message every interval secs
    trace( name + ": process() \n") ;
    msg = new CIAgentMessage("register:", name , null , null, null , null) ;
    CIAgentEvent e = new CIAgentEvent(this, msg) ;
    FacilitatorAgent.register(e) ;   // add this agent to the Facilitator???

    inventory.addElement(new BasicNegotiation("guitar",100)) ;  // item and strike price
    inventory.addElement(new BasicNegotiation("drums", 225));
    inventory.addElement(new BasicNegotiation("guitar",100)) ;  // item and strike price

    // advertise all items for sale
    Enumeration enum = inventory.elements() ;
    while (enum.hasMoreElements()) {
      current = (BasicNegotiation)enum.nextElement() ;
      msg = new CIAgentMessage("advertise", current.offer.item , null , null , current.offer.item , name) ;
      e = new CIAgentEvent(this, msg) ;
      notifyCIAgentEventListeners(e);   // signal interested observer
    }

    runnit = new Thread(this) ;
    runnit.start() ;
 }

 public void stop() {
    stopped = true;
    trace(name + " stopped\n") ;
    runnit.stop() ;
}

 // method of Runnable (Thread) interface
 public void run() {
    while(stopped == false){
        try {
           Thread.sleep((long)15 * 1000) ;  // come alive every 15 seconds
           if (traceLevel > 0) trace(name + ": active \n") ;
        }
        catch (InterruptedException e)
        {
          // interrupted
        }
        trace(name + " has " + inventory.size() + " items in inventory. \n");
        trace(name + " has earned " + income + ".\n ");

    }

 }

  public  void ciaEventFired(CIAgentEvent e) {
      if (traceLevel > 0 ) {
         trace(name + ":  CIAgentEvent received by " + name +
               " from " + e.getSource() + " with args " + e.getArgObject() + "\n") ;
      }
      Object arg = e.getArgObject() ;
      msg = (CIAgentMessage)arg ;
      if (traceLevel > 0) msg.display() ;
      processMessage(msg) ;
  }

  // generate a unique id for an item in inventory
  String genId() {
    seed++ ;
    return name+seed;
  }


  public void processMessage(CIAgentMessage msg) {

    if (msg.performative.equals("ask" )) {
      String item = msg.content ;
      // see if we have any items left
      if (itemInInventory(item)) {
        // start a new negotiation
        String id = genId() ;
        current = removeItemFromInventory(item) ;
        current.offer  = new Offer(msg.sender, item, id, 0);
        current.lastOffer = current.strikePrice + 100 ;
        negotiations.put(id, current) ;
        CIAgentMessage answer = new CIAgentMessage("make-offer", item + " " + id  + " " + current.lastOffer , msg.replyWith , msg.sender , item , name) ;
        CIAgentEvent e = new CIAgentEvent(this, answer) ;
        notifyCIAgentEventListeners(e);   // respond through Facilitator
      } else {
         // deny we have any to sell
        CIAgentMessage answer = new CIAgentMessage("deny", item , msg.replyWith , msg.sender , item , name) ;
        CIAgentEvent e = new CIAgentEvent(this, answer) ;
        trace(name + ": deny we have any " + item +
              " to sell to " + msg.sender + "\n");
        notifyCIAgentEventListeners(e);   // respond through Facilitator
      }
      return ;
    }

    Offer offer = new Offer(msg) ;  //
    trace(name + ": Offer from " + offer.sender + ": content is " +
          offer.item + " " + offer.id + " " + offer.price + "\n") ;

    current = (BasicNegotiation)negotiations.get(offer.id) ;
    if (current == null) {
       // we must have sold this item --- reject offer
       CIAgentMessage answer = new CIAgentMessage("reject-offer", msg.content , msg.replyWith , msg.sender , offer.item , name) ;
       CIAgentEvent e = new CIAgentEvent(this, answer) ;
       notifyCIAgentEventListeners(e);   // respond through Facilitator
       return;
    }
    current.newOffer(offer) ;


    // buyer has made an offer, we can accept, make a counter offer, or reject
    if (msg.performative.equals("make-offer" )) {
         negotiate(offer, msg) ;
    }

    // buyer has received and acknowledged our accept-offer
    // remove item from active list, add $ to income
    // unadvertise this item with the Facilitator
    if (msg.performative.equals("tell" )) {
        negotiations.remove(offer.id) ; // remove from negotiations list
        income += offer.price ;
        CIAgentMessage msg2 = new CIAgentMessage("unadvertise", current.offer.item , null , null , current.offer.item , name) ;
        CIAgentEvent e = new CIAgentEvent(this, msg2) ;
        notifyCIAgentEventListeners(e);   // signal interested observer
      }

  }

  void negotiate(Offer offer, CIAgentMessage msg) {
     if (offer.price > current.strikePrice) {
            // accept
            CIAgentMessage answer = new CIAgentMessage("accept-offer", msg.content, msg.replyWith , msg.sender , offer.item , name) ;
            CIAgentEvent e = new CIAgentEvent(this, answer) ;
            notifyCIAgentEventListeners(e);   // respond through Facilitator
            return ;
        }
        if (offer.price < current.strikePrice) {
            // reject
            rejectOffer(offer) ;
            return;
        }
      CIAgentMessage answer = new CIAgentMessage("make-offer", offer.item + " " + offer.id + " " + current.lastOffer , msg.replyWith , msg.sender , offer.item , name) ;
      CIAgentEvent e = new CIAgentEvent(this, answer) ;
      notifyCIAgentEventListeners(e);   // respond through Facilitator

  }


  // break off a negotiation (for whatever reason)
  // return item to inventory, remove negotiations from active list
  void rejectOffer(Offer offer) {
      CIAgentMessage answer = new CIAgentMessage("reject-offer", msg.content , msg.replyWith , msg.sender , offer.item , name) ;
      CIAgentEvent e = new CIAgentEvent(this, answer) ;
      notifyCIAgentEventListeners(e);   // respond through Facilitator
      negotiations.remove(offer.id) ;  // no further negotiations
      current.offer.id = null ; // offer id has expired
      inventory.addElement(current) ; // place item back in inventory
  }



  // returns true if the item is in stock
  boolean itemInInventory(String item) {
    Enumeration enum = inventory.elements() ;
    boolean haveItem = false ;
    while(enum.hasMoreElements()) {
      BasicNegotiation stockItem = (BasicNegotiation)enum.nextElement() ;
      if (stockItem.getItem().equals(item)) {
        haveItem = true ;
        break ;
      } else {
        continue ;
      }
    }
    return haveItem ;
  }

  // returns the specified item  from inventory
  // if we have no items, return null
  BasicNegotiation removeItemFromInventory(String item) {
    Enumeration enum = inventory.elements() ;
    BasicNegotiation stockItem = null ;
    while(enum.hasMoreElements()) {
      stockItem = (BasicNegotiation)enum.nextElement() ;
      if (stockItem.getItem().equals(item)) {
        inventory.removeElement(stockItem) ;
        break ;
      } else {
        stockItem = null ; // this isn't it
        continue ;
      }
    }
    return stockItem ;
  }

};

