/* NewsFilter application class

   Constructing Intelligent Agents with Java
   (C) Joseph P. Bigus and Jennifer Bigus 1997
   
    Modified: 8/14/98 jpb convert to 1.1 Event model
 */

import java.awt.*;
import java.io.* ;
import java.net.* ;
import java.util.* ;

public class NewsFilter extends Frame {
	void AddNewsGroup_Action(java.awt.event.ActionEvent event) {

		//{{CONNECTION
		// Create with title, show as modal...
		NewsGroupDialog dlg = new NewsGroupDialog(this, "Enter name of New Group to add to list", true);
		dlg.show();
		//}}
		String newsGroup = dlg.textField1.getText() ;
		if (newsGroup != null) {
	    	list1.addItem(newsGroup) ;
		}
	}

	void OpenNewsHost_Action(java.awt.event.ActionEvent event) {


		//{{CONNECTION
		// Create with title, show as modal...
		NewsHostDialog dlg = new NewsHostDialog(this, "Enter address or name of News Server", true) ;
	 	//}}
	 	dlg.setNewsHost(newsHost) ;  // set default to current value
	 	dlg.show() ;
		String temp = dlg.newsHost ;
		if (temp != null) {
	    	newsHost = temp ;
		}
		if (newsHost != null) {
          textArea1.append("\nConnecting to " + newsHost + " \n" ) ;
		  connectToNewsHost() ;
        }
	}

    void Keywords_Action(java.awt.event.ActionEvent event) {

		//{{CONNECTION
		// Create with title, show as modal...
		KeywordDialog dlg = new KeywordDialog(this, "Specify Filter Keywords", true);
		//}}
		dlg.setKeywords(keywords) ;  // initialize the list
		dlg.show() ;
		keywords = dlg.getKeywords();
		if (keywords != null) {
		  textArea1.append(keywords[0]) ;
		}
 	}

    // user selected a news group -- download it
    void NewsGroup_Action(java.awt.event.ActionEvent event) {
       list2.removeAll() ;
       textArea1.setText("") ;
       articles = new Vector() ;
       String newsGroup = list1.getSelectedItem() ;
       readNewsGroup(newsGroup) ;

    }

    // user selected a news item -- put body in textArea
    void NewsItem_Action(java.awt.event.ActionEvent event) {
       int index = list2.getSelectedIndex();
       if (index != -1) {
         currentArt = (NewsArticle)articles.elementAt(index) ;
         textArea1.setText(currentArt.body) ;  // clear the display area
         filterAgent.score(currentArt, keywords, filterType);
       } else {
         currentArt = null ;
       }
    }

    void Feedback_Action(double feedback) {
        if (currentArt != null) {
            currentArt.feedback = feedback ;
        }
 	}

    void FilterArticles_Action(java.awt.event.ItemEvent event) {
        filterArticles() ;
 	}

    void FilterType_Action(int fType) {
        switch (fType) {
         case 0:   // keyword
            keywordFilter.setState(true) ;
            clusterFilter.setState(false) ;
            feedbackFilter.setState(false) ;

         break ;
         case 1:   // cluster
            keywordFilter.setState(false) ;
            clusterFilter.setState(true) ;
            feedbackFilter.setState(false) ;
         break ;
         case 2:   // feedback
            keywordFilter.setState(false) ;
            clusterFilter.setState(false) ;
            feedbackFilter.setState(true) ;
         break ;
        }
        filterType = fType ;
        if (filterArt.getState() == true) filterArticles() ;
 	}

    void AddArticle_Action(java.awt.event.ActionEvent event) {
         // open the profile file and append the
         // score data for this article
         filterAgent.addArticleToProfile(currentArt) ;
 	}

    void AddAllArticles_Action(java.awt.event.ActionEvent event) {
        // open the profilefile and append the
        // score data for all articles
         filterAgent.addAllArticlesToProfile(articles) ;
 	}

    void ScoreArticle_Action(java.awt.event.ActionEvent event) {
         filterAgent.score(currentArt, keywords, filterType) ;
 	}

    void ScoreAllArticles_Action(java.awt.event.ActionEvent event) {
         filterAgent.score(articles, keywords, filterType) ; // do it unconditionally
         scored = true ;
 	}

    void CreateProfile_Action(java.awt.event.ActionEvent event) {
          // empty the newsfilter.prf file
         File prf = new File("newsfilter.prf") ;
         prf.delete() ;
         File dat = new File("newsfilter.dat") ;
         dat.delete() ;
         // create a newsfilter.prf file using current keywords
         filterAgent.writeProfileDataDefinition(keywords) ;
 	}

    void BuildKeywordFilter_Action(java.awt.event.ActionEvent event) {
         filterAgent.score(articles, keywords, filterType);
 	}


    void BuildClusterFilter_Action(java.awt.event.ActionEvent event) {
         //   filterAgent.cluster() ;
         clusterFilter.setEnabled(false);
         filterAgent.buildClusterNet();
 	}

    void BuildFeedbackFilter_Action(java.awt.event.ActionEvent event) {
         //   filterAgent.train() ; // use all profile data
         feedbackFilter.setEnabled(false);
         filterAgent.buildScoreNet();
 	}

    void clusterNetBuilt() {
       clusterFilter.setEnabled(true) ;
    }
    void scoreNetBuilt() {
       feedbackFilter.setEnabled(true) ;
    }

	void Open_Action(Event event) {
		//{{CONNECTION
		// Action from Open... Show the OpenFileDialog
		openFileDialog1.show();
		//}}
	}
    void LoadArticle_Action(Event event) {
		//{{CONNECTION
		// Action from Open... Show the OpenFileDialog
		openFileDialog1.show();
		//}}
		String fileName = openFileDialog1.getFile();
		if (fileName != null) {
		  NewsArticle art = new NewsArticle(fileName) ;
          art.readArticle(fileName) ;
          list2.addItem(fileName) ;
          textArea1.setText(art.body) ;
          articles.addElement(art) ;
        }
	}
    void SaveArticle_Action(Event event) {
		//{{CONNECTION
		// Action from Save... Show the SaveFileDialog
		saveFileDialog1.show();
		//}}
		String fileName = saveFileDialog1.getFile();
		if (fileName != null) {
		 int index = list2.getSelectedIndex();
		 if (index != -1) {
          NewsArticle art = (NewsArticle)articles.elementAt(index) ;
		  art.writeArticle(fileName) ;
		 }
		}
	}



	void About_Action(java.awt.event.ActionEvent event) {
		//{{CONNECTION
		// Action from About Create and show as modal
		(new AboutNewsFilterDialog(this, true)).show();
		//}}
	}

	void Exit_Action(java.awt.event.ActionEvent event) {
        closeNewsHost() ;
		//{{CONNECTION
		// Action from Exit Create and show as modal
		(new QuitDialog(this, true)).show();
		//}}
	}

	public NewsFilter() {

		//{{INIT_CONTROLS
		setLayout(null);
		setVisible(false);
		setSize(587,404);
    setLocation(100,100);
		setBackground(new Color(16777215));
		openFileDialog1 = new java.awt.FileDialog(this);
		openFileDialog1.setMode(FileDialog.LOAD);
		openFileDialog1.setTitle("Open");
		//$$ openFileDialog1.move(36,336);
		saveFileDialog1 = new java.awt.FileDialog(this);
		saveFileDialog1.setMode(FileDialog.SAVE);
		saveFileDialog1.setTitle("Save");
		//$$ saveFileDialog1.move(0,0);
		list1 = new java.awt.List(4);
		add(list1);
		list1.setBounds(12,76,276,88);
		list1.setBackground(new Color(16777215));
		list2 = new java.awt.List(4);
		add(list2);
		list2.setBounds(300,76,276,88);
		list2.setBackground(new Color(16777215));
		textArea1 = new java.awt.TextArea();
		textArea1.setBounds(12,184,564,208);
		add(textArea1);
		label1 = new java.awt.Label("Newsgroup");
		label1.setBounds(12,52,93,24);
		add(label1);
		label2 = new java.awt.Label("News Item");
		label2.setBounds(300,52,86,24);
		add(label2);
		setTitle("CIAgent News Filter Application");
		setResizable(false);
		//}}

		//{{INIT_MENUS
		mainMenuBar = new java.awt.MenuBar();
		menu1 = new java.awt.Menu("File");
		menu1.add("Reset");
		miOpenNewsHost = new java.awt.MenuItem("Open News Host...");
		menu1.add(miOpenNewsHost);
		micloseNewsHost = new java.awt.MenuItem("Close News Host");
		menu1.add(micloseNewsHost);
		miAddNewsGroup = new java.awt.MenuItem("Add News Group...");
		menu1.add(miAddNewsGroup);
		menu1.addSeparator();
		menu1.add("Save Article...");
		menu1.add("Load Article...");
		menu1.addSeparator();
		miExit = new java.awt.MenuItem("Exit");
		menu1.add(miExit);
		mainMenuBar.add(menu1);
		menu6 = new java.awt.Menu("Profile");
		miKeywords = new java.awt.MenuItem("Keywords...");
		menu6.add(miKeywords);
		menu6.addSeparator();
		miAddArticle = new java.awt.MenuItem("Add article");
		menu6.add(miAddArticle);
		miAddAllArticles = new java.awt.MenuItem("Add all articles");
		menu6.add(miAddAllArticles);
		menu6.addSeparator();
		miScoreArticle = new java.awt.MenuItem("Score article");
		menu6.add(miScoreArticle);
		miScoreAllArticles = new java.awt.MenuItem("Score all articles");
		menu6.add(miScoreAllArticles);
		menu6.addSeparator();
		miCreateProfile = new java.awt.MenuItem("Create profile");
		menu6.add(miCreateProfile);
		mainMenuBar.add(menu6);
		menu2 = new java.awt.Menu("Edit");
		menu2.add("Cut");
		menu2.add("Paste");
		mainMenuBar.add(menu2);
		menu4 = new java.awt.Menu("Filter");
		filterArt = new java.awt.CheckboxMenuItem("Filter articles");
		filterArt.setState(false);
		menu4.add(filterArt);
		menu4.addSeparator();
		keywordFilter = new java.awt.CheckboxMenuItem("using Keywords");
		keywordFilter.setState(true);
		menu4.add(keywordFilter);
		clusterFilter = new java.awt.CheckboxMenuItem("using Clusters");
		clusterFilter.setState(false);
		menu4.add(clusterFilter);
		feedbackFilter = new java.awt.CheckboxMenuItem("using Feedback");
		feedbackFilter.setState(false);
		menu4.add(feedbackFilter);
		menu4.addSeparator();
		miBuildKeywordFilter = new java.awt.MenuItem("Build Keyword filter");
		menu4.add(miBuildKeywordFilter);
		miBuildClusterFilter = new java.awt.MenuItem("Build Cluster filter");
		menu4.add(miBuildClusterFilter);
		miBuildFeedbackFilter = new java.awt.MenuItem("Build Feedback filter");
		menu4.add(miBuildFeedbackFilter);
		mainMenuBar.add(menu4);
		menu5 = new java.awt.Menu("Feedback");
		miFeedbackUseless = new java.awt.MenuItem("Useless");
		menu5.add(miFeedbackUseless);
		miFeedbackNotVery = new java.awt.MenuItem("Not very useful");
		menu5.add(miFeedbackNotVery);
		miFeedbackNeutral = new java.awt.MenuItem("Neutral");
		menu5.add(miFeedbackNeutral);
		miFeedbackMildly = new java.awt.MenuItem("Mildly interesting");
		menu5.add(miFeedbackMildly);
		miFeedbackInteresting = new java.awt.MenuItem("Interesting");
		menu5.add(miFeedbackInteresting);
		mainMenuBar.add(menu5);
		menu3 = new java.awt.Menu("Help");
		mainMenuBar.setHelpMenu(menu3);
		miAbout = new java.awt.MenuItem("About");
		menu3.add(miAbout);
		mainMenuBar.add(menu3);
		setMenuBar(mainMenuBar);
		//$$ mainMenuBar.move(0,336);
		//}}

		// set default values here
		newsHost = "news-s01.ny.us.ibm.net" ;
		filterAgent.newsFilter = this ;
        filterAgent.textArea = textArea1 ;
        list1.addItem("comp.ai.fuzzy") ;
        list1.addItem("comp.ai.neural-nets") ;
        list1.addItem("comp.lang.java.api") ;
        list1.addItem("comp.lang.java.misc") ;
        list1.addItem("comp.lang.java.tech") ;
        keywords = new String[10] ; // default list
        keywords[0] = "java" ;
        keywords[1] = "agents" ;
        keywords[2] = "fuzzy" ;
        keywords[3] = "intelligent" ;
        keywords[4] = "neural" ;
        keywords[5] = "network" ;
        keywords[6] = "genetic" ;
        keywords[7] = "Symantec" ;
        keywords[8] = "Cafe" ;
        keywords[9] = "Beans" ;
        filterAgent.process() ; // start the Filter agent thread
	
		//{{REGISTER_LISTENERS
		SymAction lSymAction = new SymAction();
		miAddNewsGroup.addActionListener(lSymAction);
		miOpenNewsHost.addActionListener(lSymAction);
		micloseNewsHost.addActionListener(lSymAction);
		miAddArticle.addActionListener(lSymAction);
		miAddAllArticles.addActionListener(lSymAction);
		miScoreArticle.addActionListener(lSymAction);
		miScoreAllArticles.addActionListener(lSymAction);
		miAbout.addActionListener(lSymAction);
		miExit.addActionListener(lSymAction);
		miKeywords.addActionListener(lSymAction);
		miCreateProfile.addActionListener(lSymAction);
		miFeedbackUseless.addActionListener(lSymAction);
		miFeedbackNotVery.addActionListener(lSymAction);
		miFeedbackNeutral.addActionListener(lSymAction);
		miFeedbackMildly.addActionListener(lSymAction);
		miFeedbackInteresting.addActionListener(lSymAction);
		miBuildKeywordFilter.addActionListener(lSymAction);
		miBuildClusterFilter.addActionListener(lSymAction);
		miBuildFeedbackFilter.addActionListener(lSymAction);
    SymItem lSymItem = new SymItem();
		filterArt.addItemListener(lSymItem);
		keywordFilter.addItemListener(lSymItem);
		clusterFilter.addItemListener(lSymItem);
	  feedbackFilter.addItemListener(lSymItem);
    list1.addActionListener(lSymAction);
    list2.addActionListener(lSymAction);
	  addWindowListener(new windowEvents()) ;
		//}}
	}

	public NewsFilter(String title) {
	    this();
	    setTitle(title);
	}


 //-------------------------------------------------------------
 // This file has been migrated from the 1.0 to 1.1 event model.
 // This method is not used with the new 1.1 event model. You can 
 // move any code you need to keep, then remove this method.
 //-------------------------------------------------------------
 // 
 // 	public boolean handleEvent(Event event) {
 //     	if (event.id == Event.WINDOW_DESTROY) {
 //             hide();         // hide the Frame
 //             dispose();      // free the system resources
 //             System.exit(0); // close the application
 //             return true;
 //     	}
 // 		return super.handleEvent(event);
 // 	}
 //-------------------------------------------------------------
 //-------------------------------------------------------------
 // This file has been migrated from the 1.0 to 1.1 event model.
 // This method is not used with the new 1.1 event model. You can 
 // move any code you need to keep, then remove this method.
 //-------------------------------------------------------------
 // 
 // 	public boolean action(Event event, Object arg) {
 // 
 // 		if (event.target instanceof MenuItem) {
 // 		    String label = ((MenuItem)event.target).getLabel();
 // 
 // 			if (label.equalsIgnoreCase("Add News Group...")) {
 // 				AddNewsGroup_Action(event);
 // 				return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Open News Host...")) {
 // 				OpenNewsHost_Action(event);
 // 				return true;
 // 			} else
 // 				if (label.equalsIgnoreCase("Close News Host")) {
 // 				closeNewsHost();
 // 				return true;
 // 			} else
 // 		 	    if (label.equalsIgnoreCase("Open...")) {
 // 				Open_Action(event);
 // 				return true;
 // 			} else
 // 			    if (label.equalsIgnoreCase("Save article...")) {
 // 				SaveArticle_Action(event);
 // 				return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Load article...")) {
 // 				LoadArticle_Action(event);
 // 				return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Add article")) {
 // 				AddArticle_Action(event);
 // 				return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Add all articles")) {
 // 				AddAllArticles_Action(event);
 // 				return true;
 // 			} else
 //             if (label.equalsIgnoreCase("Score article")) {
 // 				ScoreArticle_Action(event);
 // 				return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Score all articles")) {
 // 				ScoreAllArticles_Action(event);
 // 				return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("About")) {
 // 				About_Action(event);
 // 				return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Exit")) {
 // 					Exit_Action(event);
 // 					return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Keywords...")) {
 // 					Keywords_Action(event);
 // 					return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Create profile")) {
 // 					CreateProfile_Action(event);
 // 					return true;
 // 			} else
 // 		 	if (label.equalsIgnoreCase("Useless")) {
 // 					Feedback_Action(0);
 // 					return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Not very useful")) {
 // 					Feedback_Action(0.25);
 // 					return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Neutral")) {
 // 					Feedback_Action(0.5);
 // 					return true;
 // 			} else
 // 			if (label.equalsIgnoreCase("Mildly interesting")) {
 // 			        Feedback_Action(0.75);
 // 					return true;
 // 			} else
 // 
 // 			if (label.equalsIgnoreCase("Interesting")) {
 // 					Feedback_Action(1.0);
 // 					return true;
 // 			} else
 // 
 //             if (label.equalsIgnoreCase("Filter articles")) {
 // 					FilterArticles_Action(event);
 // 					return true;
 // 			} else
 // 			  if (label.equalsIgnoreCase("using Keywords")) {
 // 					FilterType_Action(0);
 // 					return true;
 // 			} else  if (label.equalsIgnoreCase("using Clusters")) {
 // 					FilterType_Action(1);
 // 					return true;
 // 			} else  if (label.equalsIgnoreCase("using Feedback")) {
 // 					FilterType_Action(2);
 // 					return true;
 // 			} else
 //             if (label.equalsIgnoreCase("Build Keyword filter")) {
 // 					BuildKeywordFilter_Action(event);
 // 					return true;
 // 			} else
 // 			 if (label.equalsIgnoreCase("Build Cluster filter")) {
 // 					BuildClusterFilter_Action(event);
 // 					return true;
 // 			} else
 //             if (label.equalsIgnoreCase("Build Feedback filter")) {
 // 					BuildFeedbackFilter_Action(event);
 // 					return true;
 // 			}
 // 
 // 		}
 // 		if (event.target instanceof List) {
 // 		    if (event.target == list1) {
 // 		        // load selected news group
 // 		        NewsGroup_Action(event) ;
 // 		        return true;
 // 		    }
 // 		    if (event.target == list2) {
 // 		        // load selected article
 // 		        NewsItem_Action(event);
 // 		        return true ;
 // 		    }
 // 		}
 // 		return super.action(event, arg);
 // 	}
 //-------------------------------------------------------------

	static public void main(String args[]) {
	    (new NewsFilter()).show();
	}

	//{{DECLARE_CONTROLS
	java.awt.FileDialog openFileDialog1;
	java.awt.FileDialog saveFileDialog1;
	java.awt.List list1;
	java.awt.List list2;
	java.awt.TextArea textArea1;
	java.awt.Label label1;
	java.awt.Label label2;
	//}}

	//{{DECLARE_MENUS
	java.awt.MenuBar mainMenuBar;
	java.awt.Menu menu1;
	java.awt.MenuItem miOpenNewsHost;
	java.awt.MenuItem micloseNewsHost;
	java.awt.MenuItem miAddNewsGroup;
	java.awt.MenuItem miExit;
	java.awt.Menu menu6;
	java.awt.MenuItem miKeywords;
	java.awt.MenuItem miAddArticle;
	java.awt.MenuItem miAddAllArticles;
	java.awt.MenuItem miScoreArticle;
	java.awt.MenuItem miScoreAllArticles;
	java.awt.MenuItem miCreateProfile;
	java.awt.Menu menu2;
	java.awt.Menu menu4;
	java.awt.CheckboxMenuItem filterArt;
	java.awt.CheckboxMenuItem keywordFilter;
	java.awt.CheckboxMenuItem clusterFilter;
	java.awt.CheckboxMenuItem feedbackFilter;
	java.awt.MenuItem miBuildKeywordFilter;
	java.awt.MenuItem miBuildClusterFilter;
	java.awt.MenuItem miBuildFeedbackFilter;
	java.awt.Menu menu5;
	java.awt.MenuItem miFeedbackUseless;
	java.awt.MenuItem miFeedbackNotVery;
	java.awt.MenuItem miFeedbackNeutral;
	java.awt.MenuItem miFeedbackMildly;
	java.awt.MenuItem miFeedbackInteresting;
	java.awt.Menu menu3;
	java.awt.MenuItem miAbout;
	//}}

	String newsHost ;         // name of the news server
	Socket news ;            // socket to news server
	PrintWriter newsOut ;    // stream to write to server
	BufferedReader newsIn ; // stream to read from server
  Vector articles = new Vector() ; // list of articles in specified news group
  Vector originalArticleList ;     //unfiltered list of articles
  String[] keywords = new String[0] ;  // user specified keywords
  FilterAgent filterAgent  = new FilterAgent() ;
  NewsArticle currentArt ;       // currently selected article
  boolean scored = false ;       // true if articles were scored
  int  filterType = 0 ;  // keyword filter as default

public void connectToNewsHost() {

   try {
        news = new Socket(newsHost, 119) ;
        newsIn = new BufferedReader(new InputStreamReader(news.getInputStream())) ;
        newsOut = new PrintWriter(news.getOutputStream(), true) ;
        String reply = newsIn.readLine();
        textArea1.append( reply + "\n") ;
   }
   catch (Exception e) {
        textArea1.append("Exception:" + e) ;
   }

}

void closeNewsHost() {

   try {
         String cmd = "QUIT \n" ;
         newsOut.println(cmd) ;
         textArea1.append( cmd + " \n" ) ;
         newsIn.close() ;
   }
   catch (Exception e) {
         textArea1.append("Exception:" + e) ;
   }

}

public void readNewsGroup(String newsGroup) {

   int maxArticles = 20 ;
   boolean exit = false ;

   scored = false ; // articles were not scored yet

   try {
         String cmd = "GROUP " + newsGroup + " \n" ;
         newsOut.println(cmd) ;
         String reply = newsIn.readLine();
         textArea1.append( cmd + " \n" + reply + "\n") ;

         StringTokenizer st = new StringTokenizer(reply) ;
         String s1 = st.nextToken() ;  // response code
         String s2 = st.nextToken() ;  // number of appends
         String s3 = st.nextToken() ;  // first id
         String s4 = st.nextToken() ;  // last id
         String s5 = st.nextToken() ;  // newsgroup

         cmd = "STAT " + s3  + "\n" ;
         newsOut.println(cmd) ;
         reply = newsIn.readLine();
         textArea1.append( cmd + " \n" + reply + "\n") ;

         String retCode ;
         do {
            textArea1.setText("");
            cmd = "HEAD \n" ;
            newsOut.println(cmd) ;
            reply = newsIn.readLine();
            textArea1.append( cmd + " \n" + reply + "\n") ;
            StringTokenizer tok = new StringTokenizer(reply, " ") ;
            retCode = tok.nextToken() ;
            String id = tok.nextToken() ;
            String msgId = tok.nextToken() ;

            if (!retCode.equals("221")) continue ;

            // now read all header records for this article and parse
            NewsArticle art = parseHeader(id) ;
            list2.addItem( art.subject ) ;  // display subject only
            articles.addElement(art) ;  // add to Vector

            cmd = "BODY \n" ;
            newsOut.println(cmd) ;
            reply = newsIn.readLine();
            StringTokenizer stok = new StringTokenizer(reply) ;
            retCode = stok.nextToken() ;  // response code
            if (!retCode.equals("222")) {  // error?
              articles.removeElement(art) ; // bad article
              continue ;
            }

            textArea1.append( cmd + " \n" + reply + "\n") ;
            do {
              reply = newsIn.readLine();
              textArea1.append( reply + "\n") ;
            } while(!reply.equals(".")) ;
            art.body = textArea1.getText() ;

            cmd = "\n NEXT \n" ;
            newsOut.println(cmd) ;
            reply = newsIn.readLine();
            textArea1.append( cmd + " \n" + reply + "\n") ;
            StringTokenizer st2 = new StringTokenizer(reply) ;
            retCode = st2.nextToken() ;  // response code

         } while (retCode.equals("223") && (articles.size() < maxArticles)) ;  // 421 = no  next article
         list2.select(0) ;
         currentArt = (NewsArticle)articles.elementAt(0) ;
         textArea1.setText(currentArt.body) ;
     }
     catch (Exception e) {
           textArea1.append("Exception:" + e) ;
     }
      originalArticleList = (Vector)articles.clone() ; // save copy
      if (filterArt.getState() == true) filterArticles();

	}


    // the user wants us to filter articles
    // clear the news Item list and redisplay
    // using the article scores to determine the order
    public void filterArticles() {
        if (filterArt.getState() == true) {
          if (!scored) filterAgent.score(articles, keywords, filterType) ;
          Vector sortedByScore = insertionSort(articles) ;
          articles = sortedByScore;
        } else {
          articles = originalArticleList ;
        }

        list2.removeAll() ; // remove all articles from list
        Enumeration enum = articles.elements() ;
        while (enum.hasMoreElements()) {
            NewsArticle art = (NewsArticle)enum.nextElement();
            list2.addItem(art.subject) ;
        }
        list2.select(0) ;
        currentArt = (NewsArticle)articles.elementAt(0) ;
    }

	public void readSingleNewsItem(String id) {

          try {

            String cmd = "STAT " + id  + "\n" ;
            newsOut.println(cmd) ;
            String reply = newsIn.readLine();
            textArea1.append( cmd + " \n" + reply + "\n") ;

            String s10 ;

            cmd = "BODY \n" ;
            newsOut.println(cmd) ;
            reply = newsIn.readLine();
            textArea1.append( cmd + " \n" + reply + "\n") ;
            do {
             reply = newsIn.readLine();
             textArea1.append( reply + "\n") ;
            } while(!reply.equals(".")) ;

       }
       catch (Exception e) {
          textArea1.append("Exception:" + e) ;
       }
	}

  // parse the article header
  // we must read each head line one at a time
  // we only want the subject line for now
  NewsArticle parseHeader(String id) {
     NewsArticle art = new NewsArticle(id) ;
     // parse to find subject line
     String subject ;
     String line ;
     try {
       do {
          line = newsIn.readLine();
          if (line.charAt(0) == 'S') {
           StringTokenizer tagTok = new StringTokenizer(line," ") ;
           if (tagTok.nextToken().equals("Subject:")) {
             art.subject = line ;
           }
          }
       } while(!line.equals(".")) ;
     }
     catch (Exception e) {
         textArea1.append("Exception:" + e) ;
     }
     return art;
  }

  // sort articles by decreasing order of score
  Vector insertionSort(Vector articles) {

     int i, j ;
     int size = articles.size() ;

     NewsArticle sortedList[] = new NewsArticle[articles.size()] ;
     articles.copyInto( sortedList ) ;
     NewsArticle temp ;

     for (i=1 ; i < size ; i++) {
       temp = sortedList[i] ;
       textArea1.append(temp.score + " ") ;
       j = i ;
       while ((j > 0) && (sortedList[j-1].score < temp.score)) {
          sortedList[j] = sortedList[j-1];
          j = j - 1 ;
       }
       sortedList[j] = temp;
     }

       Vector outList = new Vector() ;
       textArea1.append("\n Sorted list = ") ;
       for (i=0 ; i < size ; i++) {
         temp = sortedList[i] ;
         textArea1.append(temp.score + " ") ;
         outList.addElement(temp) ;
       }
     return outList ;
  }


	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == miAddNewsGroup)
				AddNewsGroup_Action(event);
			else if (object == miOpenNewsHost)
				OpenNewsHost_Action(event);
			else if (object == micloseNewsHost)
				closeNewsHost();
			else if (object == miAddArticle)
				AddArticle_Action(event);
			else if (object == miAddAllArticles)
				AddAllArticles_Action(event);
			else if (object == miScoreArticle)
				ScoreArticle_Action(event);
			else if (object == miScoreAllArticles)
				ScoreAllArticles_Action(event);
			else if (object == miAbout)
				About_Action(event);
			else if (object == miExit)
				Exit_Action(event);
			else if (object == miKeywords)
				Keywords_Action(event);
			else if (object == miCreateProfile)
				CreateProfile_Action(event);
			else if (object == miFeedbackUseless)
				Feedback_Action(0.0);
			else if (object == miFeedbackNotVery)
				Feedback_Action(0.25);
			else if (object == miFeedbackNeutral)
				Feedback_Action(0.50);
			else if (object == miFeedbackMildly)
				Feedback_Action(0.75);
			else if (object == miFeedbackInteresting)
				Feedback_Action(1.0);
			else if (object == miBuildKeywordFilter)
				BuildKeywordFilter_Action(event);
			else if (object == miBuildClusterFilter)
				BuildClusterFilter_Action(event);
			else if (object == miBuildFeedbackFilter)
				BuildFeedbackFilter_Action(event);
      else if (object == list1)
  		  // load selected news group
  		  NewsGroup_Action(event) ;
  		else if (object == list2)
  		  // load selected article
  		  NewsItem_Action(event);
  	}
	}

	 	class SymItem implements java.awt.event.ItemListener
	{
		public void itemStateChanged(java.awt.event.ItemEvent event)
		{
			Object object = event.getSource();
			if (object == keywordFilter)
				FilterType_Action(0);
			else if (object == clusterFilter)
				FilterType_Action(1);
			else if (object == feedbackFilter)
				FilterType_Action(2);
 			else if (object == filterArt)
				FilterArticles_Action(event);
  	}
	}

	class windowEvents extends java.awt.event.WindowAdapter {
        public void windowClosing(java.awt.event.WindowEvent event) {
               System.exit(0) ;
        }
    }         
}


